@@ -2,33 +2,35 @@
 #@+leo-ver=5-thin
 #@+node:ekr.20150514040239.1: * @file ../commands/spellCommands.py
 #@@first
-'''Leo's spell-checking commands.'''
-
+"""Leo's spell-checking commands."""
 #@+<< imports >>
 #@+node:ekr.20150514050530.1: ** << imports >> (spellCommands.py)
 import re
-import leo.core.leoGlobals as g
-from leo.commands.baseCommands import BaseEditCommandsClass as BaseEditCommandsClass
 try:
+    # pylint: disable=import-error
+        # We can't assume the user has this.
+    # pip install pyenchant
     import enchant
-except Exception: # May throw WinError(!)
+except Exception:  # May throw WinError(!)
     enchant = None
+from leo.commands.baseCommands import BaseEditCommandsClass
+from leo.core import leoGlobals as g
 #@-<< imports >>
 
 def cmd(name):
-    '''Command decorator for the SpellCommandsClass class.'''
+    """Command decorator for the SpellCommandsClass class."""
     return g.new_cmd_decorator(name, ['c', 'spellCommands',])
+
 #@+others
 #@+node:ekr.20180207071908.1: ** class BaseSpellWrapper
-class BaseSpellWrapper(object):
-    '''Code common to EnchantWrapper and DefaultWrapper'''
+class BaseSpellWrapper:
+    """Code common to EnchantWrapper and DefaultWrapper"""
     # pylint: disable=no-member
     # Subclasses set self.c and self.d
-    
     #@+others
     #@+node:ekr.20180207071114.3: *3* spell.add
     def add(self, word):
-        '''Add a word to the user dictionary.'''
+        """Add a word to the user dictionary."""
         self.d.add(word)
     #@+node:ekr.20150514063305.513: *3* spell.clean_dict
     def clean_dict(self, fn):
@@ -38,48 +40,56 @@ class BaseSpellWrapper(object):
             f.close()
             # Blanks lines cause troubles.
             s2 = s.replace(b'\r', b'').replace(b'\n\n', b'\n')
-            if s2.startswith(b'\n'): s2 = s2[1:]
+            if s2.startswith(b'\n'):
+                s2 = s2[1:]
             if s != s2:
                 g.es_print('cleaning', fn)
-                f = open(fn, mode='wb')
+                f = open(fn, mode='wb')  # type:ignore
                 f.write(s2)
                 f.close()
     #@+node:ekr.20180207071114.5: *3* spell.create
     def create(self, fn):
-        '''Create the given file with empty contents.'''
+        """Create the given file with empty contents."""
+        # Make the directories as needed.
         theDir = g.os_path_dirname(fn)
-        g.makeAllNonExistentDirectories(theDir, c=self.c, force=True, verbose=True)
-            # Make the directories as needed.
+        if theDir:
+            ok = g.makeAllNonExistentDirectories(theDir)
+            # #1453: Don't assume the directory exists.
+            if not ok:
+                g.error(f"did not create directory: {theDir}")
+                return
+        # Create the file.
         try:
             f = open(fn, mode='wb')
             f.close()
-            g.note('created: %s' % (fn))
+            g.note(f"created: {fn}")
         except IOError:
-            g.error('can not create: %s' % (fn))
+            g.error(f"can not create: {fn}")
         except Exception:
-            g.error('unexpected error creating: %s' % (fn))
+            g.error(f"unexpected error creating: {fn}")
             g.es_exception()
     #@+node:ekr.20180207072351.1: *3* spell.find_user_dict
     def find_user_dict(self):
-        '''Return the full path to the local dictionary.'''
+        """Return the full path to the local dictionary."""
         c = self.c
-        fn = c.config.getString('enchant-local-dictionary')
-        if not fn:
-            fn = g.os_path_finalize_join(
-                g.app.loadDir,
-                "..", "plugins",
-                'spellpyx.txt',
-            )
-        # Fix bug https://github.com/leo-editor/leo-editor/issues/108
-        if not g.os_path_exists(fn):
-            fn = g.os_path_finalize_join(
-                g.app.homeDir,
-                '.leo',
-                'spellpyx.txt')
-        return fn
+        join = g.os_path_finalize_join
+        table = (
+            c.config.getString('enchant-local-dictionary'),
+                # Settings first.
+            join(g.app.homeDir, '.leo', 'spellpyx.txt'),
+                # #108: then the .leo directory.
+            join(g.app.loadDir, "..", "plugins", 'spellpyx.txt'),
+                # The plugins directory as a last resort.
+        )
+        for path in table:
+            if g.os_path_exists(path):
+                return path
+        g.es_print('Creating ~/.leo/spellpyx.txt')
+        # #1453: Return the default path.
+        return join(g.app.homeDir, '.leo', 'spellpyx.txt')
     #@+node:ekr.20150514063305.515: *3* spell.ignore
     def ignore(self, word):
-        
+
         self.d.add_to_session(word)
     #@+node:ekr.20150514063305.517: *3* spell.process_word
     def process_word(self, word):
@@ -90,7 +100,7 @@ class BaseSpellWrapper(object):
         d = self.d
         if not d:
             return None
-        elif d.check(word):
+        if d.check(word):
             return None
         # Speed doesn't matter here. The more we find, the more convenient.
         word = ''.join([i for i in word if not i.isdigit()])
@@ -110,27 +120,25 @@ class BaseSpellWrapper(object):
                 if not d.check(word2) and not d.check(word2.lower()):
                     return d.suggest(word)
             return None
-        else:
-            return d.suggest(word)
+        return d.suggest(word)
     #@-others
-#@+node:ekr.20180207075606.1: ** class DefaultDict (object)
-class DefaultDict(object):
-    '''A class with the same interface as the enchant dict class.'''
-    
+#@+node:ekr.20180207075606.1: ** class DefaultDict
+class DefaultDict:
+    """A class with the same interface as the enchant dict class."""
+
     def __init__(self, words=None):
         self.added_words = set()
         self.ignored_words = set()
         self.words = set() if words is None else set(words)
-
     #@+others
     #@+node:ekr.20180207075740.1: *3* dict.add
     def add(self, word):
-        '''Add a word to the dictionary.'''
+        """Add a word to the dictionary."""
         self.words.add(word)
         self.added_words.add(word)
     #@+node:ekr.20180207101513.1: *3* dict.add_words_from_dict
     def add_words_from_dict(self, kind, fn, words):
-        '''For use by DefaultWrapper.'''
+        """For use by DefaultWrapper."""
         for word in words or []:
             self.words.add(word)
             self.words.add(word.lower())
@@ -140,16 +148,16 @@ class DefaultDict(object):
         self.ignored_words.add(word)
     #@+node:ekr.20180207080007.1: *3* dict.check
     def check(self, word):
-        '''Return True if the word is in the dict.'''
+        """Return True if the word is in the dict."""
         for s in (word, word.lower(), word.capitalize()):
             if s in self.words or s in self.ignored_words:
                 return True
         return False
     #@+node:ekr.20180207081634.1: *3* dict.suggest & helpers
     def suggest(self, word):
-        
+
         def known(words):
-            '''Return the words that are in the dictionary.'''
+            """Return the words that are in the dictionary."""
             return [z for z in list(set(words)) if z in self.words]
 
         assert not known([word]), repr(word)
@@ -160,6 +168,8 @@ class DefaultDict(object):
         )
         return suggestions
     #@+node:ekr.20180207085717.1: *4* dict.edits1 & edits2
+    #@@nobeautify
+
     def edits1(self, word):
         "All edits that are one edit away from `word`."
         letters    = 'abcdefghijklmnopqrstuvwxyz'
@@ -170,7 +180,7 @@ class DefaultDict(object):
         inserts    = [L + c + R               for L, R in splits for c in letters]
         return list(set(deletes + transposes + replaces + inserts))
 
-    def edits2(self, word): 
+    def edits2(self, word):
         "All edits that are two edits away from `word`."
         return [e2 for e1 in self.edits1(word) for e2 in self.edits1(e1)]
     #@-others
@@ -178,9 +188,9 @@ class DefaultDict(object):
 class DefaultWrapper(BaseSpellWrapper):
     """
     A default spell checker for when pyenchant is not available.
-    
+
     Based on http://norvig.com/spell-correct.html
-    
+
     Main dictionary: ~/.leo/main_spelling_dict.txt
     User dictionary:
     - @string enchant_local_dictionary or
@@ -212,13 +222,13 @@ class DefaultWrapper(BaseSpellWrapper):
                 self.d.add_words_from_dict(kind, fn, words)
     #@+node:ekr.20180207110701.1: *3* default.add
     def add(self, word):
-        '''Add a word to the user dictionary.'''
+        """Add a word to the user dictionary."""
         self.d.add(word)
         self.d.add(word.lower())
         self.save_user_dict()
     #@+node:ekr.20180207100238.1: *3* default.find_main_dict
     def find_main_dict(self):
-        '''Return the full path to the global dictionary.'''
+        """Return the full path to the global dictionary."""
         c = self.c
         fn = c.config.getString('main-spelling-dictionary')
         if fn and g.os_path_exists(fn):
@@ -227,47 +237,27 @@ class DefaultWrapper(BaseSpellWrapper):
         fn = g.os_path_finalize_join(
             g.app.homeDir, '.leo', 'main_spelling_dict.txt')
         return fn if g.os_path_exists(fn) else None
-    #@+node:ekr.20180207073815.1: *3* default.read_words & helper
+    #@+node:ekr.20180207073815.1: *3* default.read_words
     def read_words(self, kind, fn):
-        '''Return all the words from the dictionary file.'''
+        """Return all the words from the dictionary file."""
         words = set()
         try:
             with open(fn, 'rb') as f:
                 s = g.toUnicode(f.read())
+                # #1688: Do this in place.
                 for line in g.splitLines(s):
-                    self.add_expanded_line(line, words)
+                    line = line.strip()
+                    if line and not line.startswith('#'):
+                        words.add(line)
         except Exception:
-            g.es_print('can not open %s dictionary: %s' % (kind, fn))
+            g.es_print(f"can not open {kind} dictionary: {fn}")
         return words
-    #@+node:ekr.20180207132550.1: *4* default.add_expanded_line
-    def add_expanded_line(self, s, words):
-        '''Add the expansion of line s to the words set.'''
-        s = g.toUnicode(s).strip()
-        if not s or s.startswith('#'):
-            return
-        # Strip off everything after /
-        i = s.find('/')
-        if i > -1:
-            flags = s[i+1:].strip().lower()
-            s = s[:i].strip()
-        else:
-            flags = ''
-        if not s:
-            return
-        words.add(s)
-        words.add(s.lower())
-        # Flags are not properly documented.
-        # Adding plurals is good enough for now.
-        if 's' in flags and not s.endswith('s'):
-            words.add(s+'s')
-            words.add(s.lower()+'s')
-      
     #@+node:ekr.20180207110718.1: *3* default.save_dict
     def save_dict(self, kind, fn, trace=False):
-        '''
+        """
         Save the dictionary whose name is given, alphabetizing the file.
         Write added words to the file if kind is 'user'.
-        '''
+        """
         if not fn:
             return
         words = self.read_words(kind, fn)
@@ -284,15 +274,15 @@ class DefaultWrapper(BaseSpellWrapper):
         f.close()
     #@+node:ekr.20180211104628.1: *3* default.save_main/user_dict
     def save_main_dict(self, trace=False):
-        
+
         self.save_dict('main', self.main_fn, trace=trace)
-        
+
     def save_user_dict(self, trace=False):
 
         self.save_dict('user', self.user_fn, trace=trace)
     #@+node:ekr.20180209141933.1: *3* default.show_info
     def show_info(self):
-        
+
         if self.main_fn:
             g.es_print('Default spell checker')
             table = (
@@ -303,12 +293,12 @@ class DefaultWrapper(BaseSpellWrapper):
             g.es_print('\nSpell checking has been disabled.')
             g.es_print('To enable, put a main dictionary at:')
             g.es_print('~/.leo/main_spelling_dict.txt')
-            table = (
+            table = (  # type:ignore
                 ('user', self.user_fn),
             )
         for kind, fn in table:
-            g.es_print('%s dictionary: %s' % (
-                kind, g.os_path_normpath(fn) if fn else 'None'))
+            g.es_print(
+                f"{kind} dictionary: {(g.os_path_normpath(fn) if fn else 'None')}")
     #@-others
 #@+node:ekr.20150514063305.510: ** class EnchantWrapper (BaseSpellWrapper)
 class EnchantWrapper(BaseSpellWrapper):
@@ -324,17 +314,17 @@ class EnchantWrapper(BaseSpellWrapper):
         g.app.spellDict = self.d = self.open_dict_file(fn)
     #@+node:ekr.20180207073536.1: *3* enchant.create_dict_from_file
     def create_dict_from_file(self, fn, language):
-     
+
         return enchant.DictWithPWL(language, fn)
     #@+node:ekr.20180207074613.1: *3* enchant.default_dict
     def default_dict(self, language):
-        
+
         return enchant.Dict(language)
     #@+node:ekr.20180207072846.1: *3* enchant.init_language
     def init_language(self):
-        '''Init self.language.'''
+        """Init self.language."""
         c = self.c
-        language = g.toUnicode(c.config.getString('enchant-language'))
+        language = g.checkUnicode(c.config.getString('enchant-language'))
         if language:
             try:
                 ok = enchant.dict_exists(language)
@@ -348,7 +338,7 @@ class EnchantWrapper(BaseSpellWrapper):
         self.language = language
     #@+node:ekr.20180207102856.1: *3* enchant.open_dict_file
     def open_dict_file(self, fn):
-        '''Open or create the dict with the given fn.'''
+        """Open or create the dict with the given fn."""
         language = self.language
         if not fn or not language:
             return None
@@ -372,23 +362,9 @@ class EnchantWrapper(BaseSpellWrapper):
             # A fallback.  Unlikely to happen.
             d = enchant.Dict(language)
         return d
-    #@+node:ekr.20150514063305.513: *3* spell.clean_dict
-    def clean_dict(self, fn):
-        if g.os_path_exists(fn):
-            f = open(fn, mode='rb')
-            s = f.read()
-            f.close()
-            # Blanks lines cause troubles.
-            s2 = s.replace(b'\r', b'').replace(b'\n\n', b'\n')
-            if s2.startswith(b'\n'): s2 = s2[1:]
-            if s != s2:
-                g.es_print('cleaning', fn)
-                f = open(fn, mode='wb')
-                f.write(s2)
-                f.close()
     #@+node:ekr.20150514063305.515: *3* spell.ignore
     def ignore(self, word):
-        
+
         self.d.add_to_session(word)
     #@+node:ekr.20150514063305.517: *3* spell.process_word
     def process_word(self, word):
@@ -399,7 +375,7 @@ class EnchantWrapper(BaseSpellWrapper):
         d = self.d
         if not d:
             return None
-        elif d.check(word):
+        if d.check(word):
             return None
         # Speed doesn't matter here. The more we find, the more convenient.
         word = ''.join([i for i in word if not i.isdigit()])
@@ -419,45 +395,44 @@ class EnchantWrapper(BaseSpellWrapper):
                 if not d.check(word2) and not d.check(word2.lower()):
                     return d.suggest(word)
             return None
-        else:
-            return d.suggest(word)
+        return d.suggest(word)
     #@+node:ekr.20180209142310.1: *3* spell.show_info
     def show_info(self):
 
         g.es_print('pyenchant spell checker')
-        g.es_print('user dictionary:   %s' % self.find_user_dict())
+        g.es_print(f"user dictionary:   {self.find_user_dict()}")
         try:
             aList = enchant.list_dicts()
             aList2 = [a for a, b in aList]
-            g.es_print('main dictionaries: %s' % ', '.join(aList2))
+            g.es_print(f"main dictionaries: {', '.join(aList2)}")
         except Exception:
             g.es_exception()
-
     #@-others
 #@+node:ekr.20150514063305.481: ** class SpellCommandsClass
 class SpellCommandsClass(BaseEditCommandsClass):
-    '''Commands to support the Spell Tab.'''
+    """Commands to support the Spell Tab."""
     #@+others
     #@+node:ekr.20150514063305.482: *3* ctor & reloadSettings(SpellCommandsClass)
     def __init__(self, c):
-        '''
+        """
         Ctor for SpellCommandsClass class.
         Inits happen when the first frame opens.
-        '''
+        """
         # pylint: disable=super-init-not-called
         self.c = c
         self.handler = None
         self.reloadSettings()
-        
+
     def reloadSettings(self):
-        '''SpellCommandsClass.reloadSettings.'''
+        """SpellCommandsClass.reloadSettings."""
         c = self.c
-        self.page_width = c.config.getInt("page-width")
-            # for wrapping
+        self.page_width = c.config.getInt("page-width")  # for wrapping
     #@+node:ekr.20150514063305.484: *3* openSpellTab
     @cmd('spell-tab-open')
     def openSpellTab(self, event=None):
-        '''Open the Spell Checker tab in the log pane.'''
+        """Open the Spell Checker tab in the log pane."""
+        if g.unitTesting:
+            return
         c = self.c
         log = c.frame.log
         tabName = 'Spell'
@@ -468,7 +443,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
             self.handler = SpellTabHandler(c, tabName)
         # Bug fix: 2013/05/22.
         if not self.handler.loaded:
-            log.deleteTab(tabName, force=True)
+            log.deleteTab(tabName)
         # spell as you type stuff
         self.suggestions = []
         self.suggestions_idx = None
@@ -479,12 +454,12 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20171205043931.1: *4* add
     @cmd('spell-add')
     def add(self, event=None):
-        '''
+        """
         Simulate pressing the 'add' button in the Spell tab.
 
         Just open the Spell tab if it has never been opened.
         For minibuffer commands, we must also force the Spell tab to be visible.
-        '''
+        """
         # self.handler is a SpellTabHandler object (inited by openSpellTab)
         if self.handler:
             self.openSpellTab()
@@ -494,12 +469,12 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.486: *4* find
     @cmd('spell-find')
     def find(self, event=None):
-        '''
+        """
         Simulate pressing the 'Find' button in the Spell tab.
 
         Just open the Spell tab if it has never been opened.
         For minibuffer commands, we must also force the Spell tab to be visible.
-        '''
+        """
         # self.handler is a SpellTabHandler object (inited by openSpellTab)
         if self.handler:
             self.openSpellTab()
@@ -509,7 +484,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.487: *4* change
     @cmd('spell-change')
     def change(self, event=None):
-        '''Simulate pressing the 'Change' button in the Spell tab.'''
+        """Simulate pressing the 'Change' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
             self.handler.change()
@@ -518,7 +493,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.488: *4* changeThenFind
     @cmd('spell-change-then-find')
     def changeThenFind(self, event=None):
-        '''Simulate pressing the 'Change, Find' button in the Spell tab.'''
+        """Simulate pressing the 'Change, Find' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
             # A workaround for a pylint warning:
@@ -530,14 +505,14 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.489: *4* hide
     @cmd('spell-tab-hide')
     def hide(self, event=None):
-        '''Hide the Spell tab.'''
+        """Hide the Spell tab."""
         if self.handler:
             self.c.frame.log.selectTab('Log')
             self.c.bodyWantsFocus()
     #@+node:ekr.20150514063305.490: *4* ignore
     @cmd('spell-ignore')
     def ignore(self, event=None):
-        '''Simulate pressing the 'Ignore' button in the Spell tab.'''
+        """Simulate pressing the 'Ignore' button in the Spell tab."""
         if self.handler:
             self.openSpellTab()
             self.handler.ignore()
@@ -546,7 +521,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
     #@+node:ekr.20150514063305.491: *4* focusToSpell
     @cmd('focus-to-spell-tab')
     def focusToSpell(self, event=None):
-        '''Put focus in the spell tab.'''
+        """Put focus in the spell tab."""
         self.openSpellTab()
             # Makes Spell tab visible.
         # This is not a great idea. There is no indication of focus.
@@ -590,8 +565,8 @@ class SpellCommandsClass(BaseEditCommandsClass):
         if not self.suggestions:
             g.es('[no suggestions]')
             return
-        word = self.suggestions[self.suggestion_idx]
-        self.suggestion_idx = (self.suggestion_idx + 1) % len(self.suggestions)
+        word = self.suggestions[self.suggestion_idx]  # type:ignore
+        self.suggestion_idx = (self.suggestion_idx + 1) % len(self.suggestions)  # type:ignore
         self.as_you_type_replace(word)
     #@+node:ekr.20150514063305.496: *4* as_you_type_undo
     @cmd('spell-as-you-type-undo')
@@ -618,11 +593,11 @@ class SpellCommandsClass(BaseEditCommandsClass):
             return
         c = self.c
         spell_ok = True
-        if self.spell_as_you_type: # might just be for wrapping
+        if self.spell_as_you_type:  # might just be for wrapping
             w = c.frame.body.wrapper
             txt = w.getAllText()
             i = w.getInsertPoint()
-            word = txt[: i].rsplit(None, 1)[-1]
+            word = txt[:i].rsplit(None, 1)[-1]
             word = ''.join(i if i.isalpha() else ' ' for i in word).split()
             if word:
                 word = word[-1]
@@ -630,7 +605,7 @@ class SpellCommandsClass(BaseEditCommandsClass):
                 suggests = ec.process_word(word)
                 if suggests:
                     spell_ok = False
-                    g.es(' '.join(suggests[: 5]) +
+                    g.es(' '.join(suggests[:5]) +
                          ('...' if len(suggests) > 5 else ''),
                          color='red')
                 elif suggests is not None:
@@ -645,15 +620,15 @@ class SpellCommandsClass(BaseEditCommandsClass):
             i = w.getInsertPoint()
             # calculate the current column
             parts = txt.split('\n')
-            popped = 0 # chars on previous lines
+            popped = 0  # chars on previous lines
             while len(parts[0]) + popped < i:
-                popped += len(parts.pop(0)) + 1 # +1 for the \n that's gone
+                popped += len(parts.pop(0)) + 1  # +1 for the \n that's gone
             col = i - popped
             if col > self.page_width:
-                txt = txt[: i] + '\n' + txt[i:] # replace space with \n
+                txt = txt[:i] + '\n' + txt[i:]  # replace space with \n
                 w.setAllText(txt)
                 c.p.b = txt
-                w.setInsertPoint(i + 1) # must come after c.p.b assignment
+                w.setInsertPoint(i + 1)  # must come after c.p.b assignment
     #@+node:ekr.20150514063305.498: *4* as_you_type_replace
     def as_you_type_replace(self, word):
         """as_you_type_replace - replace the word behind the cursor
@@ -675,14 +650,14 @@ class SpellCommandsClass(BaseEditCommandsClass):
             i -= 1
         if i or (txt and not txt[0].isalpha()):
             i += 1
-        txt = txt[: i] + word + txt[j:]
+        txt = txt[:i] + word + txt[j:]
         w.setAllText(txt)
         c.p.b = txt
         w.setInsertPoint(i + len(word) + xtra - 1)
         c.bodyWantsFocusNow()
     #@-others
 #@+node:ekr.20150514063305.499: ** class SpellTabHandler
-class SpellTabHandler(object):
+class SpellTabHandler:
     """A class to create and manage Leo's Spell Check dialog."""
     #@+others
     #@+node:ekr.20150514063305.501: *3* SpellTabHandler.__init__
@@ -693,24 +668,20 @@ class SpellTabHandler(object):
         self.c = c
         self.body = c.frame.body
         self.currentWord = None
-        self.re_word = re.compile(
-            # Don't include underscores in words. It just complicates things.
-            # [^\W\d_] means any unicode char except underscore or digit.
-            r"([^\W\d_]+)(['`][^\W\d_]+)?",
-            flags=re.UNICODE)
+        # Don't include underscores in words. It just complicates things.
+        # [^\W\d_] means any unicode char except underscore or digit.
+        self.re_word = re.compile(r"([^\W\d_]+)(['`][^\W\d_]+)?", flags=re.UNICODE)
         self.outerScrolledFrame = None
-        self.seen = set()
-            # Adding a word to seen will ignore it until restart.
+        self.seen = set()  # Adding a word to seen will ignore it until restart.
+        # A text widget for scanning. Must have a parent frame.
         self.workCtrl = g.app.gui.plainTextWidget(c.frame.top)
-            # A text widget for scanning.
-            # Must have a parent frame even though it is not packed.
         if enchant:
             self.spellController = EnchantWrapper(c)
             self.tab = g.app.gui.createSpellTab(c, self, tabName)
             self.loaded = True
             return
         # Create the spellController for the show-spell-info command.
-        self.spellController = DefaultWrapper(c)
+        self.spellController = DefaultWrapper(c)  # type:ignore
         self.loaded = bool(self.spellController.main_fn)
         if self.loaded:
             # Create the spell tab only if the main dict exists.
@@ -731,24 +702,27 @@ class SpellTabHandler(object):
     def change(self, event=None):
         """Make the selected change to the text"""
         if not self.loaded:
-            return
-        c = self.c
+            return False
+        c, p, u = self.c, self.c.p, self.c.undoer
         w = c.frame.body.wrapper
         selection = self.tab.getSuggestion()
         if selection:
+            bunch = u.beforeChangeBody(p)
             # Use getattr to keep pylint happy.
-            if hasattr(self.tab, 'change_i') and getattr(self.tab, 'change_i') is not None:
-                start = getattr(self.tab, 'change_i')
-                end = getattr(self.tab, 'change_j')
-                oldSel = start, end
+            i = getattr(self.tab, 'change_i', None)
+            j = getattr(self.tab, 'change_j', None)
+            if i is not None:
+                start, end = i, j
             else:
-                start, end = oldSel = w.getSelectionRange()
+                start, end = w.getSelectionRange()
             if start is not None:
-                if start > end: start, end = end, start
+                if start > end:
+                    start, end = end, start
                 w.delete(start, end)
                 w.insert(start, selection)
                 w.setSelectionRange(start, start + len(selection))
-                c.frame.body.onBodyChanged("Change", oldSel=oldSel)
+                p.v.b = w.getAllText()
+                u.afterChangeBody(p, 'Change', bunch)
                 c.invalidateFocus()
                 c.bodyWantsFocus()
                 return True
@@ -793,10 +767,10 @@ class SpellTabHandler(object):
                     c.invalidateFocus()
                     c.bodyWantsFocus()
                     w.setSelectionRange(i, j, insert=j)
-                    w.see(j)
+                    k = g.see_more_lines(s, j, 4)
+                    w.see(k)
                     return
-                else:
-                    self.seen.add(word)
+                self.seen.add(word)
             # No more misspellings in p
             p.moveToThreadNext()
             if p:
@@ -811,7 +785,7 @@ class SpellTabHandler(object):
                 return
     #@+node:ekr.20160415033936.1: *5* showMisspelled
     def showMisspelled(self, p):
-        '''Show the position p, contracting the tree as needed.'''
+        """Show the position p, contracting the tree as needed."""
         c = self.c
         redraw = not p.isVisible(c)
         # New in Leo 4.4.8: show only the 'sparse' tree when redrawing.
@@ -848,24 +822,24 @@ def show_spell_info(event=None):
 #@+node:ekr.20180211104019.1: ** @g.command('clean-main-spell-dict')
 @g.command('clean-main-spell-dict')
 def clean_main_spell_dict(event):
-    '''
+    """
     Clean the main spelling dictionary used *only* by the default spell
     checker.
-    
+
     This command works regardless of the spell checker being used.
-    '''
+    """
     c = event and event.get('c')
     if c:
         DefaultWrapper(c).save_main_dict(trace=True)
 #@+node:ekr.20180211105748.1: ** @g.command('clean-user-spell-dict')
 @g.command('clean-user-spell-dict')
 def clean_user_spell_dict(event):
-    '''
+    """
     Clean the user spelling dictionary used *only* by the default spell
     checker. Mostly for debugging, because this happens automatically.
-    
+
     This command works regardless of the spell checker being used.
-    '''
+    """
     c = event and event.get('c')
     if c:
         DefaultWrapper(c).save_user_dict(trace=True)
