@@ -2,9 +2,13 @@
 #@+leo-ver=5-thin
 #@+node:ekr.20130302121602.10208: * @file leoDebugger.py
 #@@first
+
+# Disable all mypy errors.
+# type:ignore
+
 #@+<< leoDebugger.py docstring >>
 #@+node:ekr.20181006100710.1: ** << leoDebugger.py docstring >>
-'''
+"""
 Leo's integrated debugger supports the xdb and various db-* commands,
 corresponding to the pdb commands:
 
@@ -67,62 +71,60 @@ to any executable line. The line_numbering plugin must be enabled and @bool
 use_gutter must be True.
 
 The xdb_pane plugin creates the Debug pane in the Log window.
-'''
+"""
 #@-<< leoDebugger.py docstring >>
 #@+<< leoDebugger.py imports >>
 #@+node:ekr.20181006100604.1: ** << leoDebugger.py imports >>
-import leo.core.leoGlobals as g
 import bdb
-if g.isPython3:
-    import queue
-else:
-    import Queue as queue
+import queue
 import os
 import pdb
 import re
 import sys
 import threading
+from leo.core import leoGlobals as g
 #@-<< leoDebugger.py imports >>
 #@+others
 #@+node:ekr.20180701050839.5: ** class Xdb (pdb.Pdb, threading.Thread)
 class Xdb(pdb.Pdb, threading.Thread):
-    '''
+    """
     An debugger, a subclass of Pdb, that runs in a separate thread without
     hanging Leo. Only one debugger, g.app.xdb, can be active at any time.
-    
+
     A singleton listener method, g.app,xdb_timer, runs in the main Leo
     thread. The listener runs until Leo exists.
-    
+
     Two Queues communicate between the threads:
-        
+
     - xdb.qc contains commands from the main thread to this thread.
       All xdb/pdb input comes from this queue.
-    
+
     - xdb.qr contains requests from the xdb thread to the main thread.
       All xdb/pdb output goes to this queue.
-    
+
     Settings
     --------
-    
+
    - @bool use_xdb_pane_output_area: when True, all debugger output is sent
     to an output area in the Debug pane.
-    
-    @bool bool use_gutter: when True, line numbers appear to the left of
+
+    @bool use_gutter: when True, line numbers appear to the left of
     the body pane. Clicking to the left of the gutter toggles breakpoints
     when xdb is active.
-    '''
+    """
     #@+others
     #@+node:ekr.20180701050839.4: *3* class QueueStdin (obj)
-    class QueueStdin(object):
-        '''
+    class QueueStdin:
+        """
         A replacement for Python's stdin class containing only readline().
-        '''
+        """
+
         def __init__(self, qc):
             self.qc = qc
 
         def readline(self):
-            '''Return the next line from the qc channel.'''
-            s = self.qc.get() # blocks
+            """Return the next line from the qc channel."""
+            s = self.qc.get()  # blocks
             if 1:
                 # Just echo.
                 print(s.rstrip())
@@ -135,24 +137,25 @@ class Xdb(pdb.Pdb, threading.Thread):
                     print(s)
             return s
     #@+node:ekr.20181003020344.1: *3* class QueueStdout (obj)
-    class QueueStdout(object):
-        '''
+    class QueueStdout:
+        """
         A replacement for Python's stdout class containing only write().
-        '''
+        """
+
         def __init__(self, qr):
             self.qr = qr
-            
+
         def flush(self):
             pass
 
         def write(self, s):
-            '''Write s to the qr channel'''
+            """Write s to the qr channel"""
             self.qr.put(['put-stdout', s])
     #@+node:ekr.20181006160108.1: *3* xdb.__init__
     def __init__(self, path=None):
-            
-        self.qc = queue.Queue() # The command queue.
-        self.qr = queue.Queue() # The request queue.
+
+        self.qc = queue.Queue()  # The command queue.
+        self.qr = queue.Queue()  # The request queue.
         stdin_q = self.QueueStdin(qc=self.qc)
         stdout_q = self.QueueStdout(qr=self.qr)
         # Start the singleton listener, in the main Leo thread.
@@ -164,15 +167,12 @@ class Xdb(pdb.Pdb, threading.Thread):
             self.timer.start()
         # Init the base classes.
         threading.Thread.__init__(self)
-        if g.isPython3:
-            pdb.Pdb.__init__(self,
-                stdin=stdin_q,
-                stdout=stdout_q,
-                readrc=False,
-                    # Don't read a .rc file.
-            )
-        else:
-             pdb.Pdb.__init__(self, stdin=stdin_q, stdout=stdout_q)
+        super().__init__(
+            stdin=stdin_q,
+            stdout=stdout_q,
+            readrc=False,
+                # Don't read a .rc file.
+        )
         sys.stdout = stdout_q
         self.daemon = True
         self.path = path
@@ -193,7 +193,7 @@ class Xdb(pdb.Pdb, threading.Thread):
             return False
     #@+node:ekr.20181002061627.1: *4* xdb.cmdloop (overrides Cmd)
     def cmdloop(self, intro=None):
-        '''Override Cmd.cmdloop.'''
+        """Override Cmd.cmdloop."""
         assert not intro, repr(intro)
         stop = None
         while not stop:
@@ -203,14 +203,11 @@ class Xdb(pdb.Pdb, threading.Thread):
             else:
                 self.stdout.write(self.prompt)
                 self.stdout.flush()
-                line = self.stdin.readline()
-                    # QueueStdin.readline.
-                    # Get the input from Leo's main thread.
+                # Get the input from Leo's main thread.
+                line = self.stdin.readline()  # QueueStdin.readline:
                 line = line.rstrip('\r\n') if line else 'EOF'
-            line = self.precmd(line)
-                # Pdb.precmd.
-            stop = self.onecmd(line)
-                # Pdb.onecmd.
+            line = self.precmd(line)  # Pdb.precmd.
+            stop = self.onecmd(line)  # Pdb.onecmd.
             # Show the line in Leo.
             if stop:
                 self.select_line(self.saved_frame, self.saved_traceback)
@@ -222,7 +219,6 @@ class Xdb(pdb.Pdb, threading.Thread):
         first ask confirmation).  With a filename:lineno argument,
         clear all breaks at that line in that file.
         """
-        import bdb
         # Same as pdb.do_clear except uses self.stdin.readline (as it should).
         if not arg:
             bplist = [bp for bp in bdb.Breakpoint.bpbynumber if bp]
@@ -232,17 +228,17 @@ class Xdb(pdb.Pdb, threading.Thread):
                 if reply in ('y', 'yes'):
                     self.clear_all_breaks()
                     for bp in bplist:
-                        self.message('Deleted %s' % bp)
+                        self.message(f"Deleted {bp}")
             return
         if ':' in arg:
             # Make sure it works for "clear C:\foo\bar.py:12"
             i = arg.rfind(':')
             filename = arg[:i]
-            arg = arg[i+1:]
+            arg = arg[i + 1 :]
             try:
                 lineno = int(arg)
             except ValueError:
-                err = "Invalid line number (%s)" % arg
+                err = f"Invalid line number ({arg})"
             else:
                 bplist = self.get_breaks(filename, lineno)
                 err = self.clear_break(filename, lineno)
@@ -250,7 +246,7 @@ class Xdb(pdb.Pdb, threading.Thread):
                 self.error(err)
             else:
                 for bp in bplist:
-                    self.message('Deleted %s' % bp)
+                    self.message(f"Deleted {bp}")
             return
         numberlist = arg.split()
         for i in numberlist:
@@ -260,9 +256,9 @@ class Xdb(pdb.Pdb, threading.Thread):
                 self.error(err)
             else:
                 self.clear_bpbynumber(i)
-                self.message('Deleted %s' % bp)
+                self.message(f"Deleted {bp}")
 
-    do_cl = do_clear # 'c' is already an abbreviation for 'continue'
+    do_cl = do_clear  # 'c' is already an abbreviation for 'continue'
 
     # complete_clear = self._complete_location
     # complete_cl = self._complete_location
@@ -282,7 +278,7 @@ class Xdb(pdb.Pdb, threading.Thread):
     do_exit = do_quit
     #@+node:ekr.20180701050839.8: *4* xdb.interaction (overrides Pdb)
     def interaction(self, frame, traceback):
-        '''Override.'''
+        """Override."""
         self.saved_frame = frame
         self.saved_traceback = traceback
         self.select_line(frame, traceback)
@@ -290,8 +286,7 @@ class Xdb(pdb.Pdb, threading.Thread):
             # Call the base class method.
     #@+node:ekr.20180701050839.10: *4* xdb.set_continue (overrides Bdb)
     def set_continue(self):
-        ''' override Bdb.set_continue'''
-        import sys
+        """ override Bdb.set_continue"""
         # Don't stop except at breakpoints or when finished
         self._set_stopinfo(self.botframe, None, -1)
         if not self.breaks:
@@ -305,34 +300,35 @@ class Xdb(pdb.Pdb, threading.Thread):
                 frame = frame.f_back
     #@+node:ekr.20181006052604.1: *3* xdb.has_breakpoint & has_breakpoints
     def has_breakpoint(self, filename, lineno):
-        '''Return True if there is a breakpoint at the given file and line.'''
+        """Return True if there is a breakpoint at the given file and line."""
         filename = self.canonic(filename)
         aList = self.breaks.get(filename) or []
         return lineno in aList
 
     def has_breakpoints(self):
-        '''Return True if there are any breakpoints.'''
+        """Return True if there are any breakpoints."""
         return self.breaks
     #@+node:ekr.20181002094126.1: *3* xdb.run
     def run(self):
-        '''The thread's run method: called via start.'''
+        """The thread's run method: called via start."""
         # pylint: disable=arguments-differ
         from leo.core.leoQt import QtCore
-        QtCore.pyqtRemoveInputHook() # From g.pdb
+        QtCore.pyqtRemoveInputHook()  # From g.pdb
         if self.path:
             self.run_path(self.path)
         else:
             self.set_trace()
     #@+node:ekr.20180701090439.1: *3* xdb.run_path
     def run_path(self, path):
-        '''Begin execution of the python file.'''
+        """Begin execution of the python file."""
         source = g.readFileIntoUnicodeString(path)
         fn = g.shortFileName(path)
         try:
             code = compile(source, fn, 'exec')
         except Exception:
             g.es_exception()
-            return g.trace('can not compile', path)
+            g.trace('can not compile', path)
+            return
         self.reset()
         sys.settrace(self.trace_dispatch)
         try:
@@ -346,7 +342,7 @@ class Xdb(pdb.Pdb, threading.Thread):
             sys.settrace(None)
     #@+node:ekr.20180701151233.1: *3* xdb.select_line
     def select_line(self, frame, traceback):
-        '''Select the given line in Leo.'''
+        """Select the given line in Leo."""
         stack, curindex = self.get_stack(frame, traceback)
         frame, lineno = stack[curindex]
         filename = frame.f_code.co_filename
@@ -355,33 +351,33 @@ class Xdb(pdb.Pdb, threading.Thread):
             # xdb.show_line finalizes the file name.
     #@+node:ekr.20181007044254.1: *3* xdb.write
     def write(self, s):
-        '''Write s to the output stream.'''
+        """Write s to the output stream."""
         self.qr.put(['put-stdout', s])
     #@-others
 #@+node:ekr.20181007063214.1: ** top-level functions
 # These functions run in Leo's main thread.
 #@+node:ekr.20181004120344.1: *3* function: get_gnx_from_file
 def get_gnx_from_file(file_s, p, path):
-    '''Set p's gnx from the @file node in the derived file.'''
+    """Set p's gnx from the @file node in the derived file."""
     pat = re.compile(r'^#@\+node:(.*): \*+ @file (.+)$')
     for line in g.splitLines(file_s):
         m = pat.match(line)
         if m:
             gnx, path2 = m.group(1), m.group(2)
-            path2 = path2.replace('\\','/')
+            path2 = path2.replace('\\', '/')
             p.v.fileIndex = gnx
             if path == path2:
                 return True
-    g.trace('Not found: @+node for %s' % path)
+    g.trace(f"Not found: @+node for {path}")
     g.trace('Reverting to @auto')
     return False
 #@+node:ekr.20180701050839.3: *3* function: listener
 def listener(timer):
-    '''
+    """
     Listen, at idle-time, in Leo's main thread, for data on the qr channel.
-    
+
     This is a singleton timer, created by the xdb command.
-    '''
+    """
     if g.app.killed:
         return
     xdb = getattr(g.app, 'xdb', None)
@@ -391,7 +387,7 @@ def listener(timer):
     xpd_pane = getattr(c, 'xpd_pane', None)
     kill = False
     while not xdb.qr.empty():
-        aList = xdb.qr.get() # blocks
+        aList = xdb.qr.get()  # blocks
         kind = aList[0]
         if kind == 'clear-stdout':
             if xpd_pane:
@@ -417,16 +413,16 @@ def listener(timer):
             # self.timer.stop()
 #@+node:ekr.20181004060517.1: *3* function: make_at_file_node
 def make_at_file_node(line, path):
-    '''
+    """
     Make and populate an @auto node for the given path.
-    '''
+    """
     c = g.app.log.c
     if not c:
-        return
-    path = g.os_path_finalize(path).replace('\\','/')
+        return None
+    path = g.os_path_finalize(path).replace('\\', '/')
     if not g.os_path_exists(path):
         g.trace('Not found:', repr(path))
-        return
+        return None
     # Create the new node.
     p = c.lastTopLevel().insertAfter()
     # Like c.looksLikeDerivedFile, but retaining the contents.
@@ -437,24 +433,24 @@ def make_at_file_node(line, path):
         # Set p.v.gnx from the derived file.
         is_derived = get_gnx_from_file(file_s, p, path)
     kind = '@file' if is_derived else '@auto'
-    p.h = '%s %s' % (kind, path)
+    p.h = f"{kind} {path}"
     c.selectPosition(p)
     c.refreshFromDisk()
     return p
 #@+node:ekr.20180701061957.1: *3* function: show_line
 def show_line(line, fn):
-    '''
+    """
     Put the cursor on the requested line of the given file.
     fn should be a full path to a file.
-    '''
+    """
     c = g.app.log.c
-    target = g.os_path_finalize(fn).replace('\\','/')
+    target = g.os_path_finalize(fn).replace('\\', '/')
     if not g.os_path_exists(fn):
         g.trace('===== Does not exist', fn)
         return
     for p in c.all_positions():
         if p.isAnyAtFileNode():
-            path = g.fullPath(c, p).replace('\\','/')
+            path = g.fullPath(c, p).replace('\\', '/')
             if target == path:
                 # Select the line.
                 junk_p, junk_offset, ok = c.gotoCommands.find_file_line(n=line, p=p)
@@ -470,7 +466,7 @@ def show_line(line, fn):
 #@+node:ekr.20181003015017.1: *3* db-again
 @g.command('db-again')
 def xdb_again(event):
-    '''Repeat the previous xdb command.'''
+    """Repeat the previous xdb command."""
     xdb = getattr(g.app, 'xdb', None)
     if xdb:
         xdb.qc.put(xdb.lastcmd)
@@ -479,7 +475,7 @@ def xdb_again(event):
 #@+node:ekr.20181003054157.1: *3* db-b
 @g.command('db-b')
 def xdb_breakpoint(event):
-    '''Set the breakpoint at the presently select line in Leo.'''
+    """Set the breakpoint at the presently select line in Leo."""
     c = event.get('c')
     if not c:
         return
@@ -507,59 +503,60 @@ def xdb_breakpoint(event):
     row, col = g.convertPythonIndexToRowCol(s, i)
     n = x.node_offset_to_file_line(row, p, root)
     if n is not None:
-        xdb.qc.put('b %s:%s' % (path, n+1))
+        xdb.qc.put(f"b {path}:{n + 1}")
 #@+node:ekr.20180702074705.1: *3* db-c/h/l/n/q/r/s/w
 @g.command('db-c')
 def xdb_c(event):
-    '''execute the pdb 'continue' command.'''
+    """execute the pdb 'continue' command."""
     db_command(event, 'c')
-    
+
 @g.command('db-h')
 def xdb_h(event):
-    '''execute the pdb 'continue' command.'''
+    """execute the pdb 'continue' command."""
     db_command(event, 'h')
-    
+
 @g.command('db-l')
 def xdb_l(event):
-    '''execute the pdb 'list' command.'''
+    """execute the pdb 'list' command."""
     db_command(event, 'l')
-    
+
 @g.command('db-n')
 def xdb_n(event):
-    '''execute the pdb 'next' command.'''
+    """execute the pdb 'next' command."""
     db_command(event, 'n')
-    
+
 @g.command('db-q')
 def xdb_q(event):
-    '''execute the pdb 'quit' command.'''
+    """execute the pdb 'quit' command."""
     db_command(event, 'q')
-    
+
 @g.command('db-r')
 def xdb_r(event):
-    '''execute the pdb 'return' command.'''
+    """execute the pdb 'return' command."""
     db_command(event, 'r')
-    
+
 @g.command('db-s')
 def xdb_s(event):
-    '''execute the pdb 'step' command.'''
+    """execute the pdb 'step' command."""
     db_command(event, 's')
-    
+
 @g.command('db-w')
 def xdb_w(event):
-    '''execute the pdb 'where' command.'''
+    """execute the pdb 'where' command."""
     db_command(event, 'w')
 #@+node:ekr.20180701050839.2: *3* db-input
 @g.command('db-input')
 def xdb_input(event):
-    '''Prompt the user for a pdb command and execute it.'''
+    """Prompt the user for a pdb command and execute it."""
     c = event.get('c')
     if not c:
-        return g.trace('no c')
+        g.trace('no c')
+        return
     xdb = getattr(g.app, 'xdb', None)
     if not xdb:
         print('xdb not active')
         return
-        
+
     def callback(args, c, event):
         xdb = getattr(g.app, 'xdb', None)
         if xdb:
@@ -574,7 +571,7 @@ def xdb_input(event):
 #@+node:ekr.20181003015636.1: *3* db-status
 @g.command('db-status')
 def xdb_status(event):
-    '''Print whether xdb is active.'''
+    """Print whether xdb is active."""
     xdb = getattr(g.app, 'xdb', None)
     print('active' if xdb else 'inactive')
 #@+node:ekr.20181006163454.1: *3* do_command
@@ -588,7 +585,7 @@ def db_command(event, command):
 #@+node:ekr.20180701050839.1: *3* xdb
 @g.command('xdb')
 def xdb_command(event):
-    '''Start the external debugger on a toy test program.'''
+    """Start the external debugger on a toy test program."""
     c = event.get('c')
     if not c:
         return
@@ -597,7 +594,8 @@ def xdb_command(event):
         g.trace('Not in an @<file> tree')
         return
     if not g.os_path_exists(path):
-        return g.trace('not found', path)
+        g.trace('not found', path)
+        return
     os.chdir(g.os_path_dirname(path))
     xdb = getattr(g.app, 'xdb', None)
     if xdb:
