@@ -1,20 +1,20 @@
 #@+leo-ver=5-thin
 #@+node:ekr.20031218072017.3749: * @file leoMenu.py
 """Gui-independent menu handling for Leo."""
-import leo.core.leoGlobals as g
-# import sys
+from typing import Any, List
+from leo.core import leoGlobals as g
 #@+others
 #@+node:ekr.20031218072017.3750: ** class LeoMenu
-class LeoMenu(object):
+class LeoMenu:
     """The base class for all Leo menus."""
     #@+others
     #@+node:ekr.20120124042346.12938: *3* LeoMenu.Birth
     def __init__(self, frame):
         self.c = frame.c
-        self.enable_dict = {} # Created by finishCreate.
+        self.enable_dict = {}  # Created by finishCreate.
         self.frame = frame
         self.isNull = False
-        self.menus = {} # Menu dictionary.
+        self.menus = {}  # Menu dictionary.
         self.menuShortcuts = {}
 
     def finishCreate(self):
@@ -92,13 +92,13 @@ class LeoMenu(object):
         }
 
         for i in range(1,9):
-            d ['expand-to-level-%s' % (i)] = lambda: c.p.hasChildren()
+            d [f"expand-to-level-{i}"] = lambda: c.p.hasChildren()
 
         if 0: # Initial testing.
             commandKeys = list(c.commandsDict.keys())
             for key in sorted(d.keys()):
                 if key not in commandKeys:
-                    g.trace('*** bad entry for %s' % (key))
+                    g.trace(f"*** bad entry for {key}")
     #@+node:ekr.20031218072017.3775: *3* LeoMenu.error and oops
     def oops(self):
         g.pr("LeoMenu oops:", g.callers(4), "should be overridden in subclass")
@@ -128,7 +128,7 @@ class LeoMenu(object):
         return ''.join(result)
     #@+node:ekr.20031218072017.3785: *4* LeoMenu.createMenusFromTables & helpers
     def createMenusFromTables(self):
-        '''(leoMenu) Usually over-ridden.'''
+        """(leoMenu) Usually over-ridden."""
         c = self.c
         aList = c.config.getMenusList()
         if aList:
@@ -137,26 +137,26 @@ class LeoMenu(object):
             g.es_print('No @menu setting found')
     #@+node:ekr.20070926135612: *5* LeoMenu.createMenusFromConfigList & helpers
     def createMenusFromConfigList(self, aList):
-        '''
+        """
         Create menus from aList.
         The 'top' menu has already been created.
-        '''
+        """
         # Called from createMenuBar.
         c = self.c
         for z in aList:
             kind, val, val2 = z
             if kind.startswith('@menu'):
-                name = kind[len('@menu'):].strip()
+                name = kind[len('@menu') :].strip()
                 if not self.handleSpecialMenus(name, parentName=None):
-                    # Fix #528: Don't create duplicate menu items.
+                    # #528: Don't create duplicate menu items.
+                    # Create top-level menu.
                     menu = self.createNewMenu(name)
-                        # Create top-level menu.
                     if menu:
                         self.createMenuFromConfigList(name, val, level=0)
                     else:
                         g.trace('no menu', name)
             else:
-                self.error('%s %s not valid outside @menu tree' % (kind, val))
+                self.error(f"{kind} {val} not valid outside @menu tree")
         aList = c.config.getOpenWith()
         if aList:
             # a list of dicts.
@@ -177,7 +177,10 @@ class LeoMenu(object):
         :param list aList: list of entries as described above
         """
         parentMenu = self.getMenu(parentName)
-        table = []
+        if not parentMenu:
+            g.trace('NO PARENT', parentName, g.callers())
+            return  # #2030
+        table: List[Any] = []
         for z in aList:
             kind, val, val2 = z
             if kind.startswith('@menu'):
@@ -186,17 +189,17 @@ class LeoMenu(object):
                 if table:
                     self.createMenuEntries(parentMenu, table)
                 if not self.handleSpecialMenus(name, parentName,
-                    alt_name=val2, #848.
+                    alt_name=val2,  #848.
                     table=table,
                 ):
+                    # Create submenu of parent menu.
                     menu = self.createNewMenu(name, parentName)
-                        # Create submenu of parent menu.
                     if menu:
                         # Partial fix for #528.
                         self.createMenuFromConfigList(name, val, level + 1)
                 table = []
             elif kind == '@item':
-                name = str(val) # Item names must always be ascii.
+                name = str(val)  # Item names must always be ascii.
                 if val2:
                     # Translated names can be unicode.
                     table.append((val2, name),)
@@ -208,144 +211,168 @@ class LeoMenu(object):
             self.createMenuEntries(parentMenu, table)
     #@+node:ekr.20070927172712: *6* LeoMenu.handleSpecialMenus
     def handleSpecialMenus(self, name, parentName, alt_name=None, table=None):
-        '''
+        """
         Handle a special menu if name is the name of a special menu.
         return True if this method handles the menu.
-        '''
+        """
         c = self.c
-        if table is None: table = []
+        if table is None:
+            table = []
         name2 = name.replace('&', '').replace(' ', '').lower()
         if name2 == 'plugins':
             # Create the plugins menu using a hook.
-            g.doHook("create-optional-menus", c=c)
+            g.doHook("create-optional-menus", c=c, menu_name=name)
             return True
-        elif name2.startswith('recentfiles'):
+        if name2.startswith('recentfiles'):
             # Just create the menu.
             # createRecentFilesMenuItems will create the contents later.
-            g.app.recentFilesManager.recentFilesMenuName = alt_name or name
-                #848
+            g.app.recentFilesManager.recentFilesMenuName = alt_name or name  # #848
             self.createNewMenu(alt_name or name, parentName)
             return True
-        elif name2 == 'help' and g.isMac:
+        if name2 == 'help' and g.isMac:
             helpMenu = self.getMacHelpMenu(table)
             return helpMenu is not None
-        else:
-            return False
+        return False
     #@+node:ekr.20031218072017.3780: *4* LeoMenu.hasSelection
     # Returns True if text in the outline or body text is selected.
 
     def hasSelection(self):
-        c = self.c; w = c.frame.body.wrapper
+        c = self.c
+        w = c.frame.body.wrapper
         if c.frame.body:
             first, last = w.getSelectionRange()
             return first != last
-        else:
-            return False
+        return False
     #@+node:ekr.20051022053758.1: *3* LeoMenu.Helpers
-    #@+node:ekr.20031218072017.3783: *4* LeoMenu.canonicalizeMenuName & cononicalizeTranslatedMenuName
+    #@+node:ekr.20031218072017.3783: *4* LeoMenu.canonicalize*
     def canonicalizeMenuName(self, name):
-        return ''.join([ch for ch in name.lower() if ch.isalnum()])
+
+        # #1121 & #1188. Allow Chinese characters in command names
+        if g.isascii(name):
+            return ''.join([ch for ch in name.lower() if ch.isalnum()])
+        return name
 
     def canonicalizeTranslatedMenuName(self, name):
-        return ''.join([ch for ch in name.lower() if ch not in '& \t\n\r'])
-    #@+node:ekr.20051022044950: *4* LeoMenu.computeOldStyleShortcutKey
-    def computeOldStyleShortcutKey(self, s):
-        '''Compute the old-style shortcut key for @shortcuts entries.'''
-        return ''.join([ch for ch in s.strip().lower() if ch.isalnum()])
+
+        # #1121 & #1188. Allow Chinese characters in command names
+        if g.isascii(name):
+            return ''.join([ch for ch in name.lower() if ch not in '& \t\n\r'])
+        return ''.join([ch for ch in name if ch not in '& \t\n\r'])
     #@+node:ekr.20031218072017.1723: *4* LeoMenu.createMenuEntries & helpers
-    def createMenuEntries(self, menu, table, dynamicMenu=False):
-        '''Create a menu entry from the table.
-        New in 4.4: this method shows the shortcut in the menu,
-        but this method **never** binds any shortcuts.'''
+    def createMenuEntries(self, menu, table):
+        """
+        Create a menu entry from the table.
+
+        This method shows the shortcut in the menu, but **never** binds any shortcuts.
+        """
         c = self.c
-        if g.app.unitTesting: return
-        if not menu: return
+        if g.unitTesting:
+            return
+        if not menu:
+            return
         self.traceMenuTable(table)
         for data in table:
             label, command, done = self.getMenuEntryInfo(data, menu)
-            if done: continue
-            commandName = self.getMenuEntryBindings(command, dynamicMenu, label)
-            if not commandName: continue
-            masterMenuCallback = self.createMasterMenuCallback(
-                dynamicMenu, command, commandName)
+            if done:
+                continue
+            commandName = self.getMenuEntryBindings(command, label)
+            if not commandName:
+                continue
+            masterMenuCallback = self.createMasterMenuCallback(command, commandName)
             realLabel = self.getRealMenuName(label)
             amp_index = realLabel.find("&")
             realLabel = realLabel.replace("&", "")
             # c.add_command ensures that c.outerUpdate is called.
             c.add_command(menu, label=realLabel,
-                accelerator='', # The accelerator is now computed dynamically.
+                accelerator='',  # The accelerator is now computed dynamically.
                 command=masterMenuCallback,
                 commandName=commandName,
                 underline=amp_index)
     #@+node:ekr.20111102072143.10016: *5* LeoMenu.createMasterMenuCallback
-    def createMasterMenuCallback(self, dynamicMenu, command, commandName):
+    def createMasterMenuCallback(self, command, commandName):
+        """
+        Create a callback for the given args.
 
+        - If command is a string, it is treated as a command name.
+        - Otherwise, it should be a callable representing the actual command.
+        """
         c = self.c
 
-        def setWidget():
+        def getWidget():
+            """Carefully return the widget that has focus."""
             w = c.frame.getFocus()
             if w and g.isMac:
-                 # 2012/01/11: redirect (MacOS only).
+                # Redirect (MacOS only).
                 wname = c.widget_name(w)
                 if wname.startswith('head'):
                     w = c.frame.tree.edit_widget(c.p)
-            # 2015/05/14: return a wrapper if possible.
+            # Return a wrapper if possible.
             if not g.isTextWrapper(w):
                 w = getattr(w, 'wrapper', w)
             return w
 
-        if dynamicMenu:
-            if command:
+        if isinstance(command, str):
 
-                def masterDynamicMenuCallback(c=c, command=command):
-                    # 2012/01/07: set w here.
-                    w = setWidget()
-                    event = g.app.gui.create_key_event(c, w=w)
-                    return c.k.masterCommand(func=command, event=event)
+            def static_menu_callback():
+                event = g.app.gui.create_key_event(c, w=getWidget())
+                c.doCommandByName(commandName, event)
 
-                return masterDynamicMenuCallback
-            else:
-                g.internalError('no callback for dynamic menu item.')
+            return static_menu_callback
 
-                def dummyMasterMenuCallback():
-                    pass
+        # The command must be a callable.
+        if not callable(command):
 
-                return dummyMasterMenuCallback
-        else:
+            def dummy_menu_callback(event=None):
+                pass
 
-            def masterStaticMenuCallback(c=c, commandName=commandName):
-                # 2011/10/28: Use only the command name to dispatch the command.
-                # 2012/01/07: Bug fix: set w here.
-                w = setWidget()
-                event = g.app.gui.create_key_event(c, w=w)
-                return c.k.masterCommand(commandName=commandName, event=event)
+            g.trace(f"bad command: {command!r}", color='red')
+            return dummy_menu_callback
 
-            return masterStaticMenuCallback
+        # Create a command dynamically.
+
+        def dynamic_menu_callback():
+            event = g.app.gui.create_key_event(c, w=getWidget())
+            return c.doCommand(command, commandName, event)  # #1595
+
+        return dynamic_menu_callback
     #@+node:ekr.20111028060955.16568: *5* LeoMenu.getMenuEntryBindings
-    def getMenuEntryBindings(self, command, dynamicMenu, label):
-        '''Compute commandName from command.'''
+    def getMenuEntryBindings(self, command, label):
+        """Compute commandName from command."""
         c = self.c
-        if g.isString(command):
+        if isinstance(command, str):
             # Command is really a command name.
             commandName = command
         else:
             # First, get the old-style name.
-            commandName = self.computeOldStyleShortcutKey(label)
+            # #1121: Allow Chinese characters in command names
+            commandName = label.strip()
         command = c.commandsDict.get(commandName)
         return commandName
     #@+node:ekr.20111028060955.16565: *5* LeoMenu.getMenuEntryInfo
     def getMenuEntryInfo(self, data, menu):
+        """
+        Parse a single entry in the table passed to createMenuEntries.
+
+        Table entries have the following formats:
+
+        1. A string, used as the command name.
+        2. A 2-tuple: (command_name, command_func)
+        3. A 3-tuple: (command_name, menu_shortcut, command_func)
+
+        Special case: If command_name is None or "-" it represents a menu separator.
+        """
         done = False
-        if g.isString(data):
+        if isinstance(data, str):
             # A single string is both the label and the command.
             s = data
             removeHyphens = s and s[0] == '*'
-            if removeHyphens: s = s[1:]
+            if removeHyphens:
+                s = s[1:]
             label = self.capitalizeMinibufferMenuName(s, removeHyphens)
             command = s.replace('&', '').lower()
             if label == '-':
                 self.add_separator(menu)
-                done = True # That's all.
+                done = True  # That's all.
         else:
             ok = isinstance(data, (list, tuple)) and len(data) in (2, 3)
             if ok:
@@ -357,16 +384,17 @@ class LeoMenu(object):
                     label, junk, command = data
                 if label in (None, '-'):
                     self.add_separator(menu)
-                    done = True # That's all.
+                    done = True  # That's all.
             else:
-                g.trace('bad data in menu table: %s' % repr(data))
-                done = True # Ignore bad data
+                g.trace(f"bad data in menu table: {repr(data)}")
+                done = True  # Ignore bad data
         return label, command, done
     #@+node:ekr.20111028060955.16563: *5* LeoMenu.traceMenuTable
     def traceMenuTable(self, table):
 
         trace = False and not g.unitTesting
-        if not trace: return
+        if not trace:
+            return
         format = '%40s %s'
         g.trace('*' * 40)
         for data in table:
@@ -380,7 +408,7 @@ class LeoMenu(object):
             else:
                 print(format % (data, ''))
     #@+node:ekr.20031218072017.3784: *4* LeoMenu.createMenuItemsFromTable
-    def createMenuItemsFromTable(self, menuName, table, dynamicMenu=False):
+    def createMenuItemsFromTable(self, menuName, table):
 
         if g.app.gui.isNullGui:
             return
@@ -388,7 +416,7 @@ class LeoMenu(object):
             menu = self.getMenu(menuName)
             if menu is None:
                 return
-            self.createMenuEntries(menu, table, dynamicMenu=dynamicMenu)
+            self.createMenuEntries(menu, table)
         except Exception:
             g.es_print("exception creating items for", menuName, "menu")
             g.es_exception()
@@ -396,34 +424,34 @@ class LeoMenu(object):
     #@+node:ekr.20031218072017.3804: *4* LeoMenu.createNewMenu
     def createNewMenu(self, menuName, parentName="top", before=None):
         try:
-            parent = self.getMenu(parentName) # parent may be None.
+            parent = self.getMenu(parentName)  # parent may be None.
             menu = self.getMenu(menuName)
             if menu:
                 # Not an error.
                 # g.error("menu already exists:", menuName)
-                return None # Fix #528.
+                return None  # Fix #528.
+            menu = self.new_menu(parent, tearoff=0, label=menuName)
+            self.setMenu(menuName, menu)
+            label = self.getRealMenuName(menuName)
+            amp_index = label.find("&")
+            label = label.replace("&", "")
+            if before:  # Insert the menu before the "before" menu.
+                index_label = self.getRealMenuName(before)
+                amp_index = index_label.find("&")
+                index_label = index_label.replace("&", "")
+                index = parent.index(index_label)
+                self.insert_cascade(
+                    parent, index=index, label=label, menu=menu, underline=amp_index)
             else:
-                menu = self.new_menu(parent, tearoff=0, label=menuName)
-                self.setMenu(menuName, menu)
-                label = self.getRealMenuName(menuName)
-                amp_index = label.find("&")
-                label = label.replace("&", "")
-                if before: # Insert the menu before the "before" menu.
-                    index_label = self.getRealMenuName(before)
-                    amp_index = index_label.find("&")
-                    index_label = index_label.replace("&", "")
-                    index = parent.index(index_label)
-                    self.insert_cascade(parent, index=index, label=label, menu=menu, underline=amp_index)
-                else:
-                    self.add_cascade(parent, label=label, menu=menu, underline=amp_index)
-                return menu
+                self.add_cascade(parent, label=label, menu=menu, underline=amp_index)
+            return menu
         except Exception:
             g.es("exception creating", menuName, "menu")
             g.es_exception()
             return None
     #@+node:ekr.20031218072017.4116: *4* LeoMenu.createOpenWithMenuFromTable & helpers
     def createOpenWithMenuFromTable(self, table):
-        '''
+        """
         Table is a list of dictionaries, created from @openwith settings nodes.
 
         This menu code uses these keys:
@@ -436,10 +464,11 @@ class LeoMenu(object):
             'args':     the command-line arguments to be used to open the file.
             'ext':      the file extension.
             'kind':     the method used to open the file, such as subprocess.Popen.
-        '''
+        """
         k = self.c.k
-        if not table: return
-        g.app.openWithTable = table # Override any previous table.
+        if not table:
+            return
+        g.app.openWithTable = table  # Override any previous table.
         # Delete the previous entry.
         parent = self.getMenu("File")
         if not parent:
@@ -472,7 +501,7 @@ class LeoMenu(object):
             k.bindOpenWith(d)
     #@+node:ekr.20051022043608.1: *5* LeoMenu.createOpenWithMenuItemsFromTable & callback
     def createOpenWithMenuItemsFromTable(self, menu, table):
-        '''
+        """
         Create an entry in the Open with Menu from the table, a list of dictionaries.
 
         Each dictionary d has the following keys:
@@ -482,9 +511,10 @@ class LeoMenu(object):
         'kind':     not used here: used by efc.open_temp_file.
         'name':     menu label.
         'shortcut': optional menu shortcut.
-        '''
+        """
         c = self.c
-        if g.app.unitTesting: return
+        if g.unitTesting:
+            return
         for d in table:
             label = d.get('name')
             args = d.get('args', [])
@@ -500,11 +530,12 @@ class LeoMenu(object):
                     command=callback,
                     underline=underline)
     #@+node:ekr.20031218072017.4118: *6* LeoMenu.defineOpenWithMenuCallback
-    def defineOpenWithMenuCallback(self, d):
+    def defineOpenWithMenuCallback(self, d=None):
         # The first parameter must be event, and it must default to None.
 
         def openWithMenuCallback(event=None, self=self, d=d):
-            return self.c.openWith(d=d)
+            d1 = d.copy() if d else {}
+            return self.c.openWith(d=d1)
 
         return openWithMenuCallback
     #@+node:tbrown.20080509212202.7: *4* LeoMenu.deleteRecentFilesMenuItems
@@ -520,28 +551,6 @@ class LeoMenu(object):
             if menu:
                 self.destroy(menu)
                 self.destroyMenu(i)
-    #@+node:ekr.20031218072017.4117: *4* LeoMenu.defineMenuCallback
-    def defineMenuCallback(self, command, name, minibufferCommand):
-        c = self.c
-        if minibufferCommand:
-            # Create a dummy event as a signal to doCommand.
-            event = g.app.gui.create_key_event(c)
-            # The first parameter must be event, and it must default to None.
-
-            def minibufferMenuCallback(event=event, self=self, command=command, label=name):
-                c = self.c
-                return c.doCommand(command, label, event)
-
-            return minibufferMenuCallback
-        else:
-            # The first parameter must be event, and it must default to None.
-
-            def legacyMenuCallback(event=None, self=self, command=command, label=name):
-                c = self.c # 2012/03/04.
-                c.check_event(event)
-                return c.doCommand(command, label)
-
-            return legacyMenuCallback
     #@+node:ekr.20031218072017.3805: *4* LeoMenu.deleteMenu
     def deleteMenu(self, menuName):
         try:
@@ -607,6 +616,7 @@ class LeoMenu(object):
 
     def add_separator(self, menu):
         self.oops()
+
     # def bind (self,bind_shortcut,callback):
     #     self.oops()
 
@@ -619,16 +629,18 @@ class LeoMenu(object):
     def destroy(self, menu):
         self.oops()
 
-    def insert(self, menuName, position, label, command, underline=None): # New in Leo 4.4.3 a1
+    def insert(
+        self, menuName, position, label, command, underline=None):  # New in Leo 4.4.3 a1
         self.oops()
 
     def insert_cascade(self, parent, index, label, menu, underline):
         self.oops()
 
-    def new_menu(self, parent, tearoff=0, label=''): # 2010: added label arg for pylint.
-        self.oops(); return None
+    def new_menu(self, parent, tearoff=0, label=''):
+        # 2010: added label arg for pylint.
+        self.oops()
     #@+node:ekr.20031218072017.3810: *4* LeoMenu.9 Routines with new spellings
-    def activateMenu(self, menuName): # New in Leo 4.4b2.
+    def activateMenu(self, menuName):  # New in Leo 4.4b2.
         self.oops()
 
     def clearAccel(self, menu, name):
@@ -638,7 +650,7 @@ class LeoMenu(object):
         self.oops()
 
     def createOpenWithMenu(self, parent, label, index, amp_index):
-        self.oops(); return None
+        self.oops()
 
     def disableMenu(self, menu, name):
         self.oops()
@@ -661,8 +673,7 @@ class NullMenu(LeoMenu):
     #@+others
     #@+node:ekr.20050104094308: *3* ctor (NullMenu)
     def __init__(self, frame):
-        # Init the base class.
-        LeoMenu.__init__(self, frame)
+        super().__init__(frame)
         self.isNull = True
     #@+node:ekr.20050104094029: *3* oops
     def oops(self):
