@@ -1,8 +1,9 @@
 #@+leo-ver=5-thin
 #@+node:ekr.20031218072017.3439: * @file leoPlugins.py
-'''Classes relating to Leo's plugin architecture.'''
-import leo.core.leoGlobals as g
+"""Classes relating to Leo's plugin architecture."""
 import sys
+from typing import List
+from leo.core import leoGlobals as g
 # Define modules that may be enabled by default
 # but that mignt not load because imports may fail.
 optional_modules = [
@@ -12,13 +13,13 @@ optional_modules = [
 #@+others
 #@+node:ekr.20100908125007.6041: ** Top-level functions (leoPlugins.py)
 def init():
-    '''Init g.app.pluginsController.'''
+    """Init g.app.pluginsController."""
     g.app.pluginsController = LeoPluginsController()
 
 def registerHandler(tags, fn):
-    '''A wrapper so plugins can still call leoPlugins.registerHandler.'''
+    """A wrapper so plugins can still call leoPlugins.registerHandler."""
     return g.app.pluginsController.registerHandler(tags, fn)
-#@+node:ville.20090222141717.2: ** TryNext (exception)
+#@+node:ville.20090222141717.2: ** TryNext (Exception)
 class TryNext(Exception):
     """Try next hook exception.
 
@@ -29,11 +30,11 @@ class TryNext(Exception):
     """
 
     def __init__(self, *args, **kwargs):
-        Exception.__init__(self, *args, **kwargs)
+        super().__init__()
         self.args = args
         self.kwargs = kwargs
 #@+node:ekr.20100908125007.6033: ** class CommandChainDispatcher
-class CommandChainDispatcher(object):
+class CommandChainDispatcher:
     """ Dispatch calls to a chain of commands until some func can handle it
 
     Usage: instantiate, execute "add" to add commands (with optional
@@ -70,8 +71,6 @@ class CommandChainDispatcher(object):
 
     def add(self, func, priority=0):
         """ Add a func to the cmd chain with given priority """
-        # Fails in Python 3: func is not orderable.
-        # bisect.insort(self.chain,(priority,func))
         self.chain.append((priority, func),)
         self.chain.sort(key=lambda z: z[0])
 
@@ -82,7 +81,7 @@ class CommandChainDispatcher(object):
         """
         return iter(self.chain)
 #@+node:ekr.20100908125007.6009: ** class BaseLeoPlugin
-class BaseLeoPlugin(object):
+class BaseLeoPlugin:
     #@+<<docstring>>
     #@+node:ekr.20100908125007.6010: *3* <<docstring>>
     """A Convenience class to simplify plugin authoring
@@ -97,7 +96,7 @@ class BaseLeoPlugin(object):
 
     - import the base class::
 
-        from leoPlugins import leo.core.leoBasePlugin as leoBasePlugin
+        from leoPlugins from leo.core import leoBasePlugin
 
     - create a class which inherits from leoBasePlugin::
 
@@ -106,7 +105,7 @@ class BaseLeoPlugin(object):
     - in the __init__ method of the class, call the parent constructor::
 
         def __init__(self, tag, keywords):
-            leoBasePlugin.__init__(self, tag, keywords)
+            super().__init__(tag, keywords)
 
     - put the actual plugin code into a method; for this example, the work
       is done by myPlugin.handler()
@@ -157,7 +156,7 @@ class BaseLeoPlugin(object):
             def __init__(self, tag, keywords):
 
                 # call parent __init__
-                BaseLeoPlugin.__init__(self, tag, keywords)
+                super().__init__(tag, keywords)
 
                 # if the plugin object defines only one command,
                 # just give it a name. You can then create a button and menu entry
@@ -235,10 +234,10 @@ class BaseLeoPlugin(object):
             commandName = self.commandName
         else:
             if commandName not in self.commandNames:
-                raise NameError("setButton error, %s is not a commandName" % commandName)
+                raise NameError(f"setButton error, {commandName} is not a commandName")
         if color is None:
             color = 'grey'
-        script = "c.k.simulateCommand('%s')" % self.commandName
+        script = f"c.k.simulateCommand('{self.commandName}')"
         g.app.gui.makeScriptButton(
             self.c,
             args=None,
@@ -246,26 +245,25 @@ class BaseLeoPlugin(object):
             buttonText=buttonText, bg=color)
     #@-others
 #@+node:ekr.20100908125007.6007: ** class LeoPluginsController
-class LeoPluginsController(object):
-    '''The global plugins controller, g.app.pluginsController'''
+class LeoPluginsController:
+    """The global plugins controller, g.app.pluginsController"""
     #@+others
     #@+node:ekr.20100909065501.5954: *3* plugins.Birth
     #@+node:ekr.20100908125007.6034: *4* plugins.ctor & reloadSettings
     def __init__(self):
 
+        # Keys are tags, values are lists of bunches.
         self.handlers = {}
+        # Keys are regularized module names, values are the names of .leo files
+        # containing @enabled-plugins nodes that caused the plugin to be loaded
         self.loadedModulesFilesDict = {}
-            # Keys are regularized module names, values are the names of .leo files
-            # containing @enabled-plugins nodes that caused the plugin to be loaded
+        # Keys are regularized module names, values are modules.
         self.loadedModules = {}
-            # Keys are regularized module names, values are modules.
+        # The stack of module names. The top is the module being loaded.
         self.loadingModuleNameStack = []
-            # The stack of module names.
-            # The top is the module being loaded.
-        self.signonModule = None # A hack for plugin_signon.
+        self.signonModule = None  # A hack for plugin_signon.
         # Settings.  Set these here in case finishCreate is never called.
         self.warn_on_failure = True
-        assert(g)
         g.act_on_node = CommandChainDispatcher()
         g.visit_tree_item = CommandChainDispatcher()
         g.tree_popup_handlers = []
@@ -274,12 +272,12 @@ class LeoPluginsController(object):
         self.reloadSettings()
 
     def reloadSettings(self):
-        self.warn_on_failure = \
-            g.app.config.getBool('warn_when_plugins_fail_to_load', default=True)
+        self.warn_on_failure = g.app.config.getBool(
+            'warn_when_plugins_fail_to_load', default=True)
     #@+node:ekr.20100909065501.5952: *3* plugins.Event handlers
     #@+node:ekr.20161029060545.1: *4* plugins.on_idle
     def on_idle(self):
-        '''Call all idle-time hooks.'''
+        """Call all idle-time hooks."""
         if g.app.idle_time_hooks_enabled:
             for frame in g.app.windowList:
                 c = frame.c
@@ -308,7 +306,7 @@ class LeoPluginsController(object):
         return None
     #@+node:ekr.20100908125007.6016: *5* plugins.callTagHandler
     def callTagHandler(self, bunch, tag, keywords):
-        '''Call the event handler.'''
+        """Call the event handler."""
         handler, moduleName = bunch.fn, bunch.moduleName
         # Make sure the new commander exists.
         for key in ('c', 'new_c'):
@@ -323,16 +321,16 @@ class LeoPluginsController(object):
         try:
             result = handler(tag, keywords)
         except Exception:
-            g.es("hook failed: %s, %s, %s" % (tag, handler, moduleName))
+            g.es(f"hook failed: {tag}, {handler}, {moduleName}")
             g.es_exception()
             result = None
         self.loadingModuleNameStack.pop()
         return result
     #@+node:ekr.20100908125007.6018: *4* plugins.doPlugins (g.app.hookFunction)
     def doPlugins(self, tag, keywords):
-        '''The default g.app.hookFunction.'''
+        """The default g.app.hookFunction."""
         if g.app.killed:
-            return
+            return None
         if tag in ('start1', 'open0'):
             self.loadHandlers(tag, keywords)
         return self.doHandlersForTag(tag, keywords)
@@ -345,12 +343,10 @@ class LeoPluginsController(object):
                 aList = self.getHandlersForOneTag(tag)
                 result.extend(aList)
             return result
-        else:
-            return self.getHandlersForOneTag(tags)
+        return self.getHandlersForOneTag(tags)
 
     def getHandlersForOneTag(self, tag):
-        aList = self.handlers.get(tag, [])
-        return aList
+        return self.handlers.get(tag, [])
     #@+node:ekr.20100910075900.10204: *4* plugins.getLoadedPlugins
     def getLoadedPlugins(self):
         return list(self.loadedModules.keys())
@@ -361,151 +357,155 @@ class LeoPluginsController(object):
     def isLoaded(self, fn):
         return self.regularizeName(fn) in self.loadedModules
     #@+node:ekr.20100908125007.6025: *4* plugins.printHandlers
-    def printHandlers(self, c, moduleName=None):
-        '''Print the handlers for each plugin.'''
+    def printHandlers(self, c):
+        """Print the handlers for each plugin."""
         tabName = 'Plugins'
         c.frame.log.selectTab(tabName)
-        if moduleName:
-            s = 'handlers for %s...\n' % (moduleName)
-        else:
-            s = 'all plugin handlers...\n'
-        g.es(s + '\n', tabName=tabName)
+        g.es_print('all plugin handlers...\n', tabName=tabName)
         data = []
-        modules = {}
+        # keys are module names: values are lists of tags.
+        modules_d: dict[str, List[str]] = {}
         for tag in self.handlers:
             bunches = self.handlers.get(tag)
             for bunch in bunches:
+                fn = bunch.fn
                 name = bunch.moduleName
-                tags = modules.get(name, [])
+                tags = modules_d.get(name, [])
                 tags.append(tag)
-                modules[name] = tags
+                key = f"{name}.{fn.__name__}"
+                modules_d[key] = tags
         n = 4
-        for key in sorted(modules):
-            tags = modules.get(key)
-            if moduleName in (None, key):
-                for tag in tags:
-                    n = max(n, len(tag))
-                    data.append((tag, key),)
-        lines = ['%*s %s\n' % (-n, s1, s2) for(s1, s2) in data]
-        g.es('', ''.join(lines), tabName=tabName)
+        for module in sorted(modules_d):
+            tags = modules_d.get(module)
+            for tag in tags:
+                n = max(n, len(tag))
+                data.append((tag, module),)
+        lines = sorted(list(set(
+            ["%*s %s\n" % (-n, s1, s2) for (s1, s2) in data])))
+        g.es_print('', ''.join(lines), tabName=tabName)
     #@+node:ekr.20100908125007.6026: *4* plugins.printPlugins
     def printPlugins(self, c):
-        '''Print all enabled plugins.'''
+        """Print all enabled plugins."""
         tabName = 'Plugins'
         c.frame.log.selectTab(tabName)
         data = []
         data.append('enabled plugins...\n')
         for z in sorted(self.loadedModules):
             data.append(z)
-        lines = ['%s\n' % (s) for s in data]
+        lines = [f"{z}\n" for z in data]
         g.es('', ''.join(lines), tabName=tabName)
     #@+node:ekr.20100908125007.6027: *4* plugins.printPluginsInfo
     def printPluginsInfo(self, c):
-        '''Print the file name responsible for loading a plugin.
+        """
+        Print the file name responsible for loading a plugin.
 
         This is the first .leo file containing an @enabled-plugins node
-        that enables the plugin.'''
+        that enables the plugin.
+        """
         d = self.loadedModulesFilesDict
         tabName = 'Plugins'
         c.frame.log.selectTab(tabName)
-        data = []; n = 4
+        data = []
+        n = 4
         for moduleName in d:
             fileName = d.get(moduleName)
             n = max(n, len(moduleName))
             data.append((moduleName, fileName),)
-        lines = ['%*s %s\n' % (-n, s1, s2) for(s1, s2) in data]
+        lines = ["%*s %s\n" % (-n, s1, s2) for (s1, s2) in data]
         g.es('', ''.join(lines), tabName=tabName)
     #@+node:ekr.20100909065501.5949: *4* plugins.regularizeName
-    def regularizeName(self, fn):
-        '''Return the name used as a key to this modules dictionaries.'''
-        if not fn.endswith('.py'):
-            return fn
+    def regularizeName(self, moduleOrFileName):
+        """
+        Return the module name used as a key to this modules dictionaries.
+
+        We *must* allow .py suffixes, for compatibility with @enabled-plugins nodes.
+        """
+        if not moduleOrFileName.endswith('.py'):
+            # A module name. Return it unchanged.
+            return moduleOrFileName
         #
-        # Allow .leo/plugins
-        path = g.os_path_finalize_join('~', '.leo', 'plugins', fn)
-        if g.os_path_exists(path):
-            return fn[: -3]
-        # Return the default module for leo plugins.
-        return "leo.plugins." + fn[: -3]
-    #@+node:ekr.20100909104341.5979: *4* plugins.setLoaded
-    def setLoaded(self, fn, m):
-        self.loadedModules[self.regularizeName(fn)] = m
+        # 1880: The legacy code implictly assumed that os.path.dirname(fn) was empty!
+        #       The new code explicitly ignores any directories in the path.
+        fn = g.os_path_basename(moduleOrFileName)
+        return "leo.plugins." + fn[:-3]
     #@+node:ekr.20100909065501.5953: *3* plugins.Load & unload
     #@+node:ekr.20100908125007.6022: *4* plugins.loadHandlers
     def loadHandlers(self, tag, keys):
-        '''
+        """
         Load all enabled plugins.
 
         Using a module name (without the trailing .py) allows a plugin to
         be loaded from outside the leo/plugins directory.
-        '''
+        """
 
         def pr(*args, **keys):
-            if not g.app.unitTesting:
+            if not g.unitTesting:
                 g.es_print(*args, **keys)
 
         s = g.app.config.getEnabledPlugins()
-        if not s: return
+        if not s:
+            return
         if tag == 'open0' and not g.app.silentMode and not g.app.batchMode:
             if 0:
-                s2 = '@enabled-plugins found in %s' % (
-                    g.app.config.enabledPluginsFileName)
+                s2 = f"@enabled-plugins found in {g.app.config.enabledPluginsFileName}"
                 g.blue(s2)
         for plugin in s.splitlines():
             if plugin.strip() and not plugin.lstrip().startswith('#'):
                 self.loadOnePlugin(plugin.strip(), tag=tag)
     #@+node:ekr.20100908125007.6024: *4* plugins.loadOnePlugin & helper functions
     def loadOnePlugin(self, moduleOrFileName, tag='open0', verbose=False):
-        '''
+        """
         Load one plugin from a file name or module.
         Use extensive tracing if --trace-plugins is in effect.
 
         Using a module name allows plugins to be loaded from outside the leo/plugins directory.
-        '''
+        """
         global optional_modules
         trace = 'plugins' in g.app.debug
 
         def report(message):
             if trace and not g.unitTesting:
-                g.es_print('loadOnePlugin: %s' % message)
-                
+                g.es_print(f"loadOnePlugin: {message}")
+
         # Define local helper functions.
         #@+others
         #@+node:ekr.20180528160855.1: *5* function:callInitFunction
         def callInitFunction(result):
-            '''True to call the top-level init function.'''
+            """True to call the top-level init function."""
             try:
                 # Indicate success only if init_result is True.
+                # Careful: this may throw an exception.
                 init_result = result.init()
-                    # Careful: this may throw an exception.
                 if init_result not in (True, False):
-                    report('%s.init() did not return a bool' % moduleName)
+                    report(f"{moduleName}.init() did not return a bool")
                 if init_result:
                     self.loadedModules[moduleName] = result
-                    self.loadedModulesFilesDict[moduleName] = g.app.config.enabledPluginsFileName
+                    self.loadedModulesFilesDict[moduleName] = (
+                        g.app.config.enabledPluginsFileName
+                    )
                 else:
-                    report('%s.init() returned False' % moduleName)
+                    report(f"{moduleName}.init() returned False")
                     result = None
             except Exception:
-                report('exception loading plugin: %s' % moduleName)
+                report(f"exception loading plugin: {moduleName}")
                 g.es_exception()
                 result = None
             return result
         #@+node:ekr.20180528162604.1: *5* function:finishImport
         def finishImport(result):
-            '''Handle last-minute checks.'''
+            """Handle last-minute checks."""
             if tag == 'unit-test-load':
-                return result # Keep the result, but do no more.
+                return result  # Keep the result, but do no more.
             if hasattr(result, 'init'):
                 return callInitFunction(result)
             #
             # No top-level init function.
-            if g.app.unitTesting:
+            if g.unitTesting:
                 # Do *not* load the module.
                 self.loadedModules[moduleName] = None
                 return None
             # Guess that the module was loaded correctly.
-            report('fyi: no top-level init() function in %s' % moduleName)
+            report(f"fyi: no top-level init() function in {moduleName}")
             self.loadedModules[moduleName] = result
             return result
         #@+node:ekr.20180528160744.1: *5* function:loadOnePluginHelper
@@ -516,20 +516,20 @@ class LeoPluginsController(object):
                 # Look up through sys.modules, __import__ returns toplevel package
                 result = sys.modules[moduleName]
             except g.UiTypeException:
-                report('plugin %s does not support %s gui' % (moduleName, g.app.gui.guiName()))
+                report(f"plugin {moduleName} does not support {g.app.gui.guiName()} gui")
             except ImportError:
-                report('error importing plugin: %s' % moduleName)
+                report(f"error importing plugin: {moduleName}")
             # except ModuleNotFoundError:
                 # report('module not found: %s' % moduleName)
             except SyntaxError:
-                report('syntax error importing plugin: %s' % moduleName)
+                report(f"syntax error importing plugin: {moduleName}")
             except Exception:
-                report('exception importing plugin: %s' % moduleName)
+                report(f"exception importing plugin: {moduleName}")
                 g.es_exception()
             return result
         #@+node:ekr.20180528162300.1: *5* function:reportFailedImport
         def reportFailedImport():
-            '''Report a failed import.'''
+            """Report a failed import."""
             if g.app.batchMode or g.app.inBridge or g.unitTesting:
                 return
             if (
@@ -538,14 +538,13 @@ class LeoPluginsController(object):
                 not g.app.gui.guiName().startswith('curses') and
                 moduleName not in optional_modules
             ):
-                report('can not load enabled plugin: %s' % moduleName)
+                report(f"can not load enabled plugin: {moduleName}")
         #@-others
-
         if not g.app.enablePlugins:
-            report('plugins disabled: %s' % moduleOrFileName)
+            report(f"plugins disabled: {moduleOrFileName}")
             return None
         if moduleOrFileName.startswith('@'):
-            report('ignoring Leo directive: %s' % moduleOrFileName)
+            report(f"ignoring Leo directive: {moduleOrFileName}")
             return None
                 # Return None, not False, to keep pylint happy.
                 # Allow Leo directives in @enabled-plugins nodes.
@@ -574,20 +573,22 @@ class LeoPluginsController(object):
         finally:
             self.loadingModuleNameStack.pop()
         if result:
-            report('loaded: %s' % moduleName)
-        self.signonModule = result # for self.plugin_signon.
+            # #1688: Plugins can update globalDirectiveList.
+            #        Recalculate g.directives_pat.
+            g.update_directives_pat()
+            report(f"loaded: {moduleName}")
+        self.signonModule = result  # for self.plugin_signon.
         return result
     #@+node:ekr.20031218072017.1318: *4* plugins.plugin_signon
     def plugin_signon(self, module_name, verbose=False):
-        '''Print the plugin signon.'''
+        """Print the plugin signon."""
         # This is called from as the result of the imports
         # in self.loadOnePlugin
         m = self.signonModule
         if verbose:
-            g.es('', "...%s.py v%s: %s" % (
-                m.__name__, m.__version__, g.plugin_date(m)))
+            g.es(f"...{m.__name__}.py v{m.__version__}: {g.plugin_date(m)}")
             g.pr(m.__name__, m.__version__)
-        self.signonModule = None # Prevent double signons.
+        self.signonModule = None  # Prevent double signons.
     #@+node:ekr.20100908125007.6030: *4* plugins.unloadOnePlugin
     def unloadOnePlugin(self, moduleOrFileName, verbose=False):
         moduleName = self.regularizeName(moduleOrFileName)
@@ -615,15 +616,16 @@ class LeoPluginsController(object):
             moduleName = self.loadingModuleNameStack[-1]
         except IndexError:
             moduleName = '<no module>'
-        if 0:
-            if g.app.unitTesting: g.pr('')
-            g.pr('%6s %15s %25s %s' % (g.app.unitTesting, moduleName, tag, fn.__name__))
-        if g.app.unitTesting: return
+        # print(f"{g.unitTesting:6} {moduleName:15} {tag:25} {fn.__name__}")
+        if g.unitTesting:
+            return
         if tag in self.handlers:
-            g.es("*** Two exclusive handlers for", "'%s'" % (tag))
+            g.es(f"*** Two exclusive handlers for '{tag}'")
         else:
             bunch = g.Bunch(fn=fn, moduleName=moduleName, tag='handler')
-            self.handlers[tag] = [bunch] # Vitalije
+            aList = self.handlers.get(tag, [])
+            aList.append(bunch)
+            self.handlers[tag] = aList
     #@+node:ekr.20100908125007.6029: *4* plugins.registerHandler & registerOneHandler
     def registerHandler(self, tags, fn):
         """ Register one or more handlers"""
@@ -639,12 +641,10 @@ class LeoPluginsController(object):
             moduleName = self.loadingModuleNameStack[-1]
         except IndexError:
             moduleName = '<no module>'
-        if 0:
-            if g.app.unitTesting: g.pr('')
-            g.pr('%6s %15s %25s %s' % (g.app.unitTesting, moduleName, tag, fn.__name__))
+        # print(f"{g.unitTesting:6} {moduleName:15} {tag:25} {fn.__name__}")
         items = self.handlers.get(tag, [])
         functions = [z.fn for z in items]
-        if fn not in functions: # Vitalije
+        if fn not in functions:  # Vitalije
             bunch = g.Bunch(fn=fn, moduleName=moduleName, tag='handler')
             items.append(bunch)
         self.handlers[tag] = items
@@ -665,4 +665,5 @@ class LeoPluginsController(object):
 #@@language python
 #@@tabwidth -4
 #@@pagewidth 70
+
 #@-leo
