@@ -1,8 +1,8 @@
 #@+leo-ver=5-thin
-#@+node:tbrown.20070322113635: * @file bookmarks.py
+#@+node:tbrown.20070322113635: * @file ../plugins/bookmarks.py
 #@+<< docstring >>
 #@+node:tbrown.20070322113635.1: ** << docstring >>
-''' Manage bookmarks in a list, and show bookmarks in a pane.
+""" Manage bookmarks in a list, and show bookmarks in a pane.
 
 This plugin has two bookmark related functions.  It manages nodes that
 contain bookmarks, and can also display those nodes in a special
@@ -208,47 +208,47 @@ apples, and autos), because the top level ``aardvarks`` tab is both a bookmark
 and an organizer node. If you want it to be just and organizer node, alt-click
 it to edit the bookmark node itself, and delete the body text (UNL) there.
 
-'''
+"""
 
 #@-<< docstring >>
 # Written by Terry Brown.
 #@+<< imports >>
 #@+node:tbrown.20070322113635.3: ** << imports >>
 from collections import namedtuple
-import leo.core.leoGlobals as g
-
-# Fail gracefully if the gui is not qt.
-g.assertUi('qt')
-from leo.core.leoQt import QtCore, QtWidgets
-
 import hashlib
-
+from leo.core import leoGlobals as g
+from leo.core.leoQt import isQt6, QtCore, QtWidgets
+from leo.core.leoQt import ControlType, KeyboardModifier, MouseButton, Orientation, Policy, QAction
+#
+# Fail fast, right after all imports.
+g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
 #@-<< imports >>
 #@+others
-#@+node:ekr.20100128073941.5371: ** init
+#@+node:ekr.20100128073941.5371: ** init (bookmarks.py)
 def init():
-    '''Return True if the plugin has loaded successfully.'''
-    ok = not g.unitTesting
-    if ok:
-        g.registerHandler('after-create-leo-frame', onCreate)
-        # temporary until double-click is bindable in user settings
-        if g.app.config.getBool('bookmarks-grab-dblclick'):
-            g.registerHandler('headdclick1', lambda t,k: cmd_open_bookmark(k))
-        g.plugin_signon(__name__)
-    return ok
+    """Return True if the plugin has loaded successfully."""
+    if g.unitTesting:
+        return False
+    g.registerHandler('after-create-leo-frame', onCreate)
+    # temporary until double-click is bindable in user settings
+    if g.app.config.getBool('bookmarks-grab-dblclick', default=False):
+        g.registerHandler('headdclick1', lambda t, k: cmd_open_bookmark(k))
+    g.plugin_signon(__name__)
+    return True
 
 #@+node:tbrown.20110712121053.19751: ** onCreate
 def onCreate(tag, keys):
 
     c = keys.get('c')
-
+    if not c:
+        return
     BookMarkDisplayProvider(c)
-
 #@+node:tbrown.20120319161800.21489: ** bookmarks-open-*
 @g.command('bookmarks-open-bookmark')
 def cmd_open_bookmark(event):
     c = event.get('c')
-    if not c: return
+    if not c:
+        return
     p = c.p
     bookmark = False
     for nd in p.parents():
@@ -263,23 +263,24 @@ def cmd_open_bookmark(event):
 @g.command('bookmarks-open-node')
 def cmd_open_node(event):
     c = event.get('c')
-    if not c: return
+    if not c:
+        return
     p = c.p
     url = g.getUrlFromNode(p)
     if url:
         # No need to handle url hooks here.
-        g.handleUrl(url,c=c,p=p)
+        g.handleUrl(url, c=c, p=p)
 
 #@+node:tbrown.20110712100955.39215: ** bookmarks-show
 @g.command('bookmarks-show')
 def cmd_show(event):
+
     c = event.get('c')
     bmd = BookMarkDisplay(c)
     # Careful: we could be unit testing.
     splitter = bmd.c.free_layout.get_top_splitter()
     if splitter:
         splitter.add_adjacent(bmd.w, 'bodyFrame', 'above')
-
 #@+node:tbrown.20131226095537.26309: ** bookmarks-switch
 @g.command('bookmarks-switch')
 def cmd_switch(event):
@@ -424,7 +425,7 @@ def cmd_bookmark_find_flat(event):
     for nd in nodes[:40]:
         new = container.insertAsLastChild()
         new.h = nd.h
-        new.b = c.vnode2position(nd).get_UNL(with_proto=True)
+        new.b = c.vnode2position(nd).get_UNL()
     bm.show_list(bm.get_list())
     if len(nodes) > 40:
         g.es("Stopped after 40 hits")
@@ -460,6 +461,7 @@ def cmd_mark_as_target(event):
     g.es("Node noted - now use\nbookmarks-use-other-outline\nin the "
         "outline you want to\nstore bookmarks in this node")
 
+#@+node:ekr.20190619132530.1: ** bookmarks-use-other-outline
 @g.command('bookmarks-use-other-outline')
 def cmd_use_other_outline(event):
     """Set bookmarks for this outline from a list (node) in
@@ -470,14 +472,12 @@ def cmd_use_other_outline(event):
         g.es("Use bookmarks-mark-as-target first")
         return
     c.db['_leo_bookmarks_show'] = g._bookmarks_target
-
     bmd = BookMarkDisplay(c, g._bookmarks_target_v)
-
     splitter = c.free_layout.get_top_splitter()
     if splitter:
         splitter.add_adjacent(bmd.w, 'bodyFrame', 'above')
 
-#@+node:ekr.20140917180536.17896: ** class FlowLayout
+#@+node:ekr.20140917180536.17896: ** class FlowLayout (QLayout)
 class FlowLayout(QtWidgets.QLayout):
     """
     from http://ftp.ics.uci.edu/pub/centos0/ics-custom-build/BUILD/
@@ -486,15 +486,14 @@ class FlowLayout(QtWidgets.QLayout):
     #@+others
     #@+node:ekr.20140917180536.17897: *3* __init__
     def __init__(self, parent=None, margin=0, spacing=-1):
-        '''Ctor for FlowLayout class.'''
-        super(FlowLayout, self).__init__(parent)
+        """Ctor for FlowLayout class."""
+        super().__init__(parent)
         if parent is not None:
             self.setMargin(margin)
         else:
             self.setMargin(0)
         self.setSpacing(spacing)
         self.itemList = []
-
     #@+node:ekr.20140917180536.17898: *3* __del__
     def __del__(self):
         item = self.takeAt(0)
@@ -508,7 +507,7 @@ class FlowLayout(QtWidgets.QLayout):
     #@+node:ekr.20140917180536.17900: *3* insertWidget
     def insertWidget(self, index, item):
         x = QtWidgets.QWidgetItem(item)
-        assert x # for pyflakes
+        assert x  # for pyflakes
         # item.setParent(x)
         # self.itemList.insert(index, x)
 
@@ -518,19 +517,26 @@ class FlowLayout(QtWidgets.QLayout):
 
     #@+node:ekr.20140917180536.17902: *3* itemAt
     def itemAt(self, index):
-        if index >= 0 and index < len(self.itemList):
+        if 0 <= index < len(self.itemList):
             return self.itemList[index]
         return None
-
     #@+node:ekr.20140917180536.17903: *3* takeAt
     def takeAt(self, index):
-        if index >= 0 and index < len(self.itemList):
+        if 0 <= index < len(self.itemList):
             return self.itemList.pop(index)
         return None
 
-    #@+node:ekr.20140917180536.17904: *3* expandingDirections
+    #@+node:ekr.20140917180536.17904: *3* expandingDirections (override)
     def expandingDirections(self):
-        return QtCore.Qt.Orientations(QtCore.Qt.Orientation(0))
+
+        """
+        Override of QLayout.expandingDirections.
+
+        Returns whether this layout can make use of more space than sizeHint().
+        A value of Qt::Vertical or Qt::Horizontal means that it wants to grow in only one dimension,
+        whereas Qt::Vertical | Qt::Horizontal means that it wants to grow in both dimensions.
+        """
+        return Orientation.Horizontal  # Best guess.
 
     #@+node:ekr.20140917180536.17905: *3* hasHeightForWidth
     def hasHeightForWidth(self):
@@ -543,7 +549,7 @@ class FlowLayout(QtWidgets.QLayout):
 
     #@+node:ekr.20140917180536.17907: *3* setGeometry
     def setGeometry(self, rect):
-        super(FlowLayout, self).setGeometry(rect)
+        super().setGeometry(rect)
         self.doLayout(rect, False)
 
     #@+node:ekr.20140917180536.17908: *3* sizeHint
@@ -561,20 +567,15 @@ class FlowLayout(QtWidgets.QLayout):
 
     #@+node:ekr.20140917180536.17910: *3* doLayout
     def doLayout(self, rect, testOnly):
-
         x = rect.x()
         y = rect.y()
         lineHeight = 0
         for item in self.itemList:
             wid = item.widget()
             spaceX = self.spacing() + wid.style().layoutSpacing(
-                QtWidgets.QSizePolicy.PushButton,
-                QtWidgets.QSizePolicy.PushButton,
-                QtCore.Qt.Horizontal)
+                ControlType.PushButton, ControlType.PushButton, Orientation.Horizontal)
             spaceY = self.spacing() + wid.style().layoutSpacing(
-                QtWidgets.QSizePolicy.PushButton,
-                QtWidgets.QSizePolicy.PushButton,
-                QtCore.Qt.Vertical)
+                ControlType.PushButton, ControlType.PushButton, Orientation.Vertical)
             nextX = x + item.sizeHint().width() + spaceX
             if nextX - spaceX > rect.right() and lineHeight > 0:
                 x = rect.x()
@@ -586,7 +587,6 @@ class FlowLayout(QtWidgets.QLayout):
             x = nextX
             lineHeight = max(lineHeight, item.sizeHint().height())
         return y + lineHeight - rect.y()
-
     #@+node:tbnorth.20160315104244.1: *3* margin
     def margin(self):
         """margin - return margin
@@ -606,22 +606,21 @@ class FlowLayout(QtWidgets.QLayout):
     #@-others
 
 #@+node:tbrown.20110712100955.18924: ** class BookMarkDisplay
-class BookMarkDisplay(object):
+class BookMarkDisplay:
     """Manage a pane showing bookmarks"""
-
     Bookmark = namedtuple('Bookmark', 'head url ancestors siblings children v')
 
-    # modifier to string mapping
     ModMap = {
-        int(QtCore.Qt.NoModifier): 'None',
-        int(QtCore.Qt.AltModifier): 'Alt',
-        int(QtCore.Qt.AltModifier | QtCore.Qt.ControlModifier): 'AltControl',
-        int(QtCore.Qt.AltModifier | QtCore.Qt.ControlModifier | \
-            QtCore.Qt.ShiftModifier): 'AltControlShift',
-        int(QtCore.Qt.AltModifier | QtCore.Qt.ShiftModifier): 'AltShift',
-        int(QtCore.Qt.ControlModifier): 'Control',
-        int(QtCore.Qt.ControlModifier | QtCore.Qt.ShiftModifier): 'ControlShift',
-        int(QtCore.Qt.ShiftModifier): 'Shift'
+        KeyboardModifier.NoModifier: 'None',
+        KeyboardModifier.AltModifier: 'Alt',
+        KeyboardModifier.AltModifier | KeyboardModifier.ControlModifier: 'AltControl',
+        (KeyboardModifier.AltModifier
+        | KeyboardModifier.ControlModifier
+        | KeyboardModifier.ShiftModifier): 'AltControlShift',
+        KeyboardModifier.AltModifier | KeyboardModifier.ShiftModifier: 'AltShift',
+        KeyboardModifier.ControlModifier: 'Control',
+        KeyboardModifier.ControlModifier | KeyboardModifier.ShiftModifier: 'ControlShift',
+        KeyboardModifier.ShiftModifier: 'Shift'
     }
 
     #@+others
@@ -654,7 +653,7 @@ class BookMarkDisplay(object):
         self.w.setObjectName('show_bookmarks')
         self.w.setMinimumSize(10, 10)
         self.w.setLayout(QtWidgets.QVBoxLayout())
-        self.w.layout().setContentsMargins(0,0,0,0)
+        self.w.layout().setContentsMargins(0, 0, 0, 0)
         self.current_list = self.get_list()
         self.show_list(self.current_list)
         g.registerHandler('select1', self.update)
@@ -686,13 +685,13 @@ class BookMarkDisplay(object):
         - `event`: click event
         - `bookmarks`: bookmarks in this pane
         """
-
-        if event.button() == QtCore.Qt.RightButton:
-            return self.context_menu(event, container=row_parent)
+        if event.button() == MouseButton.RightButton:
+            self.context_menu(event, container=row_parent)
+            return
 
         # Alt => edit bookmarks in the outline
         mods = event.modifiers()
-        if mods == QtCore.Qt.AltModifier:
+        if mods == KeyboardModifier.AltModifier:
             self.edit_bookmark(None, v=row_parent)
             return
         cmd_bookmark(event={'c': row_parent.context}, container=row_parent)
@@ -706,11 +705,11 @@ class BookMarkDisplay(object):
         - `bm`: Bookmark associated with button
         - `but`: button widget
         """
+        if event.button() == MouseButton.RightButton:
+            self.button_menu(event, bm, but, up=up)
+            return
 
-        if event.button() == QtCore.Qt.RightButton:
-            return self.button_menu(event, bm, but, up=up)
-
-        action_name = self.mod_map.get(self.ModMap.get(int(event.modifiers())))
+        action_name = self.mod_map.get(self.ModMap.get(event.modifiers()))
         if action_name is None:
             g.es("Bookmarks: unknown click type")
             print(int(event.modifiers()))
@@ -725,7 +724,7 @@ class BookMarkDisplay(object):
             # simple bookmark actions
             getattr(self, action_name)(bm)
             return
-        elif action_name == 'add_child':
+        if action_name == 'add_child':
             cmd_bookmark_child(event={'c': bm.v.context})
             return
 
@@ -750,7 +749,6 @@ class BookMarkDisplay(object):
         else:
             # don't leave focus adrift when clicking organizer node
             self.c.bodyWantsFocusNow()
-
     #@+node:tbrown.20140807091931.30231: *3* button_menu
     def button_menu(self, event, bm, but, up=False):
         """button_menu - handle a button being right-clicked
@@ -776,7 +774,7 @@ class BookMarkDisplay(object):
         ]
         for action in actions:
             # pylint: disable=cell-var-from-loop
-            act = QtWidgets.QAction(action[0], menu)
+            act = QAction(action[0], menu)
             act.triggered.connect(lambda checked, bm=bm, f=action[1]: f(bm))
             menu.addAction(act)
 
@@ -785,12 +783,13 @@ class BookMarkDisplay(object):
             manager.second = True
             manager.upwards = False
             manager.show_list(manager.get_list(), up=False)
-        act = QtWidgets.QAction("Show child bookmarks", menu)
+        act = QAction("Show child bookmarks", menu)
         act.triggered.connect(follow)
         menu.addAction(act)
 
-        menu.exec_(but.mapToGlobal(event.pos()))
-
+        point = event.position().toPoint() if isQt6 else event.pos()  # Qt6 documentation is wrong.
+        global_point = but.mapToGlobal(point)
+        menu.exec_(global_point)
     #@+node:tbnorth.20160830110146.1: *3* context_menu
     def context_menu(self, event, container=None):
         """context_menu
@@ -810,24 +809,22 @@ class BookMarkDisplay(object):
             # pylint: disable=cell-var-from-loop
             # pylint: disable=undefined-variable
             # weird: bm clearly *is* defined.
-            act = QtWidgets.QAction(action[0], menu)
+            act = QAction(action[0], menu)
             act.triggered.connect(lambda checked, bm=bm, f=action[1]: f(bm))
             menu.addAction(act)
 
-        menu.exec_(self.w.mapToGlobal(event.pos()))
-
+        point = event.position().toPoint() if isQt6 else event.pos()  # Qt6 documentation is wrong.
+        global_point = menu.mapToGlobal(point)
+        menu.exec_(global_point)
     #@+node:tbrown.20110712100955.18925: *3* color
     def color(self, text, dark=False):
         """make a consistent light background color for text"""
-
-        if g.isPython3:
-            text = g.toEncodedString(text,'utf-8')
+        text = g.toEncodedString(text, 'utf-8')
         x = hashlib.md5(text).hexdigest()[-6:]
-        add = int('bb',16) if not dark else int('33',16)
-        x = tuple([int(x[2*i:2*i+2], 16)//4+add for i in range(3)])
+        add = int('bb', 16) if not dark else int('33', 16)
+        x = tuple(int(x[2 * i : 2 * i + 2], 16) // 4 + add for i in range(3))
         x = '%02x%02x%02x' % x
         return x
-
     #@+node:tbrown.20131227100801.23856: *3* find_node
     def find_node(self, url):
         """find_node - Return position which is a bookmark for url, or None
@@ -867,7 +864,7 @@ class BookMarkDisplay(object):
         # v might not be in this outline
         p = self.v.context.vnode2position(self.v)
         if not p:
-            return
+            return None
 
         if levels is None:
             levels = self.levels
@@ -881,7 +878,8 @@ class BookMarkDisplay(object):
 
         def recurse_bm(node, result, ancestors=None):
 
-            if ancestors is None: ancestors = []
+            if ancestors is None:
+                ancestors = []
 
             for p in node.children():
 
@@ -905,7 +903,7 @@ class BookMarkDisplay(object):
                 if levels == 0:  # non-hierarchical
                     recurse_bm(p, result)
                 else:
-                    recurse_bm(p, children, ancestors=ancestors+[bm])
+                    recurse_bm(p, children, ancestors=ancestors + [bm])
 
         recurse_bm(p, result)
 
@@ -919,15 +917,13 @@ class BookMarkDisplay(object):
         :Parameters:
         - `p`: position to use instead of self.c.p
         """
-
         p = p or self.c.p
         c = p.v.context  # just in case it's not self.c
         if self.v.context == c:
             # local
-            return "#"+p.get_UNL(with_file=False, with_proto=False)
-        else:
-            # not local
-            return p.get_UNL(with_file=True, with_proto=True)
+            return "#" + p.get_UNL(with_file=False, with_proto=False)
+        # not local
+        return p.get_UNL(with_file=True, with_proto=True)
 
     #@+node:tbrown.20131227100801.23858: *3* show_list
     def show_list(self, links, up=False):
@@ -953,7 +949,6 @@ class BookMarkDisplay(object):
         current_url = None
         showing_chain = []
         row_parent = self.v
-
         while todo:
             links = todo.pop(0) if todo else []
             top = QtWidgets.QWidget()
@@ -962,12 +957,8 @@ class BookMarkDisplay(object):
             # pylint: disable=undefined-variable
             top.mouseReleaseEvent = (lambda event, links=links, row_parent=row_parent:
                 self.background_clicked(event, links, row_parent))
-            top.setMinimumSize(10,10)  # so there's something to click when empty
-
-            size_policy = QtWidgets.QSizePolicy(
-                QtWidgets.QSizePolicy.Expanding,
-                QtWidgets.QSizePolicy.Expanding
-            )
+            top.setMinimumSize(10, 10)  # so there's something to click when empty
+            size_policy = QtWidgets.QSizePolicy(Policy.Expanding, Policy.Expanding)
             size_policy.setHorizontalStretch(1)
             size_policy.setVerticalStretch(1)
             top.setSizePolicy(size_policy)
@@ -1041,7 +1032,7 @@ class BookMarkDisplay(object):
                 self.levels < self.w.layout().count()
             ):
                 # hide last line, of children, if none are current
-                self.w.layout().takeAt(self.w.layout().count()-1).widget().deleteLater()
+                self.w.layout().takeAt(self.w.layout().count() - 1).widget().deleteLater()
 
             while self.w.layout().count() > self.levels:
 
@@ -1068,7 +1059,7 @@ class BookMarkDisplay(object):
         """re-show the current list of bookmarks"""
 
         if keywords['c'] is not self.c:
-            return
+            return None
 
         self.show_list(self.get_list())
 
@@ -1080,17 +1071,18 @@ class BookMarkDisplay(object):
         c = bm.v.context
         p = c.vnode2position(bm.v)
         u = c.undoer
-        if p.hasVisBack(c): newNode = p.visBack(c)
-        else: newNode = p.next()
-        dirtyVnodeList = p.setAllAncestorAtFileNodesDirty()
+        if p.hasVisBack(c):
+            newNode = p.visBack(c)
+        else:
+            newNode = p.next()
+        p.setAllAncestorAtFileNodesDirty()
 
         undoData = u.beforeDeleteNode(p)
         if self.current == p.v:
             self.current = p.v.parents[0]
         p.doDelete(newNode)  # p is deleted, newNode is where to go afterwards
-        c.setChanged(True)
-        u.afterDeleteNode(newNode, "Bookmark deletion", undoData,
-            dirtyVnodeList=dirtyVnodeList)
+        c.setChanged()
+        u.afterDeleteNode(newNode, "Bookmark deletion", undoData)
         c.redraw()
         self.c.bodyWantsFocusNow()
 
@@ -1102,7 +1094,7 @@ class BookMarkDisplay(object):
         p = bm.v.context.vnode2position(bm.v)
         p.moveToFirstChildOf(p.parent())
         bm.v.setDirty()
-        bm.v.context.setChanged(True)
+        bm.v.context.setChanged()
         bm.v.context.redraw()
         bm.v.context.bodyWantsFocusNow()
         self.show_list(self.get_list())
@@ -1116,8 +1108,8 @@ class BookMarkDisplay(object):
 
         txt = g.app.gui.runAskOkCancelStringDialog(
             self.c,
-            "Rename "+bm.head,
-            "New name for "+bm.head,
+            "Rename " + bm.head,
+            "New name for " + bm.head,
             default=default
         )
 
@@ -1138,7 +1130,7 @@ class BookMarkDisplay(object):
         g.es("Bookmark updated")
         bm.v.b = new_url
         bm.v.setDirty()
-        bm.v.context.setChanged(True)
+        bm.v.context.setChanged()
         bm.v.context.redraw()
         bm.v.context.bodyWantsFocusNow()
         self.show_list(self.get_list())
@@ -1159,15 +1151,11 @@ class BookMarkDisplay(object):
     #@-others
 
 #@+node:tbrown.20110712121053.19746: ** class BookMarkDisplayProvider
-class BookMarkDisplayProvider(object):
+class BookMarkDisplayProvider:
     #@+others
     #@+node:tbrown.20110712121053.19747: *3* __init__
     def __init__(self, c):
         self.c = c
-
-        # if hasattr(c, 'free_layout') and hasattr(c.free_layout, 'get_top_splitter'):
-            # Second hasattr temporary until free_layout merges with trunk
-
         splitter = c.free_layout.get_top_splitter()
         # Careful: we could be unit testing.
         if splitter:
@@ -1175,7 +1163,7 @@ class BookMarkDisplayProvider(object):
 
     #@+node:tbrown.20110712121053.19748: *3* ns_provides
     def ns_provides(self):
-        return[('Bookmarks', '_leo_bookmarks_show')]
+        return [('Bookmarks', '_leo_bookmarks_show')]
 
     #@+node:tbrown.20110712121053.19749: *3* ns_provide
     def ns_provide(self, id_):
@@ -1202,24 +1190,25 @@ class BookMarkDisplayProvider(object):
                         other_c = c
                     if other_c != c:
                         # don't use c.bringToFront(), it breaks --minimize
-                        if hasattr(g.app.gui,'frameFactory'):
+                        if hasattr(g.app.gui, 'frameFactory'):
                             factory = g.app.gui.frameFactory
-                            if factory and hasattr(factory,'setTabForCommander'):
+                            if factory and hasattr(factory, 'setTabForCommander'):
                                 factory.setTabForCommander(c)
 
-                        g.es("NOTE: bookmarks for this outline\nare in a different outline:\n  '%s'"%file_)
+                        g.es("NOTE: bookmarks for this outline\nare in a different outline:\n  '%s'" % file_)
 
-                    ok, depth, other_p = g.recursiveUNLFind(UNL.split('-->'), other_c)
-                    if ok:
+                    other_p = g.findUNL(UNL.split('-->'), other_c)
+                    if other_p:
                         v = other_p.v
                     else:
-                        g.es("Couldn't find '%s'"%gnx)
+                        g.es("Couldn't find '%s'" % gnx)
 
             if v is None:
                 v = c.p.v
 
             bmd = BookMarkDisplay(self.c, v=v)
             return bmd.w
+        return None
 
     #@-others
 
