@@ -1,37 +1,26 @@
 #@+leo-ver=5-thin
 #@+node:tbrown.20171028115144.6: * @file ../plugins/editpane/editpane.py
-'''Support for the edit-pane-test-open command and window.'''
+"""Support for the edit-pane-test-open command and window."""
 #@+<<editpane.py imports>>
-#@+node:tbrown.20171028115438.1: ** <<editpane.py imports>>
-import leo.core.leoGlobals as g
-
+#@+node:tbrown.20171028115438.1: ** << editpane.py imports >>
+from collections import defaultdict
+from importlib import import_module
+import os
 try:
+    # pylint: disable=import-error
     # this can fix an issue with Qt Web views in Ubuntu
     from OpenGL import GL
-    assert GL # To keep pyflakes happy.
+    assert GL  # To keep pyflakes happy.
 except Exception:
     # but not need to stop if it doesn't work
     pass
-
-import os
-
-from collections import defaultdict
-
-# pylint: disable=no-name-in-module
-if g.isPython3:
-    StringTypes = str
-else:
-    from types import StringTypes
-
-from leo.core.leoQt import QtCore, QtWidgets, QtConst # QtGui
+from leo.core.leoQt import isQt6, QtCore, QtWidgets
+from leo.core.leoQt import QAction, ContextMenuPolicy, Orientation, Policy
+from leo.core.leoQt import WidgetAttribute  # 2347
+from leo.core import leoGlobals as g
+from leo.core import signal_manager
 if QtCore is not None:
     from leo.plugins.editpane.clicky_splitter import ClickySplitter
-
-if g.isPython3:
-    from importlib import import_module
-
-import leo.core.signal_manager as sig
-
 #@-<<editpane.py imports>>
 #@+others
 #@+node:tbrown.20171028115438.2: ** DBG
@@ -40,8 +29,7 @@ def DBG(text):
 
     :param str text: text to print
     """
-    print("LEP: %s" % text)
-
+    print(f"LEP: {text}")
 #@+node:tbrown.20171028115438.3: ** edit_pane_test_open
 def edit_pane_test_open(event):
     """Make a command for opening the editpane in free_layout.
@@ -52,16 +40,22 @@ def edit_pane_test_open(event):
 
     if not hasattr(c, '__edit_pane_test'):
         c.__edit_pane_test = True
+
+
         class MinimalDemoProvider:
+
             def ns_provides(self):
                 return [("Demo editor", "__demo_provider_minimal_slider")]
+
             def ns_provide(self, id_):
                 if id_ == "__demo_provider_minimal_slider":
                     w = LeoEditPane(c=c, mode='split')
                     return w
                 return None
+
             def ns_provider_id(self):
                 return "__demo_provider_minimal"
+
         c.free_layout.get_top_splitter().register_provider(MinimalDemoProvider())
 
     s = c.free_layout.get_top_splitter()
@@ -75,7 +69,7 @@ def edit_pane_csv(event):
         w = w.parent()
     w.insert(-1, LeoEditPane(c=c, show_control=False, lep_type='EDITOR-CSV'))
 #@+node:tbrown.20171028115438.4: ** class LeoEditPane
-class LeoEditPane(QtWidgets.QWidget):
+class LeoEditPane(QtWidgets.QWidget):  # type:ignore
     """
     Leo node body editor / viewer
     """
@@ -83,7 +77,7 @@ class LeoEditPane(QtWidgets.QWidget):
     #@+node:tbrown.20171028115438.5: *3* __init__
     def __init__(self, c=None, p=None, mode='edit', show_head=True, show_control=True,
         update=True, recurse=False, lep_type=None, *args, **kwargs):
-        """__init__ - bind to outline
+        """LeoEditPane.__init__ - bind to outline
 
         :param outline c: outline to bind to
         :param position p: initial position
@@ -97,18 +91,18 @@ class LeoEditPane(QtWidgets.QWidget):
         :param dict **kwargs: pass through
         """
         DBG("__init__ LEP")
-        super(LeoEditPane, self).__init__(*args, **kwargs)
-        self.setAttribute(QtConst.WA_DeleteOnClose)
+        super().__init__(*args, **kwargs)
+        self.setAttribute(WidgetAttribute.WA_DeleteOnClose)  # #2347.
 
         lep_type = lep_type or ['EDITOR', 'TEXT']
-        if isinstance(lep_type, StringTypes):
+        if isinstance(lep_type, str):
             lep_type = [lep_type]
         if len(lep_type) < 2:
             lep_type.append('TEXT')
 
         self.modules = []  # modules we collect widgets from
         self.widget_classes = []  # collected widgets
-        self.widget_for = defaultdict(lambda:[])  # widget by class.lep_type
+        self.widget_for = defaultdict(lambda: [])  # widget by class.lep_type
 
         self.c = c
         p = p or self.c.p
@@ -125,10 +119,10 @@ class LeoEditPane(QtWidgets.QWidget):
             recurse=recurse,
         )
 
-        self.track   = self.cb_track.isChecked()
-        self.update  = self.cb_update.isChecked()
+        self.track = self.cb_track.isChecked()
+        self.update = self.cb_update.isChecked()
         self.recurse = self.cb_recurse.isChecked()
-        self.goto    = self.cb_goto.isChecked()
+        self.goto = self.cb_goto.isChecked()
 
         for type_ in lep_type:
             self.set_widget(lep_type=type_)
@@ -141,7 +135,6 @@ class LeoEditPane(QtWidgets.QWidget):
             # ('bodykey2', self._after_body_key),
         ]
         self._register_handlers()
-
     #@+node:tbrown.20171028115438.6: *3* _add_checkbox
     def _add_checkbox(self, text, state_changed, tooltip, checked=True,
         enabled=True, button_label=True):
@@ -161,8 +154,10 @@ class LeoEditPane(QtWidgets.QWidget):
         if button_label:
             btn = QtWidgets.QPushButton(text, self)
             self.control.layout().addWidget(btn)
+
             def cb(checked, cbox=cbox, state_changed=state_changed):
                 state_changed(cbox.isChecked(), one_shot=True)
+
             btn.clicked.connect(cb)
             btn.setToolTip(tooltip)
         cbox.setChecked(checked)
@@ -171,7 +166,6 @@ class LeoEditPane(QtWidgets.QWidget):
         cbox.setToolTip(tooltip)
         self.control.layout().addItem(QtWidgets.QSpacerItem(20, 0))
         return cbox
-
     #@+node:tbrown.20171028115438.7: *3* _add_frame
     def _add_frame(self):
         """_add_frame - add a widget with a layout as a hiding target.
@@ -179,12 +173,11 @@ class LeoEditPane(QtWidgets.QWidget):
         i.e. a container we can hide / show easily"""
         w = QtWidgets.QWidget(self)
         self.layout().addWidget(w)
-        w.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
+        w.setSizePolicy(Policy.Expanding, Policy.Maximum)
         w.setLayout(QtWidgets.QHBoxLayout())
         w.layout().setContentsMargins(0, 0, 0, 0)
         w.layout().setSpacing(0)
         return w
-
     #@+node:tbrown.20171028115438.8: *3* _after_body_key
     def _after_body_key(self, v):
         """_after_body_key - after Leo selects another node
@@ -199,16 +192,10 @@ class LeoEditPane(QtWidgets.QWidget):
         :param dict keywords: c, p, etc.
         :return: None
         """
-
         p = self.c.vnode2position(v)
-
         DBG("after body key")
-
         #X if self.update:
         self.update_position(p)
-
-        return None
-
     #@+node:tbrown.20171028115438.9: *3* _after_select
     def _after_select(self, tag, keywords):
         """_after_select - after Leo selects another node
@@ -226,7 +213,6 @@ class LeoEditPane(QtWidgets.QWidget):
         if self.track:
             self.new_position(keywords['new_p'])
         return None
-
     #@+node:tbrown.20171028115438.10: *3* _before_select
     def _before_select(self, tag, keywords):
         """_before_select - before Leo selects another node
@@ -249,10 +235,9 @@ class LeoEditPane(QtWidgets.QWidget):
         DBG("before select")
 
         return None
-
     #@+node:tbrown.20171028115438.11: *3* _find_gnx_node
     def _find_gnx_node(self, gnx):
-        '''Return the first position having the given gnx.'''
+        """Return the first position having the given gnx."""
         if self.c.p.gnx == gnx:
             return self.c.p
         for p in self.c.all_unique_positions():
@@ -260,8 +245,7 @@ class LeoEditPane(QtWidgets.QWidget):
                 return p
         g.es("Edit/View pane couldn't find node")
         return None
-
-    #@+node:tbrown.20171028115438.12: *3* _register_handlers
+    #@+node:tbrown.20171028115438.12: *3* _register_handlers (editpane.py)
     def _register_handlers(self):
         """_register_handlers - attach to Leo signals
         """
@@ -269,10 +253,10 @@ class LeoEditPane(QtWidgets.QWidget):
         for hook, handler in self.handlers:
             g.registerHandler(hook, handler)
 
-        sig.connect(self.c, 'body_changed', self._after_body_key)
-
+        signal_manager.connect(self.c, 'body_changed', self._after_body_key)
     #@+node:tbrown.20171028115438.13: *3* _build_layout
-    def _build_layout(self, show_head=True, show_control=True, update=True, recurse=False):
+    def _build_layout(
+        self, show_head=True, show_control=True, update=True, recurse=False):
         """build_layout - build layout
         """
         DBG("build layout")
@@ -296,38 +280,37 @@ class LeoEditPane(QtWidgets.QWidget):
         # checkboxes
         txt = ",\ncheck to do this always"
         self.cb_track = self._add_checkbox("Track", self.change_track,
-            "Track the node selected in the tree"+txt)
+            "Track the node selected in the tree" + txt)
         self.cb_goto = self._add_checkbox("Goto", self.change_goto,
-            "Make the tree go to this node"+txt)
+            "Make the tree go to this node" + txt)
         self.cb_update = self._add_checkbox("Update", self.change_update,
-            "Update view to match changed node"+txt)
+            "Update view to match changed node" + txt)
         self.cb_recurse = self._add_checkbox("Recurse", self.change_recurse,
-            "Recursive view"+txt, checked=recurse)
+            "Recursive view" + txt, checked=recurse)
         # mode menu
         btn = self.btn_mode = QtWidgets.QPushButton("Mode", self)
         self.control.layout().addWidget(btn)
-        btn.setContextMenuPolicy(QtConst.CustomContextMenu)
+        btn.setContextMenuPolicy(ContextMenuPolicy.CustomContextMenu)
         btn.customContextMenuRequested.connect(  # right click
             lambda pnt: self.mode_menu())
         btn.clicked.connect(  # or left click
             lambda checked: self.mode_menu())
 
         # misc. menu
-        btn = self.control_menu_button = QtWidgets.QPushButton(u"More\u25BE", self)
+        btn = self.control_menu_button = QtWidgets.QPushButton("More\u25BE", self)
         self.control.layout().addWidget(btn)
-        btn.setContextMenuPolicy(QtConst.CustomContextMenu)
+        btn.setContextMenuPolicy(ContextMenuPolicy.CustomContextMenu)
         btn.customContextMenuRequested.connect(  # right click
             lambda pnt: self.misc_menu())
         btn.clicked.connect(  # or left click
             lambda checked: self.misc_menu())
 
         # padding
-        self.control.layout().addItem(
-            QtWidgets.QSpacerItem(0, 0, hPolicy=QtWidgets.QSizePolicy.Expanding))
+        self.control.layout().addItem(QtWidgets.QSpacerItem(0, 0, hPolicy=Policy.Expanding))
 
         # content
         self.splitter = ClickySplitter(self)
-        self.splitter.setOrientation(QtCore.Qt.Vertical)
+        self.splitter.setOrientation(Orientation.Vertical)
         self.layout().addWidget(self.splitter)
         self.edit_frame = self._add_frame()
         self.splitter.addWidget(self.edit_frame)
@@ -342,7 +325,8 @@ class LeoEditPane(QtWidgets.QWidget):
         self.line_edit.setText("test")
 
         # toggle control visibility
-        self.toggle_ctrl.clicked.connect(lambda checked: self.control.setVisible(not self.control.isVisible()))
+        self.toggle_ctrl.clicked.connect(
+            lambda checked: self.control.setVisible(not self.control.isVisible()))
     #@+node:tbrown.20171028115438.14: *3* header_visible
     @property
     def header_visible(self):
@@ -364,13 +348,11 @@ class LeoEditPane(QtWidgets.QWidget):
         self.goto = one_shot or bool(state)
         self.state_changed()
         self.goto = bool(state)
-
     #@+node:tbrown.20171028115438.19: *3* change_recurse
     def change_recurse(self, state, one_shot=False):
         self.recurse = one_shot or bool(state)
         self.state_changed()
         self.recurse = bool(state)
-
     #@+node:tbrown.20171028115438.20: *3* change_track
     def change_track(self, state, one_shot=False):
         self.track = one_shot or bool(state)
@@ -378,7 +360,6 @@ class LeoEditPane(QtWidgets.QWidget):
             p = self.c.p
             self.new_position(p)
         self.track = bool(state)
-
     #@+node:tbrown.20171028115438.21: *3* change_update
     def change_update(self, state, one_shot=False):
         self.update = one_shot or bool(state)
@@ -387,19 +368,17 @@ class LeoEditPane(QtWidgets.QWidget):
             if p is not None:
                 self.new_position(p)
         self.update = bool(state)
-
     #@+node:tbrown.20171028115438.22: *3* close
     def close(self):
         """close - clean up
         """
         do_close = QtWidgets.QWidget.close(self)
         if do_close:
-            sig.disconnect_all(self)
+            signal_manager.disconnect_all(self)
             DBG("unregister handlers\n")
             for hook, handler in self.handlers:
                 g.unregisterHandler(hook, handler)
         return do_close
-
     #@+node:tbrown.20171028115438.23: *3* edit_widget_focus
     def edit_widget_focus(self):
         """edit_widget_focus - edit widget got focus"""
@@ -410,14 +389,12 @@ class LeoEditPane(QtWidgets.QWidget):
     def get_position(self):
         """get_position - get current position"""
         return self._find_gnx_node(self.gnx)
-
     #@+node:tbrown.20171028115438.25: *3* goto_node
     def goto_node(self):
         """goto_node - goto node being edited / viewed"""
         p = self.get_position()
         if p and p != self.c.p:
             self.c.selectPosition(p)
-
     #@+node:tbrown.20171028115438.26: *3* load_modules
     def load_modules(self):
         """load_modules - load modules to find widgets
@@ -430,18 +407,12 @@ class LeoEditPane(QtWidgets.QWidget):
         modules = []
         for name in [i[0] for i in names if i[1].lower() == '.py']:
             try:
-                if g.isPython3:
-                    modules.append(import_module('leo.plugins.editpane.'+name))
-                else:
-                    try:
-                        exec ( "import %s" % name )  # parens for Python 3 syntax
-                        modules.append(locals()[name])
-                    except Exception:
-                        pass
-                DBG("Loaded module: %s" % name)
-            except ImportError as err:
-                DBG("Module not loaded (unmet dependencies?): %s" % name)
-
+                modules.append(import_module('leo.plugins.editpane.' + name))
+                DBG(f"Loaded module: {name}")
+            except ImportError as e:
+                DBG(
+                    f"{e.__class__.__name__}: "
+                    f"Module not loaded (unmet dependencies?): {name}")
         for module in modules:
             for key in dir(module):
                 value = getattr(module, key)
@@ -453,7 +424,6 @@ class LeoEditPane(QtWidgets.QWidget):
     #@+node:tbrown.20171028115438.27: *3* misc_menu
     def misc_menu(self):
         """build menu on Action button"""
-
         # info needed to separate edit and view widgets in self.widget_classes
         name_test_current = [
             ("Editor", lambda x: x.lep_type == 'EDITOR', self.edit_widget.__class__),
@@ -464,29 +434,39 @@ class LeoEditPane(QtWidgets.QWidget):
         for name, is_one, current in name_test_current:
             # list Editor widgets, then Viewer widgets
             for widget_class in [i for i in self.widget_classes if is_one(i)]:
+
                 def cb(checked, widget_class=widget_class):
                     self.set_widget(widget_class=widget_class)
-                act = QtWidgets.QAction("%s: %s" % (name, widget_class.lep_name), self)
+
+                act = QAction(f"{name}: {widget_class.lep_name}", self)
                 act.setCheckable(True)
                 act.setChecked(widget_class == current)
                 act.triggered.connect(cb)
                 menu.addAction(act)
-        menu.exec_(self.mapToGlobal(self.control_menu_button.pos()))
 
+        button = self.control_menu_button
+        point = button.position().toPoint() if isQt6 else button.pos()  # Qt6 documentation is wrong.
+        global_point = button.mapToGlobal(point)
+        menu.exec_(global_point)
     #@+node:tbrown.20171028115438.28: *3* mode_menu
     def mode_menu(self):
         """build menu on Action button"""
         menu = QtWidgets.QMenu()
-
         for mode in 'edit', 'view', 'split':
-            act = QtWidgets.QAction(mode.title(), self)
+            act = QAction(mode.title(), self)
+
             def cb(checked, self=self, mode=mode):
                 self.set_mode(mode)
+
             act.triggered.connect(cb)
             act.setCheckable(True)
             act.setChecked(mode == self.mode)
             menu.addAction(act)
-        menu.exec_(self.mapToGlobal(self.btn_mode.pos()))
+
+        button = self.btn_mode
+        point = button.position().toPoint() if isQt6 else button.pos()  # Qt6 documentation is wrong.
+        global_point = button.mapToGlobal(point)
+        menu.exec_(global_point)
 
     #@+node:tbrown.20171028115438.29: *3* new_position
     def new_position(self, p):
@@ -501,7 +481,6 @@ class LeoEditPane(QtWidgets.QWidget):
 
         self.new_position_edit(p)
         self.new_position_view(p)
-
     #@+node:tbrown.20171028115438.30: *3* new_position_edit
     def new_position_edit(self, p):
         """new_position_edit - update editor for new position
@@ -515,7 +494,6 @@ class LeoEditPane(QtWidgets.QWidget):
         DBG("new edit position")
         if self.mode != 'view':
             self.edit_widget.new_text(p.b)
-
     #@+node:tbrown.20171028115438.31: *3* new_position_view
     def new_position_view(self, p):
         """new_position_view - update viewer for new position
@@ -532,17 +510,16 @@ class LeoEditPane(QtWidgets.QWidget):
             else:
                 text = p.b
             self.view_widget.new_text(text)
-
     #@+node:tbrown.20171028115438.32: *3* text_changed
     def text_changed(self, new_text):
         """text_changed - node text changed by this LEP's editor"""
 
         # Update p.b
         p = self.get_position()
-        sig.lock(self)
+        signal_manager.lock(self)
         p.b = new_text  # triggers 'body_changed' signal from c
         self.update_position_view(p)  # as we're ignoring signals
-        sig.unlock(self)
+        signal_manager.unlock(self)
     #@+node:tbrown.20171028115438.33: *3* update_position
     def update_position(self, p):
         """update_position - update editor and view for current Leo position
@@ -559,7 +536,6 @@ class LeoEditPane(QtWidgets.QWidget):
             self.update_position_edit(p)
             if self.update:
                 self.update_position_view(p)
-
     #@+node:tbrown.20171028115438.34: *3* update_position_edit
     def update_position_edit(self, p):
         """update_position_edit - update editor for current position
@@ -573,7 +549,6 @@ class LeoEditPane(QtWidgets.QWidget):
         DBG("update edit position")
         if self.mode != 'view':
             self.edit_widget.update_text(p.b)
-
     #@+node:tbrown.20171028115438.35: *3* update_position_view
     def update_position_view(self, p):
         """update_position_view - update viewer for current position
@@ -591,11 +566,9 @@ class LeoEditPane(QtWidgets.QWidget):
             else:
                 text = p.b
             self.view_widget.update_text(text)
-
     #@+node:tbrown.20171028115438.36: *3* render
     def render(self, checked):
         pass
-
     #@+node:tbrown.20171028115438.37: *3* set_widget
     def set_widget(self, widget_class=None, lep_type='TEXT'):
         """set_widget - set edit or view widget
@@ -605,7 +578,8 @@ class LeoEditPane(QtWidgets.QWidget):
 
         if widget_class is None:
             widget_class = [i for i in self.widget_classes if i.lep_type == lep_type][0]
-        if hasattr(widget_class, 'lep_type') and widget_class.lep_type.startswith('EDITOR'):
+        if hasattr(
+            widget_class, 'lep_type') and widget_class.lep_type.startswith('EDITOR'):
             frame = self.edit_frame
             attr = 'edit_widget'
             update = self.new_position_edit
@@ -620,7 +594,6 @@ class LeoEditPane(QtWidgets.QWidget):
             frame.layout().itemAt(i).widget().setParent(None)
         frame.layout().addWidget(widget)
         update(self.get_position())
-
     #@+node:tbrown.20171028115438.38: *3* set_mode
     def set_mode(self, mode):
         """set_mode - change mode edit / view / split
@@ -629,14 +602,12 @@ class LeoEditPane(QtWidgets.QWidget):
 
         """
         self.mode = mode
-        self.btn_mode.setText(u"%s\u25BE" % mode.title())
+        self.btn_mode.setText(f"{mode.title()}\u25BE")
         self.state_changed()
-
     #@+node:tbrown.20171028115438.39: *3* state_changed
     def state_changed(self):
         """state_changed - control state has changed
         """
-
         if self.goto and self.get_position() != self.c.p:
             self.goto_node()
 
@@ -647,13 +618,10 @@ class LeoEditPane(QtWidgets.QWidget):
             self.edit_frame.hide()
             self.view_frame.show()
         else:
-           self.edit_frame.show()
-           self.view_frame.show()
+            self.edit_frame.show()
+            self.view_frame.show()
 
         self.update_position(self.c.p)
-
-
-
     #@-others
 #@-others
 #@@language python
