@@ -27,20 +27,32 @@ free-layout-zoom
 # Written by Terry Brown.
 #@+<< imports >>
 #@+node:tbrown.20110203111907.5520: ** << imports >> (free_layout.py)
-import leo.core.leoGlobals as g
-from leo.core.leoQt import QtWidgets, QtCore
-if QtWidgets:
-    from leo.plugins.nested_splitter import NestedSplitter
-        # NestedSplitterChoice
 import json
+from typing import Any, List
+from leo.core import leoGlobals as g
+#
+# Qt imports. May fail from the bridge.
+try:  # #1973
+    from leo.core.leoQt import QtWidgets
+    from leo.core.leoQt import MouseButton
+    from leo.plugins.nested_splitter import NestedSplitter  # NestedSplitterChoice
+except Exception:
+    QtWidgets = None  # type:ignore
+    MouseButton = None  # type:ignore
+    NestedSplitter = None  # type:ignore
+#
+# Do not call g.assertUi('qt') here. It's too early in the load process.
 #@-<< imports >>
 #@+others
 #@+node:tbrown.20110203111907.5521: ** free_layout:init
 def init():
-    '''Return True if the free_layout plugin can be loaded.'''
-    return g.app.gui.guiName() == "qt"
+    """Return True if the free_layout plugin can be loaded."""
+    return bool(NestedSplitter and g.app.gui.guiName() == "qt")
 #@+node:ekr.20110318080425.14389: ** class FreeLayoutController
-class FreeLayoutController(object):
+class FreeLayoutController:
+
+    #@+<< FreeLayoutController docstring >>
+    #@+node:ekr.20201013042712.1: *3* << FreeLayoutController docstring >>
     """Glue between Leo and the NestedSplitter gui widget.  All Leo aware
     code should be in here, none in NestedSplitter.
 
@@ -49,8 +61,7 @@ class FreeLayoutController(object):
     NestedSplitter uses as callbacks to populate splitter-handle context-menu
     and the empty pane Action button menu:
 
-    see (ctrl-click this URL)
-    file://{{g.getBaseDirectory(c)}}/LeoPyRef.leo#Code-->Qt%20gui-->@file%20../plugins/nested_splitter.py-->class%20NestedSplitter%20(QSplitter)-->register_provider
+    see nested_splitter.py-->class%20NestedSplitter%20(QSplitter)-->register_provider
 
     ns_provides
       tell NestedSplitter which Action button items we can provide
@@ -64,24 +75,34 @@ class FreeLayoutController(object):
       provide the advertised service when a splitter-handle context-menu
       item we advertised is selected
     """
+    #@-<< FreeLayoutController docstring >>
+    #@+<< define default_layout >>
+    #@+node:ekr.20201013042741.1: *3* << define default_layout >>
+    default_layout = {
+        'content': [
+            {
+                'content': [
+                    '_leo_pane:outlineFrame',
+                    '_leo_pane:logFrame',
+                ],
+                'orientation': 1,
+                'sizes': [509, 275],
+            },
+            '_leo_pane:bodyFrame',
+        ],
+        'orientation': 2,
+        'sizes': [216, 216],
+    }
+    #@-<< define default_layout >>
+
     #@+others
     #@+node:ekr.20110318080425.14390: *3*  flc.ctor
     def __init__(self, c):
-        '''Ctor for FreeLayoutController class.'''
-
-        # if hasattr(c,'free_layout'):
-            # return
+        """Ctor for FreeLayoutController class."""
         self.c = c
-        # c.free_layout = self
-            # To be removed
-        # g.registerHandler('after-create-leo-frame',self.bindControllers)
-        # attach to an outline
         g.registerHandler('after-create-leo-frame', self.init)
-        # now that the outline's set up (plugins etc.), load layout for
-        # outline, can't do that sooner as plugins must be loaded first
-        # to provide their widgets in panels etc.
+        # Plugins must be loaded first to provide their widgets in panels etc.
         g.registerHandler('after-create-leo-frame2', self.loadLayouts)
-        # self.init()
     #@+node:tbrown.20110203111907.5522: *3*  flc.init
     def init(self, tag, keys):
         """Attach to an outline and
@@ -96,13 +117,14 @@ class FreeLayoutController(object):
           for NestedSplitter
         """
         c = self.c
+        if not NestedSplitter:
+            return
         if c != keys.get('c'):
             return
-
         # Careful: we could be unit testing.
-        splitter = self.get_top_splitter() # A NestedSplitter.
+        splitter = self.get_top_splitter()  # A NestedSplitter.
         if not splitter:
-            return None
+            return
         # by default NestedSplitter's context menus are disabled, needed
         # once to globally enable them
         NestedSplitter.enabled = True
@@ -123,19 +145,21 @@ class FreeLayoutController(object):
         logWidget = splitter.findChild(QtWidgets.QFrame, "logFrame")
         logWidget._is_permanent = True
         # tag core Leo components (see ns_provides)
-        splitter.findChild(QtWidgets.QWidget, "outlineFrame")._ns_id = '_leo_pane:outlineFrame'
+        splitter.findChild(
+            QtWidgets.QWidget, "outlineFrame")._ns_id = '_leo_pane:outlineFrame'
         splitter.findChild(QtWidgets.QWidget, "logFrame")._ns_id = '_leo_pane:logFrame'
         splitter.findChild(QtWidgets.QWidget, "bodyFrame")._ns_id = '_leo_pane:bodyFrame'
         splitter.register_provider(self)
         splitter.splitterClicked_connect(self.splitter_clicked)
-    #@+node:tbrown.20120119080604.22982: *3* flc.embed (FreeLayoutController)
+    #@+node:tbrown.20120119080604.22982: *3* flc.embed
     def embed(self):
         """called from ns_do_context - embed layout in outline's
         @settings, an alternative to the Load/Save named layout system
         """
         # Careful: we could be unit testing.
         top_splitter = self.get_top_splitter()
-        if not top_splitter: return
+        if not top_splitter:
+            return
         c = self.c
         layout = top_splitter.get_saveable_layout()
         nd = g.findNodeAnywhere(c, "@data free-layout-layout")
@@ -143,23 +167,24 @@ class FreeLayoutController(object):
             settings = g.findNodeAnywhere(c, "@settings")
             if not settings:
                 settings = c.rootPosition().insertAfter()
-                settings.h = "@settings"
+                settings.h = "@settings"  # type:ignore
             nd = settings.insertAsNthChild(0)
         nd.h = "@data free-layout-layout"
         nd.b = json.dumps(layout, indent=4)
         nd = nd.parent()
         if not nd or nd.h != "@settings":
-            g.es("WARNING: @data free-layout-layout node is not " "under an active @settings node")
+            g.es(
+                "WARNING: @data free-layout-layout node is not " "under an active @settings node")
         c.redraw()
-    #@+node:ekr.20160424035257.1: *3* flc.get_main_splitter & helper
+    #@+node:ekr.20160424035257.1: *3* flc.get_main_splitter
     def get_main_splitter(self, w=None):
-        '''
+        """
         Return the splitter the main splitter, or None. The main splitter is a
         NestedSplitter that contains the body pane.
 
         Yes, the user could delete the secondary splitter but if so, there is
         not much we can do here.
-        '''
+        """
         top = self.get_top_splitter()
         if top:
             w = top.find_child(QtWidgets.QWidget, "bodyFrame")
@@ -168,15 +193,15 @@ class FreeLayoutController(object):
                     return w
                 w = w.parent()
         return None
-    #@+node:ekr.20160424035254.1: *3* flc.get_secondary_splitter & helper
+    #@+node:ekr.20160424035254.1: *3* flc.get_secondary_splitter
     def get_secondary_splitter(self):
-        '''
+        """
         Return the secondary splitter, if it exists. The secondary splitter
         contains the outline pane.
 
         Yes, the user could delete the outline pane, but if so, there is not
         much we can do here.
-        '''
+        """
         top = self.get_top_splitter()
         if top:
             w = top.find_child(QtWidgets.QWidget, 'outlineFrame')
@@ -187,16 +212,16 @@ class FreeLayoutController(object):
         return None
     #@+node:tbrown.20110621120042.22914: *3* flc.get_top_splitter
     def get_top_splitter(self):
-        '''Return the top splitter of c.frame.top.'''
+        """Return the top splitter of c.frame.top."""
         # Careful: we could be unit testing.
         f = self.c.frame
         if hasattr(f, 'top') and f.top:
-            return f.top.findChild(NestedSplitter).top()
-        else:
-            return None
+            child = f.top.findChild(NestedSplitter)
+            return child and child.top()
+        return None
     #@+node:ekr.20120419095424.9927: *3* flc.loadLayouts (sets wrap=True)
     def loadLayouts(self, tag, keys, reloading=False):
-        """loadLayouts - Load the outlines layout
+        """loadLayouts - Load the outline's layout
 
         :Parameters:
         - `tag`: from hook event
@@ -209,12 +234,16 @@ class FreeLayoutController(object):
         Useful if you want to temporarily switch to a different layout and then
         back, without having to remember the original layouts name.
         """
+        trace = 'layouts' in g.app.debug
         c = self.c
         if not (g.app and g.app.db):
-            return # Can happen when running from the Leo bridge.
-        d = g.app.db.get('ns_layouts') or {}
+            return  # Can happen when running from the Leo bridge.
         if c != keys.get('c'):
             return
+        d = g.app.db.get('ns_layouts') or {}
+        if trace:
+            g.trace(tag)
+            g.printObj(keys, tag="keys")
         layout = c.config.getData("free-layout-layout")
         if layout:
             layout = json.loads('\n'.join(layout))
@@ -234,23 +263,26 @@ class FreeLayoutController(object):
         if d:
             for name in sorted(d.keys()):
 
-                def func(event, c=c, d=d, name=name):
+                # pylint: disable=cell-var-from-loop
+
+                def func(event):
                     layout = d.get(name)
                     if layout:
-                        c.free_layout.get_top_splitter().load_layout(layout)
+                        c.free_layout.get_top_splitter().load_layout(c, layout)
                     else:
                         g.trace('no layout', name)
 
-                commandName = 'free-layout-load-%s' % name.strip().lower().replace(' ', '-')
+                name_s = name.strip().lower().replace(' ', '-')
+                commandName = f"free-layout-load-{name_s}"
                 c.k.registerCommand(commandName, func)
         # Careful: we could be unit testing or in the Leo bridge.
         if layout:
             splitter = c.free_layout.get_top_splitter()
             if splitter:
-                splitter.load_layout(layout)
+                splitter.load_layout(c, layout)
     #@+node:tbrown.20110628083641.11730: *3* flc.ns_context
     def ns_context(self):
-        ans = [
+        ans: List[Any] = [
             ('Embed layout', '_fl_embed_layout'),
             ('Save layout', '_fl_save_layout'),
         ]
@@ -263,24 +295,24 @@ class FreeLayoutController(object):
         ans.append(('Restore default layout', '_fl_restore_default:'))
         ans.append(('Help for this menu', '_fl_help:'))
         return ans
-    #@+node:tbrown.20110628083641.11732: *3* flc.ns_do_context (FreeLayoutController)
+    #@+node:tbrown.20110628083641.11732: *3* flc.ns_do_context
     def ns_do_context(self, id_, splitter, index):
+
+        c = self.c
         if id_.startswith('_fl_embed_layout'):
             self.embed()
             return True
         if id_.startswith('_fl_restore_default'):
-            self.get_top_splitter().load_layout(
-                {'content': [{'content': ['_leo_pane:outlineFrame',
-                 '_leo_pane:logFrame'], 'orientation': 1, 'sizes':
-                 [509, 275]}, '_leo_pane:bodyFrame'],
-                 'orientation': 2, 'sizes': [216, 216]})
+            self.get_top_splitter().load_layout(c, layout=self.default_layout)
         if id_.startswith('_fl_help'):
             self.c.putHelpFor(__doc__)
             # g.handleUrl("http://leoeditor.com/")
             return True
         if id_ == '_fl_save_layout':
             if self.c.config.getData("free-layout-layout"):
-                g.es("WARNING: embedded layout in @settings/@data free-layout-layout " "will override saved layout")
+                g.es("WARNING: embedded layout in")
+                g.es("@settings/@data free-layout-layout")
+                g.es("will override saved layout")
             layout = self.get_top_splitter().get_saveable_layout()
             name = g.app.gui.runAskOkCancelStringDialog(self.c,
                 title="Save layout",
@@ -295,17 +327,19 @@ class FreeLayoutController(object):
             return True
         if id_.startswith('_fl_load_layout:'):
             if self.c.config.getData("free-layout-layout"):
-                g.es("WARNING: embedded layout in @settings/@data free-layout-layout " "will override saved layout")
+                g.es("WARNING: embedded layout in")
+                g.es("@settings/@data free-layout-layout")
+                g.es("will override saved layout")
             name = id_.split(':', 1)[1]
             self.c.db['_ns_layout'] = name
             layout = g.app.db['ns_layouts'][name]
-            self.get_top_splitter().load_layout(layout)
+            self.get_top_splitter().load_layout(c, layout)
             return True
         if id_.startswith('_fl_delete_layout:'):
             name = id_.split(':', 1)[1]
-            if ('yes' == g.app.gui.runAskYesNoCancelDialog(self.c,
+            if ('yes' == g.app.gui.runAskYesNoCancelDialog(c,
                 "Really delete Layout?",
-                "Really permanently delete the layout '%s'?" % name)
+                f"Really permanently delete the layout '{name}'?")
             ):
                 d = g.app.db.get('ns_layouts', {})
                 del d[name]
@@ -326,7 +360,8 @@ class FreeLayoutController(object):
     def ns_provide(self, id_):
         if id_.startswith('_leo_tab:'):
             id_ = id_.split(':', 1)[1]
-            logTabWidget = self.get_top_splitter().find_child(QtWidgets.QWidget, "logTabWidget")
+            top = self.get_top_splitter()
+            logTabWidget = top.find_child(QtWidgets.QWidget, "logTabWidget")
             for n in range(logTabWidget.count()):
                 if logTabWidget.tabText(n) == id_:
                     w = logTabWidget.widget(n)
@@ -340,7 +375,7 @@ class FreeLayoutController(object):
             id_ = id_.split(':', 1)[1]
             w = self.get_top_splitter().find_child(QtWidgets.QWidget, id_)
             if w:
-                w.setHidden(False) # may be from Tab holder
+                w.setHidden(False)  # may be from Tab holder
                 w.setMinimumSize(20, 20)
             return w
         return None
@@ -348,11 +383,12 @@ class FreeLayoutController(object):
     def ns_provides(self):
         ans = []
         # list of things in tab widget
-        logTabWidget = self.get_top_splitter().find_child(QtWidgets.QWidget, "logTabWidget")
+        logTabWidget = self.get_top_splitter(
+            ).find_child(QtWidgets.QWidget, "logTabWidget")
         for n in range(logTabWidget.count()):
-            text = str(logTabWidget.tabText(n)) # not QString
+            text = str(logTabWidget.tabText(n))
             if text in ('Body', 'Tree'):
-                continue # handled below
+                continue  # handled below
             if text == 'Log':
                 # if Leo can't find Log in tab pane, it creates another
                 continue
@@ -373,7 +409,7 @@ class FreeLayoutController(object):
         :param bool release: was it a Press or Release event
         :param bool double: was it a double click event
         """
-        if not release or event.button() != QtCore.Qt.MidButton:
+        if not release or event.button() != MouseButton.MiddleButton:
             return
         if splitter.root.zoomed:  # unzoom if *any* handle clicked
             splitter.zoom_toggle()
@@ -394,8 +430,9 @@ class FreeLayoutController(object):
 #@+node:tbrown.20140524112944.32658: *3* @g.command free-layout-context-menu
 @g.command('free-layout-context-menu')
 def free_layout_context_menu(event):
-    """free_layout_context_menu - open free layout's context menu, using
-    the first divider of the top splitter for context, for now.
+    """
+    Open free layout's context menu, using the first divider of the top
+    splitter for context.
     """
     c = event.get('c')
     splitter = c.free_layout.get_top_splitter()
@@ -404,16 +441,18 @@ def free_layout_context_menu(event):
 #@+node:tbrown.20130403081644.25265: *3* @g.command free-layout-restore
 @g.command('free-layout-restore')
 def free_layout_restore(event):
-    """free_layout_restore - restore layout outline had when it was loaded
+    """
+    Restore layout outline had when it was loaded.
     """
     c = event.get('c')
     c.free_layout.loadLayouts('reload', {'c': c}, reloading=True)
 #@+node:tbrown.20131111194858.29876: *3* @g.command free-layout-load
 @g.command('free-layout-load')
 def free_layout_load(event):
-    """free_layout_load - load layout from menu
-    """
+    """Load layout from menu."""
     c = event.get('c')
+    if not c:
+        return
     d = g.app.db.get('ns_layouts', {})
     menu = QtWidgets.QMenu(c.frame.top)
     for k in d:
@@ -428,17 +467,16 @@ def free_layout_load(event):
     layouts = g.app.db.get('ns_layouts', {})
     layout = layouts.get(name)
     if layout:
-        c.free_layout.get_top_splitter().load_layout(layout)
+        c.free_layout.get_top_splitter().load_layout(c, layout)
 #@+node:tbrown.20140522153032.32658: *3* @g.command free-layout-zoom
 @g.command('free-layout-zoom')
 def free_layout_zoom(event):
-    """free_layout_zoom - (un)zoom the current pane.
-    """
+    """(un)zoom the current pane."""
     c = event.get('c')
     c.free_layout.get_top_splitter().zoom_toggle()
 #@+node:ekr.20160327060009.1: *3* free_layout:register_provider
 def register_provider(c, provider_instance):
-    '''Register the provider instance with the top splitter.'''
+    """Register the provider instance with the top splitter."""
     # Careful: c.free_layout may not exist during unit testing.
     if c and hasattr(c, 'free_layout'):
         splitter = c.free_layout.get_top_splitter()
