@@ -1,10 +1,10 @@
 #@+leo-ver=5-thin
-#@+node:ekr.20101110092851.5742: * @file leoOPML.py
+#@+node:ekr.20101110092851.5742: * @file ../plugins/leoOPML.py
 #@+<< docstring >>
 #@+node:ekr.20060904103412.1: ** << docstring >>
 #@@language rest
 
-r'''A plugin to read and write Leo outlines in .opml
+r"""A plugin to read and write Leo outlines in .opml
 (http://en.wikipedia.org/wiki/OPML) format.
 
 The OPML plugin creates two new commands that read and write Leo outlines in
@@ -44,8 +44,7 @@ Leo writes body text to the OPML file only if this is True.
 If True, Leo writes the native attributes of Leo's <v> elements as attributes of
 the opml <outline> elements.
 
-The native attributes of <v> elements are a, t, vtag (new), tnodeList,
-marks, expanded and descendentTnodeUnknownAttributes.
+FastRead.nativeVnodeAttributes defines the native attributes of <v> elements.
 
 - @bool opml_write_leo_globals_attributes = True
 
@@ -67,33 +66,29 @@ opml, if that feature is implemented in the future.
 
 If True, when expanding as above, skip blank dict entries.
 
-'''
+"""
 #@-<< docstring >>
 # 2014/10/21: support Android outliner by treating _note attributes as body text.
 # To do: read/write uA's.
-printElements = [] # ['all','outline','head','body',]
-    # For traces.
 #@+<< imports >>
 #@+node:ekr.20060904103412.3: ** << imports >>
-import leo.core.leoGlobals as g
-import leo.core.leoPlugins as leoPlugins
-import leo.core.leoNodes as leoNodes
-import string
+import io
 import xml.sax
 import xml.sax.saxutils
-if g.isPython3:
-    import io # Python 3.x
-    StringIO = io.StringIO
-    BytesIO = io.BytesIO
-else:
-    import cStringIO # Python 2.x
-    StringIO = cStringIO.StringIO
+from leo.core import leoGlobals as g
+from leo.core import leoNodes
+from leo.core import leoPlugins
+# Abbreviations.
+StringIO = io.StringIO
+BytesIO = io.BytesIO
 #@-<< imports >>
+printElements = []  # ['all','outline','head','body',]
+    # For traces.
 #@+others
 #@+node:ekr.20060904132527.9: ** Module level
 #@+node:ekr.20060904103412.4: *3* init
 def init():
-    '''Return True if the plugin has loaded successfully.'''
+    """Return True if the plugin has loaded successfully."""
     leoPlugins.registerHandler(('open2', 'new'), onCreate)
     g.plugin_signon(__name__)
     return True
@@ -103,12 +98,12 @@ def onCreate(tag, keys):
     if c:
         c.opmlController = OpmlController(c)
 #@+node:ekr.20060904141220: ** class NodeClass
-class NodeClass(object):
-    '''
+class NodeClass:
+    """
     A class representing one outline element.
 
     Use getters to access the attributes, properties and rules of this mode.
-    '''
+    """
     #@+others
     #@+node:ekr.20060904141220.1: *3*  node.__init__
     def __init__(self):
@@ -135,32 +130,32 @@ class NodeClass(object):
         print('attrs: %s' % self.attributes.values())
     #@-others
 #@+node:ekr.20060904103412.6: ** class OpmlController
-class OpmlController(object):
-    '''The controller class for this plugin.'''
+class OpmlController:
+    """The controller class for this plugin."""
     #@+others
     #@+node:ekr.20060904103412.7: *3* oc.__init__& reloadSettings
     def __init__(self, c):
-        '''Ctor for OpmlController class.'''
+        """Ctor for OpmlController class."""
         self.c = c
         c.opmlCommands = self
         c.k.registerCommand('read-opml-file', self.readOpmlCommand)
         c.k.registerCommand('write-opml-file', self.writeOpmlCommand)
         self.currentVnode = None
         self.topVnode = None
-        self.generated_gnxs = {} # Keys are gnx's (strings).  Values are vnodes.
+        self.generated_gnxs = {}  # Keys are gnx's (strings).  Values are vnodes.
         self.reloadSettings()
-        
+
     def reloadSettings(self):
         c = self.c
         c.registerReloadSettings(self)
-        self.opml_read_derived_files = c.config.getBool('opml-read-derived-files')
+        # self.opml_read_derived_files = c.config.getBool('opml-read-derived-files')
         self.opml_write_derived_files = c.config.getBool('opml-write-derived-files')
 
     #@+node:ekr.20060914163456: *3* oc.createVnodes & helpers
     def createVnodes(self, c, dummyRoot):
-        '''**Important**: this method and its helpers are low-level code
+        """**Important**: this method and its helpers are low-level code
         corresponding to link/unlink methods in leoNodes.py.
-        Modify this with extreme care.'''
+        Modify this with extreme care."""
         self.generated_gnxs = {}
         parent_v = c.hiddenRootNode
         parent_v.children = []
@@ -198,18 +193,14 @@ class OpmlController(object):
     def handleVnodeAttributes(self, node, v):
         a = node.attributes.get('leo:a')
         if a:
-            # 'C' (clone) and 'D' bits are not used.
-            if 'M' in a: v.setMarked()
-            if 'E' in a: v.expand()
-            # if 'O' in a: v.setOrphan()
-            if 'T' in a: self.topVnode = v
-            if 'V' in a: self.currentVnode = v
-        if 0: # Leo no longer uses the tnodeList.
-            s = node.attributes.get('leo:tnodeList')
-            tnodeList = s and s.split(',')
-            if tnodeList:
-                # This tnode list will be resolved later.
-                v.tempTnodeList = tnodeList
+            if 'M' in a:
+                v.setMarked()
+            if 'E' in a:
+                v.expand()
+            if 'T' in a:
+                self.topVnode = v
+            if 'V' in a:
+                self.currentVnode = v
     #@+node:ekr.20060913220707: *3* oc.dumpTree
     def dumpTree(self, root, dummy=True):
         if not dummy:
@@ -225,16 +216,13 @@ class OpmlController(object):
         path = g.os_path_normpath(g.os_path_join(g.app.loadDir, fn))
         try:
             f = open(path, 'rb')
-            s = f.read() # type(s) is bytes for Python 3.x.
+            s = f.read()  # type(s) is bytes for Python 3.x.
             s = self.cleanSaxInputString(s)
         except IOError:
             return g.trace('can not open %s' % path)
         # pylint:disable=catching-non-exception
         try:
-            if g.isPython3:
-                theFile = BytesIO(s)
-            else:
-                theFile = cStringIO.StringIO(s)
+            theFile = BytesIO(s)
             parser = xml.sax.make_parser()
             parser.setFeature(xml.sax.handler.feature_external_ges, 1)
             # Do not include external general entities.
@@ -242,7 +230,7 @@ class OpmlController(object):
             parser.setFeature(xml.sax.handler.feature_external_pes, 0)
             handler = SaxContentHandler(c, fn)
             parser.setContentHandler(handler)
-            parser.parse(theFile) # expat does not support parseString
+            parser.parse(theFile)  # expat does not support parseString
             sax_node = handler.getNode()
         except xml.sax.SAXParseException:
             g.error('error parsing', fn)
@@ -255,8 +243,8 @@ class OpmlController(object):
         return sax_node
     #@+node:ekr.20111003220434.15490: *4* oc.cleanSaxInputString
     def cleanSaxInputString(self, s):
-        '''Clean control characters from s.
-        s may be a bytes or a (unicode) string.'''
+        """Clean control characters from s.
+        s may be a bytes or a (unicode) string."""
         # Note: form-feed ('\f') is 12 decimal.
         badchars = [chr(ch) for ch in range(32)]
         badchars.remove('\t')
@@ -264,33 +252,28 @@ class OpmlController(object):
         badchars.remove('\n')
         flatten = ''.join(badchars)
         pad = ' ' * len(flatten)
-        # pylint:disable=no-member
-        if g.isPython3:
-            flatten = bytes(flatten, 'utf-8')
-            pad = bytes(pad, 'utf-8')
-            transtable = bytes.maketrans(flatten, pad)
-        else:
-            transtable = string.maketrans(flatten, pad)
+        flatten = bytes(flatten, 'utf-8')
+        pad = bytes(pad, 'utf-8')
+        transtable = bytes.maketrans(flatten, pad)
         return s.translate(transtable)
-    # for i in range(32): print i,repr(chr(i))
     #@+node:ekr.20141020112451.18342: *3* oc.putToOPML
     def putToOPML(self, owner):
-        '''
-        Write the c.p as OPML, using the owner's put method.'''
+        """
+        Write the c.p as OPML, using the owner's put method."""
         PutToOPML(owner)
-    #@+node:ekr.20060904103721: *3* oc.readFile & helpers
+    #@+node:ekr.20060904103721: *3* oc.readFile & helper
     def readFile(self, fileName):
-        '''Read the opml file.'''
+        """Read the opml file."""
         dumpTree = False
         if not fileName:
-            return g.trace('no fileName')
+            g.trace('no fileName')
+            return None
+        # Create the new commander *now*, so that created vnodes will have the proper context.
         c = self.c.new()
-            # Create the new commander *now*
-            # so that created vnodes will have the proper context.
         # Pass one: create the intermediate nodes.
         dummyRoot = self.parse_opml_file(fileName)
         if not dummyRoot:
-            return
+            return None
         if dumpTree:
             self.dumpTree(dummyRoot)
         # Pass two: create the outline from the sax nodes.
@@ -300,27 +283,11 @@ class OpmlController(object):
         errors = c.checkOutline()
         if errors:
             c.dumpOutline()
-            return g.trace('%s errors!' % errors)
-        # if self.opml_read_derived_files:
-            # at = c.atFileCommands
-            # c.fileCommands.tnodesDict = self.createTnodesDict()
-            # self.resolveTnodeLists(c)
-            # if self.opml_read_derived_files:
-                # c.atFileCommands.readAll(c.rootPosition())
+            g.trace('%s errors!' % errors)
+            return None
         c.selectPosition(p)
         c.redraw()
-        return c # for testing.
-    #@+node:ekr.20060921153603: *4* oc.createTnodesDict
-    def createTnodesDict(self):
-        '''
-        Create c.tnodesDict by from self.generated_gnxs
-        by converting VNode entries to tnodes.
-        '''
-        d = {}
-        for key in list(self.generated_gnxs.keys()):
-            v = self.generated_gnxs.get(key)
-            d[key] = v
-        return d
+        return c  # for testing.
     #@+node:ekr.20060917214140: *4* oc.setCurrentPosition
     def setCurrentPosition(self, c):
         v = self.currentVnode
@@ -330,22 +297,9 @@ class OpmlController(object):
             if p.v == v:
                 c.selectPosition(p)
                 break
-    #@+node:ekr.20060918132045: *4* oc.resolveTnodeLists
-    def resolveTnodeLists(self, c):
-        for p in c.allNodes_iter():
-            if hasattr(p.v, 'tempTnodeList'):
-                result = []
-                for gnx in p.v.tempTnodeList:
-                    v = self.generated_gnxs.get(gnx)
-                    if v:
-                        result.append(v)
-                    else:
-                        g.trace('No tnode for %s' % gnx)
-                p.v.tnodeList = result
-                delattr(p.v, 'tempTnodeList')
     #@+node:ekr.20060919201810: *3* oc.readOpmlCommand
     def readOpmlCommand(self, event=None):
-        '''Open a Leo window containing the contents of an .opml file.'''
+        """Open a Leo window containing the contents of an .opml file."""
         c = self.c
         fileName = g.app.gui.runOpenFileDialog(c,
             title="Read OPML",
@@ -358,7 +312,7 @@ class OpmlController(object):
             c.bodyWantsFocus()
     #@+node:ekr.20060904103721.1: *3* oc.writeFile
     def writeFile(self, fileName):
-        '''Write fileName as an OPML file.'''
+        """Write fileName as an OPML file."""
         if not fileName:
             return
         ok = self.c.fileCommands.write_Leo_file(
@@ -371,7 +325,7 @@ class OpmlController(object):
             g.es_print('did not write %s' % fileName)
     #@+node:ekr.20060919201330: *3* oc.writeOpmlCommand
     def writeOpmlCommand(self, event=None):
-        '''Save a Leo outline to an OPMLfile.'''
+        """Save a Leo outline to an OPMLfile."""
         c = self.c
         if g.app.disableSave:
             g.es("Save commands disabled", color="purple")
@@ -379,10 +333,8 @@ class OpmlController(object):
         # Make sure we never pass None to the ctor.
         if not c.mFileName:
             c.frame.title = ""
-        initialfile = g.ensure_extension(c.mFileName, ".opml")
         # set local fileName, _not_ c.mFileName
         fileName = g.app.gui.runSaveFileDialog(c,
-            initialfile=initialfile,
             title="Write OPML",
             filetypes=[("OPML files", "*.opml")],
             defaultextension=".opml")
@@ -392,13 +344,13 @@ class OpmlController(object):
             c.opmlCommands.writeFile(fileName)
     #@-others
 #@+node:ekr.20060919172012.2: ** class PutToOPML
-class PutToOPML(object):
-    '''Write c.p's tree as OPML, using the owner's put method.'''
+class PutToOPML:
+    """Write c.p's tree as OPML, using the owner's put method."""
 
     def __init__(self, owner):
         self.c = owner.c
         self.leo_file_encoding = owner.leo_file_encoding
-        self.owner = owner # a leoFileCommands.FileCommand instance.
+        self.owner = owner  # a leoFileCommands.FileCommand instance.
         self.initConfig()
         self.putAll()
 
@@ -407,7 +359,7 @@ class PutToOPML(object):
     #@+others
     #@+node:ekr.20141020112451.18340: *3* initConfig
     def initConfig(self):
-        '''Init all configuration settings.'''
+        """Init all configuration settings."""
         c = self.c
         # These prevent pylint warnings
         self.opml_use_outline_elements = True
@@ -415,7 +367,7 @@ class PutToOPML(object):
         self.opml_write_leo_details = True
         self.opml_write_leo_globals_attributes = True
         self.opml_write_body_text = True
-        self.opml_write_ua_attributes = True
+        self.opml_write_uAs = True
         self.opml_expand_ua_dictionary = True
         self.opml_skip_ua_dictionary_blanks = True
         for ivar in (
@@ -424,18 +376,21 @@ class PutToOPML(object):
             'opml_write_leo_details',
             'opml_write_leo_globals_attributes',
             'opml_write_body_text',
-            'opml_write_ua_attributes',
+            'opml_write_uAs',
             'opml_expand_ua_dictionary',
             'opml_skip_ua_dictionary_blanks',
         ):
-            setattr(self, ivar, c.config.getBool(ivar))
+            val = c.config.getBool(ivar)
+            if val in (True, False):
+                g.trace(ivar, val)
+                setattr(self, ivar, val)
     #@+node:ekr.20141020112451.18337: *3* putAll
     def putAll(self):
-        '''
+        """
         Put the selected outline as OPML.
         All elements and attributes prefixed by 'leo:' are leo-specific.
         All other elements and attributes are specified by the OPML 1 spec.
-        '''
+        """
         self.putXMLLine()
         self.putOPMLProlog()
         self.putOPMLHeader()
@@ -447,21 +402,52 @@ class PutToOPML(object):
         ver = self.c.config.getString('opml-version') or '2.0'
         self.put('<opml version="%s" xmlns:leo="%s">' % (ver, s))
     #@+node:ekr.20060919172012.4: *3* putOPMLHeader
+    # The <head> element may include any of these optional elements:
+    # title, dateCreated, dateModified, ownerName, ownerEmail, expansionState,
+    # vertScrollState, windowTop, windowLeft, windowBottom, windowRight.
+
+    # Each element is a simple text element.
+
+    # dateCreated and dateModified contents conform to the date-time format
+    # specified in RFC 822.
+
+    # expansionState contains a comma-separated list of line numbers that should
+    # be expanded on display.
+
+
+
     def putOPMLHeader(self):
-        '''Put the OPML header, including attributes for globals, prefs and  find settings.'''
-        c = self.c; indent = ' ' * 4
-        if self.opml_write_leo_globals_attributes:
-            self.put('\n<head leo:body_outline_ratio="%s">' % str(c.frame.ratio))
-            width, height, left, top = c.frame.get_window_info()
-            self.put('\n%s<leo:global_window_position' % indent)
-            self.put(' top="%s" left="%s" height="%s" width="%s"/>' % (
-                str(top), str(left), str(height), str(width)))
-            self.put('\n</head>')
-        else:
-            self.put('\n<head/>')
+        """
+        Put the OPML header, including attributes for globals, prefs and  find settings.
+
+        An OPML processor may ignore all the head sub-elements.
+
+        The windowXXX elements define the position and size of the display
+        window.
+
+        If the outline is opened inside another outline then the processor must
+        ignore the window elements.
+        """
+        if not self.opml_write_leo_globals_attributes:
+            self.put('<head />')
+            return
+        c = self.c
+        indent = ' ' * 4
+        width, height, left, top = c.frame.get_window_info()
+        bottom = str(top + height)
+        right = str(left + width)
+        left, top = str(left), str(top)
+        self.put('\n<head>')
+        self.put(f'\n{indent}<windowTop>{top}</windowTop>')
+        self.put(f'\n{indent}<windowLeft>{left}</windowLeft>')
+        self.put(f'\n{indent}<windowBottom>{bottom}</windowBottom>')
+        self.put(f'\n{indent}<windowRight>{right}</windowRight>')
+        self.put('\n</head>')
+
     #@+node:ekr.20060919172012.5: *3* putOPMLNodes
     def putOPMLNodes(self):
-        c = self.c; root = c.rootPosition()
+        c = self.c
+        root = c.rootPosition()
         self.put('\n<body>')
         for p in root.self_and_siblings_iter():
             self.putOPMLNode(p)
@@ -469,41 +455,46 @@ class PutToOPML(object):
     #@+node:ekr.20060919172012.6: *3* putOPMLNode
     def putOPMLNode(self, p):
 
-        indent = ' ' * (4 * p.level()) # Always use 4-space indents.
-        body = p.bodyString() or ''; head = p.headString() or ''
-        attrFormat = ' %s="%s"'
-        self.put('\n%s<outline' % indent)
-        if self.opml_write_leo_details: # Put leo-specific attributes.
+        indent = ''
+        body = p.bodyString() or ''
+        head = p.headString() or ''
+        self.put(f'\n{indent}<outline')
+        head_s = self.attributeEscape(head)
+        self.put(f' text="{head_s}"')
+        if self.opml_write_leo_details:
+            # Put leo-specific attributes.
             for name, val in (
                 ('leo:v', p.v.fileIndex),
                 ('leo:a', self.aAttributes(p)),
-                # ('leo:tnodeList',self.tnodeListAttributes(p)),
             ):
-                if val: self.put(attrFormat % (name, val))
+                if val:
+                    self.put(f' {name}="{val}"')
             data = self.uAAttributes(p)
             if data:
-                # for name,val in data.iteritems():
+                # g.printObj(data, tag=f'uAs for {p.h}')
                 for name in list(data.keys()):
                     val = data.get(name)
-                    self.put(attrFormat % (name, val))
-        self.put(attrFormat % ('text', self.attributeEscape(head)))
+                    self.put(f' {name}="{val}"')
         closed = False
         if body and self.opml_write_body_text:
             if self.opml_use_outline_elements:
-                self.put('>'); closed = True
-                self.put('<leo:body>%s</leo:body>' % xml.sax.saxutils.escape(body))
+                self.put('>')
+                body_s = xml.sax.saxutils.escape(body)
+                self.put(f'\n{indent}<leo:body>{body_s}</leo:body>')
+                closed = True
             else:
-                self.put(attrFormat % ('leo:body', self.attributeEscape(body)))
+                body_s = self.attributeEscape(body)
+                self.put(f' leo:body="{body_s}"')
         if p.hasChildren():
             if not closed:
-                self.put('>'); closed = True
+                self.put('>')
             for p2 in p.children_iter():
                 self.putOPMLNode(p2)
+                closed = True
         if closed:
             self.put('\n%s</outline>' % indent)
-            # self.put('</outline>\n')
         else:
-            self.put('/>')
+            self.put(' />')
     #@+node:ekr.20060919172012.7: *4* attributeEscape
     def attributeEscape(self, s):
         # Unlike xml.sax.saxutils.escape, replace " by &quot; and replace newlines by character reference.
@@ -519,39 +510,23 @@ class PutToOPML(object):
     def aAttributes(self, p):
         c = self.c
         attr = []
-        if p.isExpanded(): attr.append('E')
-        if p.isMarked(): attr.append('M')
-        if c.isCurrentPosition(p): attr.append('V')
+        if p.isExpanded():
+            attr.append('E')
+        if p.isMarked():
+            attr.append('M')
+        if c.isCurrentPosition(p):
+            attr.append('V')
         return ''.join(attr)
-    #@+node:ekr.20060919172012.9: *4* tnodeListAttributes (Not used)
-    # Based on fileCommands.putTnodeList.
-
-    def tnodeListAttributes(self, p):
-        '''Put the tnodeList attribute of p.v'''
-        # Remember: entries in the tnodeList correspond to @+node sentinels, _not_ to tnodes!
-        if not hasattr(p.v, 'tnodeList') or not p.v.tnodeList:
-            return None
-        # Assign fileIndices.
-        for v in p.v.tnodeList:
-            try: # Will fail for None or any pre 4.1 file index.
-                theId, time, n = p.v.fileIndex
-            except Exception:
-                g.trace("assigning gnx for ", p.v)
-                gnx = g.app.nodeIndices.getNewIndex()
-                p.v.setFileIndex(gnx) # Don't convert to string until the actual write.
-        s = ','.join([g.app.nodeIndices.toString(v.fileIndex) for v in p.v.tnodeList])
-        return s
     #@+node:tbrown.20061004094757: *4* uAAttributes
     def uAAttributes(self, p):
         """write unknownAttributes with various levels of expansion"""
         data = {}
-        if self.opml_write_ua_attributes and hasattr(p.v, 'unknownAttributes'):
-            # for uak, uav in p.v.unknownAttributes.iteritems():
+        # g.trace(self.opml_write_uAs, getattr(p.v, 'unknownAttributes', None))
+        if self.opml_write_uAs and hasattr(p.v, 'unknownAttributes'):
             d = p.u
             for uak in list(d.keys()):
                 uav = d.get(uak)
                 if self.opml_expand_ua_dictionary and isinstance(uav, dict):
-                    # for uakc, uavc in uav.iteritems():
                     for uakc in list(uav.keys()):
                         uavc = uav.get(uakc)
                         if str(uavc) != '' or not self.opml_skip_ua_dictionary_blanks:
@@ -564,7 +539,7 @@ class PutToOPML(object):
         self.put('\n</opml>\n')
     #@+node:ekr.20141020112451.18339: *3* putXMLLine
     def putXMLLine(self):
-        '''Put the **properly encoded** <?xml> element.'''
+        """Put the **properly encoded** <?xml> element."""
         self.put('%s"%s"%s\n' % (
             g.app.prolog_prefix_string,
             self.leo_file_encoding,
@@ -572,15 +547,14 @@ class PutToOPML(object):
     #@-others
 #@+node:ekr.20060904134958.164: ** class SaxContentHandler (XMLGenerator)
 class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
-    '''A sax content handler class that reads OPML files.'''
+    """A sax content handler class that reads OPML files."""
     #@+others
     #@+node:ekr.20060904134958.165: *3*  __init__ & helper
     def __init__(self, c, inputFileName):
-        '''Ctor for SaxContentHandler class (OMPL plugin).'''
+        """Ctor for SaxContentHandler class (OMPL plugin)."""
         self.c = c
         self.inputFileName = inputFileName
-        xml.sax.saxutils.XMLGenerator.__init__(self)
-            # Init the base class.
+        super().__init__()
         self.dispatchDict = self.define_dispatch_dict()
         # Semantics.
         self.content = []
@@ -589,7 +563,7 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
         self.level = 0
         self.node = None
         self.nodeStack = []
-        self.ratio = 0.5 # body-outline ratio.
+        self.ratio = 0.5  # body-outline ratio.
         self.rootNode = None
     #@+node:ekr.20060917185525: *4* define_disptatch_dict
     def define_dispatch_dict(self):
@@ -607,10 +581,10 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
     #@+node:ekr.20060904134958.166: *3* helpers
     #@+node:ekr.20060904134958.167: *4* attrsToList
     def attrsToList(self, attrs):
-        '''
+        """
         Convert the attributes to a list of g.Bunches.
         attrs: an Attributes item passed to startElement.
-        '''
+        """
         return [g.Bunch(name=name, val=attrs.getValue(name))
             for name in attrs.getNames()]
     #@+node:ekr.20060904134958.170: *4* error
@@ -636,11 +610,11 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
             print('')
     #@+node:ekr.20060904134958.168: *5* attrsToString
     def attrsToString(self, attrs, sep='\n'):
-        '''Convert the attributes to a string.
+        """Convert the attributes to a string.
 
         attrs: an Attributes item passed to startElement.
 
-        sep: the separator charater between attributes.'''
+        sep: the separator charater between attributes."""
         result = [
             '%s="%s"' % (bunch.name, bunch.val)
             for bunch in self.attrsToList(attrs)
@@ -700,7 +674,7 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
         assert name == name2
     #@+node:ekr.20060919193501: *4* endBodyText
     def endBodyText(self):
-        '''End a <leo:body> element.'''
+        """End a <leo:body> element."""
         if self.content:
             self.node.bodyString = ''.join(self.content)
         self.content = []
@@ -723,7 +697,7 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
                 func(attrs)
     #@+node:ekr.20060919193501.1: *4* startBodyText
     def startBodyText(self, attrs):
-        '''Start a <leo:body> element.'''
+        """Start a <leo:body> element."""
         self.content = []
     #@+node:ekr.20060922072852: *4* startHead
     def startHead(self, attrs):
@@ -734,7 +708,8 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
     def doHeadAttributes(self, attrs):
         ratio = 0.5
         for bunch in self.attrsToList(attrs):
-            name = bunch.name; val = bunch.val
+            name = bunch.name
+            val = bunch.val
             if name == 'leo:body_outline_ratio':
                 try:
                     ratio = float(val)
@@ -751,7 +726,7 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
         if self.rootNode:
             parent = self.node
         else:
-            self.rootNode = parent = NodeClass() # The dummy parent node.
+            self.rootNode = parent = NodeClass()  # The dummy parent node.
             parent.headString = 'dummyNode'
         self.node = NodeClass()
         parent.children.append(self.node)
@@ -762,7 +737,7 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
         node = self.node
         for bunch in self.attrsToList(attrs):
             name, val = bunch.name, bunch.val
-            if name == 'text': # Text is the 'official' opml attribute for headlines.
+            if name == 'text':  # Text is the 'official' opml attribute for headlines.
                 node.headString = val
             elif name in ('_note', 'leo:body'):
                 # Android outliner uses _note.
@@ -779,14 +754,22 @@ class SaxContentHandler(xml.sax.saxutils.XMLGenerator):
     #@+node:ekr.20060922071010.1: *5* doGlobalWindowAttributes
     def doGlobalWindowAttributes(self, attrs):
         c = self.c
-        top = 50; left = 50; height = 500; width = 700 # Reasonable defaults.
+        top = 50
+        left = 50
+        height = 500
+        width = 700  # Reasonable defaults.
         try:
             for bunch in self.attrsToList(attrs):
-                name = bunch.name; val = bunch.val
-                if name == 'top': top = int(val)
-                elif name == 'left': left = int(val)
-                elif name == 'height': height = int(val)
-                elif name == 'width': width = int(val)
+                name = bunch.name
+                val = bunch.val
+                if name == 'top':
+                    top = int(val)
+                elif name == 'left':
+                    left = int(val)
+                elif name == 'height':
+                    height = int(val)
+                elif name == 'width':
+                    width = int(val)
         except ValueError:
             pass
         c.frame.setTopGeometry(width, height, left, top)
