@@ -1,8 +1,8 @@
 #@+leo-ver=5-thin
-#@+node:tbrown.20100226095909.12777: * @file leoscreen.py
+#@+node:tbrown.20100226095909.12777: * @file ../plugins/leoscreen.py
 #@+<< docstring >>
 #@+node:tbrown.20100226095909.12778: ** << docstring >>
-'''Allows interaction with shell apps via screen.
+"""Allows interaction with shell apps via screen.
 
 status: daily-use py2.7 Wed Aug  5 09:30:38 2015
 
@@ -133,7 +133,7 @@ Leo.
 with the same stdout as the target screen, vs. a different stdout. Although
 stdout is ignored, Popen() needs to ensure it's not just inherited.
 
-'''
+"""
 #@-<< docstring >>
 
 # By TNB.
@@ -141,42 +141,39 @@ stdout is ignored, Popen() needs to ensure it's not just inherited.
 
 #@+<< imports >>
 #@+node:tbrown.20100226095909.12779: ** << imports >>
-import leo.core.leoGlobals as g
-
 import subprocess
 import os
 import time
 import tempfile
 import difflib
-
+from leo.core import leoGlobals as g
 try:
-    import leo.plugins.stickynotes as stickynotes
+    from leo.plugins import stickynotes
 except ImportError:
     stickynotes = None
-
 from leo.plugins.attrib_edit import ListDialog
 #@-<< imports >>
 #@+others
 #@+node:tbrown.20100226095909.12781: ** init
 def init():
-    '''Return True if the plugin has loaded successfully.'''
-    g.registerHandler('after-create-leo-frame',onCreate)
+    """Return True if the plugin has loaded successfully."""
+    g.registerHandler('after-create-leo-frame', onCreate)
     g.plugin_signon(__name__)
     return True
 #@+node:tbrown.20100226095909.12782: ** onCreate
-def onCreate (tag,key):
+def onCreate(tag, key):
     """Bind an instance of leoscreen_Controller to c"""
     c = key.get('c')
 
     leoscreen_Controller(c)
 #@+node:tbrown.20100226095909.12783: ** class leoscreen_Controller
-class leoscreen_Controller(object):
+class leoscreen_Controller:
 
-    '''A per-commander class that manages screen interaction.'''
+    """A per-commander class that manages screen interaction."""
 
     #@+others
     #@+node:tbrown.20100226095909.12784: *3* __init__& reloadSettings (leoscreen_Controller)
-    def __init__ (self, c):
+    def __init__(self, c):
         """set up vars., prepare temporary file"""
         self.c = c
         c.leo_screen = self
@@ -195,9 +192,8 @@ class leoscreen_Controller(object):
         self.reloadSettings()
         self._get_output()  # prime output diffing system
         self.popups = []  # store references to popup windows
-        self.stack_frame = 0
-            # used by jump to error commands, 0 = innermost frame
-        
+        self.stack_frame = 0  # For jump to error commands, 0 = innermost frame
+
     def reloadSettings(self):
         c = self.c
         c.registerReloadSettings(self)
@@ -224,7 +220,7 @@ class leoscreen_Controller(object):
 
         cmd.extend([
             '-X', 'eval',
-            'msgwait 0',    # avoid waiting for message display
+            'msgwait 0',  # avoid waiting for message display
         ])
         cmd.extend(cmds)
 
@@ -255,10 +251,10 @@ class leoscreen_Controller(object):
             self.old_output = self.output
         self.output = []  # forget previous output (mostly)
 
-        open(self.tmpfile,'w').write(txt)
+        open(self.tmpfile, 'w').write(txt)
 
         self.screen_cmd([
-            'readbuf "%s"'%self.tmpfile,
+            'readbuf "%s"' % self.tmpfile,
             'paste .',
         ])
     #@+node:tbrown.20100421115534.21602: *3* insert_line
@@ -271,26 +267,20 @@ class leoscreen_Controller(object):
         editor = c.frame.body.wrapper
 
         insert_point = editor.getInsertPoint()
-        editor.insert(insert_point, self.get_line_prefix+line+'\n')
+        editor.insert(insert_point, self.get_line_prefix + line + '\n')
         editor.setInsertPoint(insert_point)
-        c.setChanged(True)
+        c.setChanged()
     #@+node:tbrown.20100528205637.5725: *3* _get_output
     def _get_output(self):
         """grab some output"""
-        self.screen_cmd(['hardcopy -h "%s"'%self.tmpfile])
-
+        self.screen_cmd(['hardcopy -h "%s"' % self.tmpfile])
         # seems new output file isn't visible to the process
         # without this call
         cmd = ['ls', self.tmpfile]
         proc = subprocess.Popen(cmd,
             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         proc.communicate()
-
-        if g.isPython3:
-            f = open(self.tmpfile, encoding='latin-1')
-        else:
-            f = open(self.tmpfile)
-
+        f = open(self.tmpfile, encoding='latin-1')
         self.output = f.read().strip().split('\n')
         self.next_unread_line = self.first_line
     #@+node:tbrown.20100226095909.12788: *3* get_line
@@ -298,45 +288,32 @@ class leoscreen_Controller(object):
         """Get the next line of output from the last command"""
 
         if c and c != self.c:
-            return
-
+            return None
         if not c:
             c = self.c
-
         if not self.output:
             self._get_output()
-
         if not self.output:
             g.es('No output retreived')
-            return
-
+            return ''
         line = self.output[self.next_unread_line]
-
         self.next_unread_line -= 1
-
         return line
     #@+node:tbrown.20100422203442.5579: *3* get_all
     def get_all(self, c=None):
         """Get all output from the last command"""
 
         if c and c != self.c:
-            return
-
+            return None
         if not c:
             c = self.c
-
         self.output = None  # trick get_line into getting output
-        self.get_line()     # updates self.output, ignore returned line
-
+        self.get_line()  # updates self.output, ignore returned line
         sm = difflib.SequenceMatcher(None, self.old_output, self.output)
-        x = sm.find_longest_match(0, len(self.old_output)-1, 0, len(self.output)-1)
-        # print x, len(self.old_output), len(self.output)
-
+        x = sm.find_longest_match(0, len(self.old_output) - 1, 0, len(self.output) - 1)
         ans = self.output[:]
-        del ans[x.b:x.b+x.size]
-
+        del ans[x.b : x.b + x.size]
         return '\n'.join(ans[:self.first_line])
-
     #@+node:tbrown.20100502155649.5599: *3* get_note
     def get_note(self, c=None):
         """Get all output from the last command"""
@@ -353,7 +330,7 @@ class leoscreen_Controller(object):
         n = p.insertAsLastChild()
         n.h = time.strftime(self.time_fmt)
         n.b = dat
-        c.setChanged(True)
+        c.setChanged()
         c.selectPosition(n)
         c.redraw()
     #@+node:tbrown.20100424115939.5735: *3* show
@@ -384,7 +361,7 @@ class leoscreen_Controller(object):
     #@+node:tbrown.20100502155649.5605: *3* show_note
     def show_note(self):
         if stickynotes:
-            stickynotes.stickynote_f({'c':self.c})
+            stickynotes.stickynote_f({'c': self.c})
         else:
             g.es('stickynotes not available')
 
@@ -393,7 +370,7 @@ class leoscreen_Controller(object):
         """get the prefix for insertions from get_line"""
 
         x = g.app.gui.runAskOkCancelStringDialog(
-            self.c,'Prefix for text loading' ,'Prefix for text loading')
+            self.c, 'Prefix for text loading', 'Prefix for text loading')
 
         if x is not None:
             self.get_line_prefix = x
@@ -408,7 +385,7 @@ class leoscreen_Controller(object):
         out, err = proc.communicate()
         out = out.decode('utf-8')
 
-        screens = [[('CURRENT: ' if i == self.use_screen else '')+i, False, i]
+        screens = [[('CURRENT: ' if i == self.use_screen else '') + i, False, i]
                    for i in out.split('\n') if i.startswith('\t')]
 
         ld = ListDialog(None, 'Pick screen', 'Pick screen', screens)
@@ -465,11 +442,11 @@ def cmd_run_text(event):
     b = w.getAllText()
     i = w.getInsertPoint()
     try:
-        j = b[i:].index('\n')+i+1
-        w.setSelectionRange(i,j)
+        j = b[i:].index('\n') + i + 1
+        w.setSelectionRange(i, j)
     except ValueError:  # no more \n in text
-        w.setSelectionRange(i,i)
-    c.leo_screen.run_text(txt,c)
+        w.setSelectionRange(i, i)
+    c.leo_screen.run_text(txt, c)
 #@+node:tbrown.20120905091352.20333: ** cmd_run_all_text (leoscreen_Controller)
 @g.command('leoscreen-run-all-text')
 def cmd_run_all_text(event, move=True):
@@ -478,7 +455,7 @@ def cmd_run_all_text(event, move=True):
     txt = c.p.b
     if txt[-1] != '\n':
         txt += '\n'
-    c.leo_screen.run_text(txt,c)
+    c.leo_screen.run_text(txt, c)
     if move:
         c.selectThreadNext()
     c.redraw()
@@ -549,7 +526,7 @@ def jump_to_error_internal(c):
         match = regex.match(i)
         if match:
             if skipped == c.leo_screen.stack_frame:
-                g.es("Line %s in %s"%(match.group(2), match.group(1)))
+                g.es("Line %s in %s" % (match.group(2), match.group(1)))
                 filename = g.os_path_basename(match.group(1))
                 for p in c.all_unique_positions():
                     if p.h.startswith('@') and p.h.endswith(filename):
@@ -560,7 +537,7 @@ def jump_to_error_internal(c):
                 break
             skipped += 1
     else:
-        g.es("%d error frames found in console content"%skipped)
+        g.es("%d error frames found in console content" % skipped)
 #@+node:tbrown.20150805095656.1: ** cmd_select_screen
 @g.command('leoscreen-select-screen')
 def cmd_select_screen(event):
