@@ -1,6 +1,6 @@
 # coding: utf-8
 #@+leo-ver=5-thin
-#@+node:tbrown.20070117104409: * @file quickMove.py
+#@+node:tbrown.20070117104409: * @file ../plugins/quickMove.py
 #@@first
 #@+<< docstring >>
 #@+node:tbrown.20070117104409.1: ** << docstring >>
@@ -128,41 +128,27 @@ Tags
 
 """
 #@-<< docstring >>
-__version__ = '0.7'
-#@+<< version history >>
-#@+node:tbrown.20070117104409.6: ** << version history >>
-#@+at
-# 0.1 by Terry Brown, 2007-01-12
-# 0.2 EKR:
-# - Revised docstring.
-# - Use positions and position methods rather than vnodes.
-# - Use checkMoveWithParentWithWarning.
-# - Support undo.
-# - Clearer command names.
-# 0.3 EKR: Various small mods suggested by Terry.
-# 0.4 EKR: Added checkMove method.
-# 0.5 EKR: Added c arg to p.visNext & p.visBack.
-# 0.6 TNB: Store vnodes rather than positions, vnodes are more durable
-# 0.7 TNB: Added "clone to" as well as "move to"
-#@-<< version history >>
+
+# By Terry Brown, 2007-01-12
+
 #@+<< imports >>
 #@+node:tbrown.20070117104409.2: ** << imports >>
 from copy import deepcopy
-
-import leo.core.leoGlobals as g
-
+from leo.core import leoGlobals as g
 from leo.plugins.mod_scripting import scriptingController
-
-if g.app.gui.guiName() == "qt":
-    # for the right click context menu, and child items
-    from leo.core.leoQt import QtWidgets # QtCore,QtGui
-    from leo.plugins.attrib_edit import ListDialog
+# for the right click context menu, and child items
+from leo.core.leoQt import QtWidgets
+from leo.core.leoQt import QAction
+from leo.plugins.attrib_edit import DialogCode, ListDialog
+#
+# Fail fast, right after all imports.
+g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
 #@-<< imports >>
 # pylint: disable=cell-var-from-loop
 #@+others
 #@+node:tbrown.20070117104409.3: ** init and onCreate
 def init():
-    '''Return True if the plugin has loaded successfully.'''
+    """Return True if the plugin has loaded successfully."""
     g.registerHandler('after-create-leo-frame', onCreate)
     g.plugin_signon(__name__)
     if '_quickmove' not in g.app.db:
@@ -211,7 +197,7 @@ def keyboard_action(event):
         return
     c.quickMove.keyboard_action()
 #@+node:tbrown.20070117104409.4: ** class quickMove
-class quickMove(object):
+class quickMove:
 
     """quickMove binds to a controller, adds menu entries for
        creating buttons, and creates buttons as needed
@@ -233,7 +219,7 @@ class quickMove(object):
     def add_target(self, p):
         """add the current node as a target for global operations"""
 
-        name, ok = QtWidgets.QInputDialog.getText(None,"Target name","Target name",text=p.h)
+        name, ok = QtWidgets.QInputDialog.getText(None, "Target name", "Target name", text=p.h)
         if not ok:
             return
 
@@ -260,8 +246,8 @@ class quickMove(object):
     def __init__(self, c):
 
         self.table = (
-            ("Make ALL Buttons Here Permanent",None,self.permanentButton),
-            ("Clear ALL Permanent Buttons Here",None,self.clearButton),
+            ("Make ALL Buttons Here Permanent", None, self.permanentButton),
+            ("Clear ALL Permanent Buttons Here", None, self.clearButton),
         )
 
         self.recent_moves = []  # recent move/copy/bookmark to commands for
@@ -274,7 +260,7 @@ class quickMove(object):
         # build callables for imp list
 
         for name, first_last, long, short in quickMove.flavors:
-
+            # pylint: disable=undefined-loop-variable
             self.txts[name] = short
 
             if first_last:
@@ -286,16 +272,16 @@ class quickMove(object):
 
                 def func(self=self, which=which, name=name, event=None):
                     self.addButton(which=which, type_=name)
-                fname = 'func_'+name+'_'+short+'_' +which
+                fname = 'func_' + name + '_' + short + '_' + which
                 if which:
-                    which = " "+which.title()
-                self.imps.append((func, fname, long+" "+short+which+" Button"))
-                cmdname = 'quickmove_'+long+" "+short+which
+                    which = " " + which.title()
+                self.imps.append((func, fname, long + " " + short + which + " Button"))
+                cmdname = 'quickmove_' + long + " " + short + which
                 cmdname = cmdname.strip().lower().replace(' ', '_')
                 # tried to use g.command() but global commands all use the same c
                 # so register only at the c level, not g level
                 # g.command(cmdname)(func)
-                c.k.registerCommand(cmdname, lambda e:func())
+                c.k.registerCommand(cmdname, lambda e: func())
 
         # c.k.registerCommand('quickmove_keyboard_popup', lambda e:self.keyboard_popup())
         # c.k.registerCommand('quickmove_keyboard_action', lambda e:self.keyboard_action())
@@ -315,16 +301,16 @@ class quickMove(object):
                 if 'buttons' in nd.u['quickMove']:  # new dict based storage
                     for rec in nd.u['quickMove']['buttons']:
                         buttons_todo.append(rec.copy())
-                        buttons_todo[-1].update({'v':nd})
+                        buttons_todo[-1].update({'v': nd})
                 else:  # read old list format and convert
                     new_dicts = []
                     for rec in nd.u['quickMove']:
                         if len(rec) != 2:
                             continue  # silently drop even older style permanent button
-                        first,type_ = rec
+                        first, type_ = rec
                         buttons_todo.append({'type': type_, 'first': first})
                         new_dicts.append(buttons_todo[-1].copy())
-                        buttons_todo[-1].update({'v':nd})
+                        buttons_todo[-1].update({'v': nd})
                     nd.u['quickMove'] = {'buttons': new_dicts}
 
         # legacy stuff
@@ -357,15 +343,15 @@ class quickMove(object):
         if g.app.gui.guiName() == "qt":
             g.tree_popup_handlers.remove(self.popup)
     #@+node:ekr.20070117113133.2: *3* addButton (quickMove.py)
-    def addButton (self, which, type_="move", v=None, parent=None):
-        '''Add a button that creates a target for future moves.'''
+    def addButton(self, which, type_="move", v=None, parent=None):
+        """Add a button that creates a target for future moves."""
         c = self.c
         p = c.p
         if v is None:
             v = p.v
         sc = scriptingController(c)
-        mb = quickMoveButton(self,v,which,type_=type_)
-        txt=self.txts[type_]
+        mb = quickMoveButton(self, v, which, type_=type_)
+        txt = self.txts[type_]
 
         if parent:  # find parent button
             for i in self.buttons:
@@ -381,26 +367,27 @@ class quickMove(object):
         text = txt + ":" + header if txt else header
         # createButton truncates text.
 
+
         if parent and g.app.gui.guiName().startswith("qt"):
             pb = parent.button
-            rc = QtWidgets.QAction(text, pb)
+            rc = QAction(text, pb)
             rc.triggered.connect(mb.moveCurrentNodeToTarget)
-            pb.insertAction(pb.actions()[0], rc) # insert at top
+            pb.insertAction(pb.actions()[0], rc)  # insert at top
             b = None
             mb.has_parent = True
             # New code.
             t = c.config.getString('mod-scripting-subtext') or ''
-            t2 = g.u(pb.text())
+            t2 = pb.text()
             if not t.endswith(t):
-                pb.setText(t2+t)
+                pb.setText(t2 + t)
         else:
             b = sc.createIconButton(
                 args=None,
                 text=text,
-                command = mb.moveCurrentNodeToTarget,
-                statusLine = '%s current node to %s child of %s' % (
+                command=mb.moveCurrentNodeToTarget,
+                statusLine='%s current node to %s child of %s' % (
                     type_.title(), which, v.h),
-                kind = "quick-move"
+                kind="quick-move"
             )
             if g.app.gui.guiName() == "qt":
 
@@ -425,14 +412,14 @@ class quickMove(object):
                     (cb_set_parent, 'Set parent'),
                 ]:
                     but = b.button
-                    rc = QtWidgets.QAction(txt, but)
+                    rc = QAction(txt, but)
                     rc.triggered.connect(cb)
                     but.insertAction(but.actions()[-1], rc)
                         # insert rc before Remove Button
 
-        self.buttons.append((mb,b))
+        self.buttons.append((mb, b))
     #@+node:tbrown.20091217114654.5372: *3* permanentButton
-    def permanentButton (self, event=None, v=None, type_=None, first=None):
+    def permanentButton(self, event=None, v=None, type_=None, first=None):
         """make buttons on this node permanent
 
         NOTE: includes buttons deleted"""
@@ -446,7 +433,7 @@ class quickMove(object):
 
         if 'quickMove' not in v.u or 'buttons' not in v.u['quickMove']:
             # possibly deleting old style list
-            v.u['quickMove'] = {'buttons':[]}
+            v.u['quickMove'] = {'buttons': []}
 
         cnt = 0
         for mover, button in qm.buttons:
@@ -456,15 +443,15 @@ class quickMove(object):
             ):  # TNB untested .first -> .which
                 cnt += 1
                 v.u['quickMove']['buttons'].append(
-                    {'first':mover.which, 'type': mover.type_})
+                    {'first': mover.which, 'type': mover.type_})
 
         if cnt:
             g.es('Made buttons permanent')
-            c.setChanged(True)
+            c.setChanged()
         else:
             g.es("Didn't find button")
     #@+node:tbrown.20091217114654.5374: *3* clearButton
-    def clearButton (self, event=None, v=None):
+    def clearButton(self, event=None, v=None):
         """clear permanent buttons specs from uA"""
         c = self.c
         if not v:
@@ -473,7 +460,7 @@ class quickMove(object):
 
         if 'quickMove' in v.u:
             del v.u['quickMove']
-            c.setChanged(True)
+            c.setChanged()
             g.es('Removing buttons - reload to apply')
         else:
             g.es('Quickmove buttons not found')
@@ -485,7 +472,7 @@ class quickMove(object):
         if c != self.c:
             return  # wrong commander
         for cb, name in reversed(self.recent_moves):
-            a = QtWidgets.QAction(name, menu)
+            a = QAction(name, menu)
             a.triggered.connect(lambda checked, cb=cb, name=name: self.do_wrap(cb, name))
             menu.insertAction(menu.actions()[0], a)
         pathmenu = menu.addMenu("Move")
@@ -498,7 +485,7 @@ class quickMove(object):
                 a = sub.addAction(target['name'])
                 def cb(c2=target['unl'], cut=cut):
                     self.to_other(c2, cut=cut)
-                def wrap(checked, cb=cb, name=txt.strip('.')+' '+target['name']):
+                def wrap(checked, cb=cb, name=txt.strip('.') + ' ' + target['name']):
                     self.do_wrap(cb, name)
                 a.triggered.connect(wrap)
             # top of open outlines
@@ -507,7 +494,7 @@ class quickMove(object):
                     g.os_path_basename(c2.fileName()))
                 def cb(c2=c2, cut=cut):
                     self.to_other(c2, cut=cut)
-                def wrap(checked, cb=cb, name=txt.strip('.')+' top of '+g.os_path_basename(c2.fileName())):
+                def wrap(checked, cb=cb, name=txt.strip('.') + ' top of ' + g.os_path_basename(c2.fileName())):
                     self.do_wrap(cb, name)
                 a.triggered.connect(wrap)
         # bookmark to other outline
@@ -517,7 +504,7 @@ class quickMove(object):
             a = sub.addAction(target['name'])
             def cb(c2=target['unl'], cut=cut):
                 self.bookmark_other(c2)
-            def wrap(checked, cb=cb, name="Bookmark to "+target['name']):
+            def wrap(checked, cb=cb, name="Bookmark to " + target['name']):
                 self.do_wrap(cb, name)
             a.triggered.connect(wrap)
         # top of open outlines
@@ -525,14 +512,14 @@ class quickMove(object):
             a = sub.addAction(g.os_path_basename(c2.fileName()))
             def cb(c2=c2):
                 self.bookmark_other(c2)
-            def wrap(checked, cb=cb, name="Bookmark to top of "+g.os_path_basename(c2.fileName())):
+            def wrap(checked, cb=cb, name="Bookmark to top of " + g.os_path_basename(c2.fileName())):
                 self.do_wrap(cb, name)
             a.triggered.connect(wrap)
         # actions within this outline
         need_submenu = 'Move', 'Copy', 'Clone', 'Bookmark', 'Link'
         current_kind = None
         current_submenu = None
-        for name,dummy,command in self.local_imps:
+        for name, dummy, command in self.local_imps:
             kind = name.split()[0]
             if kind in need_submenu:
                 if current_kind != kind:
@@ -569,8 +556,8 @@ class quickMove(object):
                 todo = ['']
             for which in todo:
                 if which:
-                    which = " "+which.title()
-                k = "Set as "+long+" "+short+which+' target'
+                    which = " " + which.title()
+                k = "Set as " + long + " " + short + which + ' target'
                 cmds[k] = {'first': which, 'type': name}
                 kind = long.split()[0]
                 if kind in need_submenu:
@@ -619,8 +606,8 @@ class quickMove(object):
         if not ans:
             g.es("Didn't find button")
             return
-        elif len(ans) != 1:
-            g.es("Note: found multiple %s/first=%s buttons, using first"%(type_,first))
+        if len(ans) != 1:
+            g.es("Note: found multiple %s/first=%s buttons, using first" % (type_, first))
 
         qmb, b = ans[0]
 
@@ -635,8 +622,7 @@ class quickMove(object):
 
         ld = ListDialog(None, 'Pick parent', 'Pick parent', parents)
         ld.exec_()
-
-        if ld.result() == QtWidgets.QDialog.Rejected:
+        if ld.result() == DialogCode.Rejected:
             return
 
         for i in parents:
@@ -651,12 +637,12 @@ class quickMove(object):
             self.permanentButton(v=v, type_=type_, first=first)
 
         for i in v.u['quickMove']['buttons']:
-            if i['type'] == qmb.type_ and i['first'] == qmb.which:   # TNB untested .first -> .which
+            if i['type'] == qmb.type_ and i['first'] == qmb.which:  # TNB untested .first -> .which
                 i['parent'] = parent.gnx
                 break
         else:
-            v.u['quickMove']['buttons'].append({'type':qmb.type_,
-                'first':qmb.which, 'parent':parent.gnx})  # TNB untested .first -> .which
+            v.u['quickMove']['buttons'].append({'type': qmb.type_,
+                'first': qmb.which, 'parent': parent.gnx})  # TNB untested .first -> .which
 
         self.addButton(qmb.which, qmb.type_, v=qmb.target, parent=parent.gnx)  # TNB untested .first -> .which
         self.buttons = [i for i in self.buttons if i[0] is not qmb]
@@ -679,8 +665,8 @@ class quickMove(object):
         if c2 is None:
             return
 
-        p = self.c.vnode2position(p_v)  # in case nd was created in this outline,
-                                        # invalidating p
+        # in case nd was created in this outline, invalidating p
+        p = self.c.vnode2position(p_v)
 
         self.copy_recursively(p, nd)
 
@@ -691,12 +677,12 @@ class quickMove(object):
         if cut:
             self.c.selectPosition(p)
             self.c.deleteOutline()
-            self.c.setChanged(True)
+            self.c.setChanged()
 
         if nxt:
             self.c.selectPosition(self.c.vnode2position(nxt))
 
-        c2.setChanged(True)
+        c2.setChanged()
         c2.redraw()
         self.c.bringToFront(c2=self.c)
         self.c.redraw()  # must come second to keep focus
@@ -720,8 +706,8 @@ class quickMove(object):
         if c2 is None:
             return
 
-        p = self.c.vnode2position(p_v)  # in case nd was created in this outline,
-                                        # invalidating p
+        # in case nd was created in this outline, invalidating p
+        p = self.c.vnode2position(p_v)
 
         in_bookmarks = False
         for i in nd.parents():
@@ -732,7 +718,7 @@ class quickMove(object):
         if in_bookmarks:
             nd.h = p.h
         else:
-            nd.h = "@url %s"%p.h
+            nd.h = "@url %s" % p.h
 
         nd.b = p.get_UNL()
         nd.v.u = dict(p.v.u)
@@ -744,7 +730,7 @@ class quickMove(object):
         c2.redraw()
         self.c.bringToFront(c2=self.c)
         self.c.redraw()  # must come second to keep focus
-    #@+node:tbrown.20120620073922.33740: *3* unl_to_pos
+    #@+node:tbrown.20120620073922.33740: *3* unl_to_pos (quickmove.py)
     def unl_to_pos(self, c2, for_p, bookmark=False):
         """"c2 may be an outline (like c) or an UNL (string)
 
@@ -755,23 +741,20 @@ class quickMove(object):
         copy / move
         """
 
-        if g.isString(c2):
+        if isinstance(c2, str):
             # c2 is an UNL indicating where to insert
             full_path = c2
             path, unl = full_path.split('#', 1)
             c2 = g.openWithFileName(path, old_c=self.c)
             self.c.bringToFront(c2=self.c)
-            found, maxdepth, maxp = g.recursiveUNLFind(unl.split('-->'), c2)
-
-            if found:
-
+            maxp = g.findUNL(unl.split('-->'), c2)
+            if maxp:
                 if not bookmark and (for_p == maxp or for_p.isAncestorOf(maxp)):
                     g.es("Invalid move")
                     return None, None
-
                 nd = maxp.insertAsNthChild(0)
             else:
-                g.es("Could not find '%s'"%full_path)
+                g.es("Could not find '%s'" % full_path)
                 self.c.bringToFront(c2=self.c)
                 return None, None
         else:
@@ -841,7 +824,7 @@ class quickMove(object):
     #@-others
 
 #@+node:tbrown.20070117104409.5: ** class quickMoveButton
-class quickMoveButton(object):
+class quickMoveButton:
 
     """contains target data and function for moving node"""
 
@@ -860,7 +843,7 @@ class quickMoveButton(object):
     #@+node:ekr.20070117121326.1: *3* moveCurrentNodeToTarget
     def moveCurrentNodeToTarget(self, checked=False):
 
-        '''Move the current position to the last child of self.target.'''
+        """Move the current position to the last child of self.target."""
 
         c = self.c
         p = c.p
@@ -882,7 +865,7 @@ class quickMoveButton(object):
                 return
 
             if self.type_ in ('clone', 'move'):  # all others are always valid?
-                if p.v == p2.v or not self.checkMove(p,p2):
+                if p.v == p2.v or not self.checkMove(p, p2):
                     g.error('Invalid move: %s' % (self.targetHeadString))
                     return
             if p2.isAncestorOf(p):  # not for sibling moves
@@ -894,7 +877,7 @@ class quickMoveButton(object):
             if self.type_ != 'jump':
                 p.setDirty()  # before move to dirty current parent
                 p2.setDirty()
-                c.setChanged(changedFlag=True)
+                c.setChanged()
 
             if self.type_ == 'clone':
                 p = p.clone()
@@ -906,13 +889,13 @@ class quickMoveButton(object):
                     p.moveToLastChildOf(p2)
                 elif self.which in ('next sibling', 'prev sibling'):
                     if not p2.parent():
-                        raise Exception("Not implemented for top-level nodes") #FIXME
+                        raise Exception("Not implemented for top-level nodes")  #FIXME
                     if self.which == 'next sibling':
                         p.moveToNthChildOf(p2.parent(), p2._childIndex)
                     elif self.which == 'prev sibling':
-                        p.moveToNthChildOf(p2.parent(), p2._childIndex-1)
+                        p.moveToNthChildOf(p2.parent(), p2._childIndex - 1)
                 else:
-                    raise Exception("Unknown move type "+self.which)
+                    raise Exception("Unknown move type " + self.which)
 
             elif self.type_ == 'bkmk':
                 unl = self.computeUNL(p)  # before tree changes
@@ -925,7 +908,7 @@ class quickMoveButton(object):
                 elif self.which == 'prev sibling':
                     nd = p2.insertBefore()
                 else:
-                    raise Exception("Unknown move type "+self.which)
+                    raise Exception("Unknown move type " + self.which)
                 h = p.anyAtFileNodeName() or p.h
                 while h and h[0] == '@':
                     h = h[1:]
@@ -948,7 +931,7 @@ class quickMoveButton(object):
                     nd = p2.insertBefore()
                     quickMove.copy_recursively(p, nd)
                 else:
-                    raise Exception("Unknown move type "+self.which)
+                    raise Exception("Unknown move type " + self.which)
 
             elif self.type_ in ('linkTo', 'linkFrom'):
                 blc = getattr(c, 'backlinkController', None)
@@ -971,22 +954,19 @@ class quickMoveButton(object):
                 c.selectPosition(nxt)
 
         if needs_undo:
-            c.undoer.afterMoveNode(p,'Quick Move', bunch)
-            c.setChanged(True)
+            c.undoer.afterMoveNode(p, 'Quick Move', bunch)
+            c.setChanged()
 
         c.redraw()
     #@+node:ekr.20070123061606: *3* checkMove
-    def checkMove (self,p,p2):
-
+    def checkMove(self, p, p2):
         c = self.c
-
         for z in p2.parents():
             if z == p:
                 return False
-
         return (
-            c.checkMoveWithParentWithWarning (p,p2,warningFlag=False) and
-            c.checkMoveWithParentWithWarning (p2,p,warningFlag=False)
+            c.checkMoveWithParentWithWarning(p, p2, warningFlag=False) and
+            c.checkMoveWithParentWithWarning(p2, p, warningFlag=False)
         )
     #@+node:tbrown.20100114111020.15726: *3* computeUNL
     def computeUNL(self, p):
@@ -996,7 +976,7 @@ class quickMoveButton(object):
         while p:
             heads.insert(0, p.h)
             p = p.parent()
-        return "#"+"-->".join(heads)
+        return "#" + "-->".join(heads)
     #@-others
 #@-others
 #@@language python
