@@ -1,8 +1,8 @@
 #@+leo-ver=5-thin
-#@+node:ville.20090314215508.4: * @file quicksearch.py
+#@+node:ville.20090314215508.4: * @file ../plugins/quicksearch.py
 #@+<< docstring >>
 #@+node:ville.20090314215508.5: ** << docstring >> (quicksearch.py)
-'''
+"""
 Adds a fast-to-use search widget, like the "Find in files" feature of many editors.
 
 Quicksearch searches node headlines only, *not* body text
@@ -73,25 +73,24 @@ This plugin defines the following commands that can be bound to keys:
 - marked-list:
   List all marked nodes.
 
-'''
+"""
 #@-<< docstring >>
 # Ville M. Vainio <vivainio@gmail.com>.
 #@+<< imports >>
 #@+node:ville.20090314215508.7: ** << imports >>
-import leo.core.leoGlobals as g
-import itertools
 from collections import OrderedDict
-# Fail gracefully if the gui is not qt.
-g.assertUi('qt')
-from leo.core.leoQt import QtCore,QtConst,QtWidgets # isQt5,QtGui,
-
-from leo.core import leoNodes
-    # Uses leoNodes.PosList.
 import fnmatch
+import itertools
 import re
+from leo.core import leoGlobals as g
+from leo.core.leoQt import QtCore, QtConst, QtWidgets
+from leo.core.leoQt import KeyboardModifier
+from leo.core import leoNodes
 from leo.plugins import threadutil
-    # Bug fix. See: https://groups.google.com/forum/?fromgroups=#!topic/leo-editor/PAZloEsuk7g
 from leo.plugins import qt_quicksearch_sub as qt_quicksearch
+#
+# Fail fast, right after all imports.
+g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
 #@-<< imports >>
 #@+others
 #@+node:ekr.20190210123045.1: ** top level
@@ -101,11 +100,11 @@ def dumpfocus():
     g.es("Focus: " + f)
     print("Focus: " + f)
 #@+node:ville.20090314215508.8: *3* init (quicksearch.py)
-def init ():
-    '''Return True if the plugin has loaded successfully.'''
+def init():
+    """Return True if the plugin has loaded successfully."""
     ok = g.app.gui.guiName() == "qt"
     if ok:
-        g.registerHandler('after-create-leo-frame',onCreate)
+        g.registerHandler('after-create-leo-frame', onCreate)
         g.plugin_signon(__name__)
     return ok
 
@@ -113,7 +112,7 @@ def init ():
 def install_qt_quicksearch_tab(c):
 
     wdg = LeoQuickSearchWidget(c, mode="nav")
-    c.frame.log.createTab("Nav", widget = wdg)
+    c.frame.log.createTab("Nav", widget=wdg)
 
     def focus_quicksearch_entry(event):
         c.frame.log.selectTab('Nav')
@@ -161,10 +160,10 @@ def install_qt_quicksearch_tab(c):
     @g.command('go-anywhere')
     def find_popout_f(event):
         c = event['c']
-        w = LeoQuickSearchWidget(c, mode="popout", parent = c.frame.top)
+        w = LeoQuickSearchWidget(c, mode="popout", parent=c.frame.top)
         topgeo = c.frame.top.geometry()
         wid = topgeo.width()
-        w.setGeometry(wid/2,0, wid/2, 500)
+        w.setGeometry(wid / 2, 0, wid / 2, 500)
         w.show()
         w.setFocus(QtConst.OtherFocusReason)
         c._popout = w
@@ -193,14 +192,15 @@ def matchlines(b, miter):
     for m in miter:
         st, en = g.getLine(b, m.start())
         li = b[st:en].strip()
-        res.append((li, (m.start(), m.end() )))
+        res.append((li, (m.start(), m.end())))
     return res
 
 #@+node:ville.20090314215508.9: *3* onCreate (quicksearch.py)
-def onCreate (tag, keys):
+def onCreate(tag, keys):
 
     c = keys.get('c')
-    if not c: return
+    if not c:
+        return
 
     install_qt_quicksearch_tab(c)
 
@@ -228,16 +228,16 @@ def show_unittest_failures(event):
                     c.selectPosition(pos)
                 return focus
 
-            it = nav.scon.addGeneric(pos.h, mkcb(pos,stack))
+            it = nav.scon.addGeneric(pos.h, mkcb(pos, stack))
             it.setToolTip(stack)
 
     c.k.simulateCommand('focus-to-nav')
 #@+node:jlunz.20151027094647.1: ** class OrderedDefaultDict (OrderedDict)
 class OrderedDefaultDict(OrderedDict):
-    '''
+    """
     Credit:  http://stackoverflow.com/questions/4126348/
     how-do-i-rewrite-this-function-to-implement-ordereddict/4127426#4127426
-    '''
+    """
     def __init__(self, *args, **kwargs):
         if not args:
             self.default_factory = None
@@ -246,9 +246,9 @@ class OrderedDefaultDict(OrderedDict):
                 raise TypeError('first argument must be callable or None')
             self.default_factory = args[0]
             args = args[1:]
-        super(OrderedDefaultDict, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-    def __missing__ (self, key):
+    def __missing__(self, key):
         if self.default_factory is None:
             raise KeyError(key)
         self[key] = default = self.default_factory()
@@ -262,30 +262,32 @@ class QuickSearchEventFilter(QtCore.QObject):
 
     #@+others
     #@+node:ekr.20111015194452.15718: *3* quick_ev.ctor
-    def __init__(self,c,w, lineedit):
-        QtCore.QObject.__init__(self)
-            # Init the base class.
+    def __init__(self, c, w, lineedit):
+
+        super().__init__()
         self.c = c
         self.listWidget = w
         self.lineEdit = lineedit
     #@+node:ekr.20111015194452.15719: *3* quick_ev.eventFilter
-    def eventFilter(self,obj,event):
+    def eventFilter(self, obj, event):
 
         eventType = event.type()
         ev = QtCore.QEvent
-        # QLineEdit generates ev.KeyRelease only on Windows,Ubuntu
+        # QLineEdit generates ev.KeyRelease only on Windows, Ubuntu
+        if not hasattr(ev, 'KeyRelease'):  # 2021/07/18.
+            return False
         if eventType == ev.KeyRelease:
             lw = self.listWidget
             k = event.key()
             moved = False
             if k == QtConst.Key_Up:
-                lw.setCurrentRow(lw.currentRow()-1)
+                lw.setCurrentRow(lw.currentRow() - 1)
                 moved = True
             if k == QtConst.Key_Down:
-                lw.setCurrentRow(lw.currentRow()+1)
+                lw.setCurrentRow(lw.currentRow() + 1)
                 moved = True
             if k == QtConst.Key_Return:
-                lw.setCurrentRow(lw.currentRow()+1)
+                lw.setCurrentRow(lw.currentRow() + 1)
                 moved = True
             if moved:
                 self.lineEdit.setFocus(True)
@@ -299,15 +301,15 @@ class LeoQuickSearchWidget(QtWidgets.QWidget):
 
     #@+others
     #@+node:ekr.20111015194452.15695: *3* quick_w.ctor
-    def __init__(self,c, mode = "nav", parent=None):
+    def __init__(self, c, mode="nav", parent=None):
 
-        QtWidgets.QWidget.__init__(self, parent)
+        super().__init__(parent)
         self.ui = qt_quicksearch.Ui_LeoQuickSearchWidget()
         self.ui.setupUi(self)
-        self.frozen = False # True: disable live updates.
+        self.frozen = False  # True: disable live updates.
         w = self.ui.listWidget
         u = self.ui
-        cc = QuickSearchController(c,w,u)
+        cc = QuickSearchController(c, w, u)
         self.scon = cc
         if mode == "popout":
             self.setWindowTitle("Go anywhere")
@@ -316,20 +318,20 @@ class LeoQuickSearchWidget(QtWidgets.QWidget):
         else:
             self.ui.lineEdit.returnPressed.connect(self.returnPressed)
         self.ui.lineEdit.textChanged.connect(self.liveUpdate)
-        self.ev_filter = QuickSearchEventFilter(c,w, self.ui.lineEdit)
+        self.ev_filter = QuickSearchEventFilter(c, w, self.ui.lineEdit)
         self.ui.lineEdit.installEventFilter(self.ev_filter)
         self.c = c
     #@+node:ekr.20111015194452.15696: *3* quick_w.returnPressed
     def returnPressed(self):
         w = self.ui.listWidget
         self.scon.freeze()
-        t = g.u(self.ui.lineEdit.text())
+        t = self.ui.lineEdit.text()
         if not t.strip():
             return
         # Handle Easter eggs.
-        if t == g.u('m'):
+        if t == 'm':
             self.scon.doShowMarked()
-        elif t == g.u('h'):
+        elif t == 'h':
             self.scon.doSearchHistory()
         else:
             self.scon.doSearch(t)
@@ -342,33 +344,33 @@ class LeoQuickSearchWidget(QtWidgets.QWidget):
     #@+node:ville.20121118193144.3622: *3* quick_w.liveUpdate
     def liveUpdate(self):
 
-        t = g.u(self.ui.lineEdit.text())
+        t = self.ui.lineEdit.text()
         if not t.strip():
             if self.scon.frozen:
                 self.scon.freeze(False)
                 self.scon.clear()
             return
-        if len(t) < 3:
-            return
         if self.scon.frozen:
             return
-        if t == g.u('m'):
+        if t == 'm':
             self.scon.doShowMarked()
             return
+        if len(t) < 3:  # #2466.
+            return
         self.scon.worker.set_input(t)
     #@+node:ekr.20190210152123.1: *3* quick_w.selectAndDismiss
     def selectAndDismiss(self):
         self.hide()
     #@-others
-#@+node:ville.20090314215508.12: ** class QuickSearchController (Object)
-class QuickSearchController(object):
+#@+node:ville.20090314215508.12: ** class QuickSearchController
+class QuickSearchController:
 
     #@+others
     #@+node:ekr.20111015194452.15685: *3* __init__
-    def __init__(self,c,listWidget,ui):
+    def __init__(self, c, listWidget, ui):
         self.c = c
-        self.lw = w = listWidget # A QListWidget.
-        self.its = {} # Keys are id(w),values are tuples (p,pos)
+        self.lw = w = listWidget  # A QListWidget.
+        self.its = {}  # Keys are id(w),values are tuples (p,pos)
         self.worker = threadutil.UnitWorker()
         self.widgetUI = ui
         self.fileDirectives = ["@clean", "@file", "@asis", "@edit",
@@ -377,12 +379,13 @@ class QuickSearchController(object):
 
         self.frozen = False
         self._search_patterns = []
+
         def searcher(inp):
             #print("searcher", inp)
             if self.frozen:
-                return
+                return None
             exp = inp.replace(" ", "*")
-            res =  self.bgSearch(exp)
+            res = self.bgSearch(exp)
             return res
 
         def dumper():
@@ -390,18 +393,16 @@ class QuickSearchController(object):
             if self.frozen:
                 return
             out = self.worker.output
-            #print("dumper")
             self.throttler.add(out)
 
         def throttledDump(lst):
             """ dumps the last output """
-            #print "Throttled dump"
             # we do get called with empty list on occasion
             if not lst:
                 return
             if self.frozen:
                 return
-            hm,bm = lst[-1]
+            hm, bm = lst[-1]
             self.clear()
             self.addHeadlineMatches(hm)
             self.addBodyMatches(bm)
@@ -412,24 +413,13 @@ class QuickSearchController(object):
         #self.worker.set_output_f(dumper)
         self.worker.resultReady.connect(dumper)
         self.worker.start()
-        if 1: # Compatible with PyQt5
-            # we want both single-clicks and activations (press enter)
-            w.itemActivated.connect(self.onActivated)
-            w.itemPressed.connect(self.onSelectItem)
-            w.currentItemChanged.connect(self.onSelectItem)
-        # else:
-            # we want both single-clicks and activations (press enter)
-            # w.connect(w,
-                # QtCore.SIGNAL("itemActivated(QListWidgetItem*)"),
-                # self.onActivated)
-            # w.connect(w,
-                # QtCore.SIGNAL("itemPressed(QListWidgetItem*)"),
-                # self.onSelectItem)
-            # w.connect(w,
-                # QtCore.SIGNAL("currentItemChanged(QListWidgetItem*,QListWidgetItem *)"),
-                # self.onSelectItem)
+        # we want both single-clicks and activations (press enter)
+        w.itemActivated.connect(self.onActivated)
+        w.itemPressed.connect(self.onSelectItem)
+        w.currentItemChanged.connect(self.onSelectItem)
+
     #@+node:ville.20121120225024.3636: *3* freeze
-    def freeze(self, val = True):
+    def freeze(self, val=True):
         self.frozen = val
 
     #@+node:vitalije.20170705203722.1: *3* addItem
@@ -444,18 +434,20 @@ class QuickSearchController(object):
             f = it.font()
             f.setBold(True)
             it.setFont(f)
-            if self.addItem(it, (p, None)):return lineMatchHits
+            if self.addItem(it, (p, None)):
+                return lineMatchHits
             ms = matchlines(p.b, p.matchiter)
             for ml, pos in ms:
                 lineMatchHits += 1
-                it = QtWidgets.QListWidgetItem("    "+ml, self.lw)
-                if self.addItem(it, (p,pos)):return lineMatchHits
+                it = QtWidgets.QListWidgetItem("    " + ml, self.lw)
+                if self.addItem(it, (p, pos)):
+                    return lineMatchHits
         return lineMatchHits
     #@+node:jlunz.20151027092130.1: *3* addParentMatches
     def addParentMatches(self, parent_list):
         lineMatchHits = 0
         for parent_key, parent_value in parent_list.items():
-            if g.isString(parent_key):
+            if isinstance(parent_key, str):
                 v = self.c.fileCommands.gnxDict.get(parent_key)
                 h = v.h if v else parent_key
                 it = QtWidgets.QListWidgetItem(h, self.lw)
@@ -464,19 +456,22 @@ class QuickSearchController(object):
             f = it.font()
             f.setItalic(True)
             it.setFont(f)
-            if self.addItem(it, (parent_key, None)): return lineMatchHits
+            if self.addItem(it, (parent_key, None)):
+                return lineMatchHits
             for p in parent_value:
-                it = QtWidgets.QListWidgetItem("    "+p.h, self.lw)
+                it = QtWidgets.QListWidgetItem("    " + p.h, self.lw)
                 f = it.font()
                 f.setBold(True)
                 it.setFont(f)
-                if self.addItem(it, (p, None)):return lineMatchHits
-                if hasattr(p,"matchiter"): #p might be not have body matches
+                if self.addItem(it, (p, None)):
+                    return lineMatchHits
+                if hasattr(p, "matchiter"):  #p might be not have body matches
                     ms = matchlines(p.b, p.matchiter)
                     for ml, pos in ms:
                         lineMatchHits += 1
-                        it = QtWidgets.QListWidgetItem("    "+"    "+ml, self.lw)
-                        if self.addItem(it, (p, pos)):return lineMatchHits
+                        it = QtWidgets.QListWidgetItem("    " + "    " + ml, self.lw)
+                        if self.addItem(it, (p, pos)):
+                            return lineMatchHits
         return lineMatchHits
 
     #@+node:ekr.20111015194452.15690: *3* addGeneric
@@ -493,7 +488,8 @@ class QuickSearchController(object):
             f = it.font()
             f.setBold(True)
             it.setFont(f)
-            if self.addItem(it, (p, None)): return
+            if self.addItem(it, (p, None)):
+                return
     #@+node:ekr.20111015194452.15691: *3* clear
     def clear(self):
 
@@ -546,7 +542,7 @@ class QuickSearchController(object):
         self.clear()
         self.pushSearchHistory(pat)
         if not pat.startswith('r:'):
-            hpat = fnmatch.translate('*'+ pat + '*').replace(r"\Z(?ms)","")
+            hpat = fnmatch.translate('*' + pat + '*').replace(r"\Z(?ms)", "")
             bpat = fnmatch.translate(pat).rstrip('$').replace(r"\Z(?ms)", "")
             # in python 3.6 there is no (?ms) at the end
             # only \Z
@@ -568,7 +564,8 @@ class QuickSearchController(object):
             node = self.c.p
             while not found and not hitBase:
                 h = node.h
-                if h: h=h.split()[0]
+                if h:
+                    h = h.split()[0]
                 if h in self.fileDirectives:
                     found = True
                 else:
@@ -583,7 +580,8 @@ class QuickSearchController(object):
             node = self.c.p
             while not found and not hitBase:
                 h = node.h
-                if h: h=h.split()[0]
+                if h:
+                    h = h.split()[0]
                 if h == "@chapter":
                     found = True
                 else:
@@ -594,7 +592,7 @@ class QuickSearchController(object):
             if hitBase:
                 # If I hit the base then revert to all positions
                 # this is basically the "main" chapter
-                hitBase = False #reset
+                hitBase = False  #reset
                 hNodes = self.c.all_positions()
                 bNodes = self.c.all_positions()
             else:
@@ -609,11 +607,11 @@ class QuickSearchController(object):
             hm = self.find_h(hpat, hNodes, flags)
             bm = self.find_b(bpat, bNodes, flags)
             bm_keys = [match.key() for match in bm]
-            numOfHm = len(hm) #do this before trim to get accurate count
+            numOfHm = len(hm)  #do this before trim to get accurate count
             hm = [match for match in hm if match.key() not in bm_keys]
             if self.widgetUI.showParents.isChecked():
                 parents = OrderedDefaultDict(list)
-                for nodeList in [hm,bm]:
+                for nodeList in [hm, bm]:
                     for node in nodeList:
                         if node.level() == 0:
                             parents["Root"].append(node)
@@ -629,7 +627,7 @@ class QuickSearchController(object):
 
         else:
             if combo == "File":
-                self.lw.insertItem(0, "External file directive not found "+
+                self.lw.insertItem(0, "External file directive not found " +
                                       "during search")
     #@+node:ville.20121118193144.3620: *3* bgSearch
     def bgSearch(self, pat):
@@ -637,10 +635,9 @@ class QuickSearchController(object):
         #self.clear()
 
         if self.frozen:
-            return
-
+            return None
         if not pat.startswith('r:'):
-            hpat = fnmatch.translate('*'+ pat + '*').replace(r"\Z(?ms)","")
+            hpat = fnmatch.translate('*' + pat + '*').replace(r"\Z(?ms)", "")
             # bpat = fnmatch.translate(pat).rstrip('$').replace(r"\Z(?ms)","")
             flags = re.IGNORECASE
         else:
@@ -673,8 +670,8 @@ class QuickSearchController(object):
         for p in nodes:
             m = re.match(pat, p.h)
             if m:
+                # #2012: Don't inject pc.mo.
                 pc = p.copy()
-                pc.mo = m
                 res.append(pc)
         return res
     #@+node:jlunz.20150826091424.1: *3* find_b
@@ -692,10 +689,7 @@ class QuickSearchController(object):
             m = re.finditer(pat, p.b)
             t1, t2 = itertools.tee(m, 2)
             try:
-                if g.isPython3:
-                    t1.__next__()
-                else:
-                    t1.next()
+                t1.__next__()
             except StopIteration:
                 continue
             pc = p.copy()
@@ -713,19 +707,17 @@ class QuickSearchController(object):
                 pl.append(p.copy())
         self.addHeadlineMatches(pl)
     #@+node:ekr.20111015194452.15700: *3* Event handlers
-    #@+node:ekr.20111015194452.15686: *4* onSelectItem
+    #@+node:ekr.20111015194452.15686: *4* onSelectItem (quicksearch.py)
     def onSelectItem(self, it, it_prev=None):
 
         c = self.c
-
         tgt = self.its.get(it and id(it))
-
-        if not tgt: return
-
+        if not tgt:
+            return
         # if Ctrl key is down, delete item and
         # children (based on indent) and return
         modifiers = QtWidgets.QApplication.keyboardModifiers()
-        if modifiers == QtCore.Qt.ControlModifier:
+        if modifiers == KeyboardModifier.ControlModifier:
             row = self.lw.row(it)
             init_indent = len(it.text()) - len(str(it.text()).lstrip())
             self.lw.blockSignals(True)
@@ -733,35 +725,35 @@ class QuickSearchController(object):
                 self.lw.item(row).setHidden(True)
                 row += 1
                 cur = self.lw.item(row)
-                indent = len(cur.text()) - len(str(cur.text()).lstrip())
+                # #1751.
+                if not cur:
+                    break
+                s = cur.text() or ''
+                indent = len(s) - len(str(s).lstrip())
                 if indent <= init_indent:
                     break
             self.lw.setCurrentRow(row)
             self.lw.blockSignals(False)
             return
-
         # generic callable
         if callable(tgt):
             tgt()
         elif len(tgt) == 2:
             p, pos = tgt
-            if hasattr(p,'v'): #p might be "Root"
+            if hasattr(p, 'v'):  #p might be "Root"
                 if not c.positionExists(p):
-                    g.es(
-                        "Node moved or deleted.\nMaybe re-do search.",
-                        color='red'
-                    )
+                    g.es("Node moved or deleted.\nMaybe re-do search.",
+                        color='red')
                     return
                 c.selectPosition(p)
                 if pos is not None:
                     st, en = pos
                     w = c.frame.body.wrapper
-                    w.setSelectionRange(st,en)
+                    w.setSelectionRange(st, en)
                     w.seeInsertPoint()
-
                 self.lw.setFocus()
     #@+node:tbrown.20111018130925.3642: *4* onActivated
-    def onActivated (self,event):
+    def onActivated(self, event):
 
         c = self.c
 
