@@ -1,8 +1,8 @@
 #@+leo-ver=5-thin
-#@+node:ekr.20100103093121.5329: * @file stickynotes.py
+#@+node:ekr.20100103093121.5329: * @file ../plugins/stickynotes.py
 #@+<< docstring >>
 #@+node:vivainio2.20091008133028.5821: ** << docstring >>
-''' Adds simple "sticky notes" feature (popout editors) for Qt gui.
+""" Adds simple "sticky notes" feature (popout editors) for Qt gui.
 
 Adds the following (``Alt-X``) commands:
 
@@ -50,20 +50,14 @@ The decoded data should appear in the popoup window, if not, close the Leo
 file **without saving**.  If you have multiple encoded nodes, repeat this
 process for each one.
 
-'''
+"""
 #@-<< docstring >>
 #@+<< imports >>
-#@+node:vivainio2.20091008133028.5823: ** << imports >>
-import leo.core.leoGlobals as g
-
-# Fail gracefully if the gui is not qt.
-g.assertUi('qt')
-
+#@+node:vivainio2.20091008133028.5823: ** << imports >> (stickynotes.py)
 import os
 import time
-# import webbrowser
-
-encOK = False
+#
+# Third-party imports.
 try:
     from Crypto.Cipher import AES
     from Crypto.Hash import MD5, SHA
@@ -72,46 +66,57 @@ try:
     __ENCKEY = [None]
     encOK = True
 except ImportError:
-    pass
-
-from leo.core.leoQt import isQt5, Qt, QtWidgets
-
-# pylint: disable=no-name-in-module
-if isQt5:
-    from PyQt5.QtCore import QTimer
-    try:
-        from PyQt5.QtCore import QString
-    except ImportError:
-        QString = str
-    from PyQt5.QtGui import QFont,QTextCharFormat # QIcon
-    from PyQt5.QtWidgets import (
-        QAction,QInputDialog,QLineEdit,QMainWindow,QMdiArea,QTextEdit)
-            # QPlainTextEdit,
-else:
-    from PyQt4.QtCore import QTimer
-    try:
-        from PyQt4.QtCore import QString
-    except ImportError:
-        QString = str
-    from PyQt4.QtGui import QAction,QFont,QTextCharFormat,QTextEdit # QIcon
-    from PyQt4.QtGui import QInputDialog,QMainWindow,QMdiArea,QLineEdit
-        # QPlainTextEdit,
+    encOK = False
+#
+# Leo imports.
+from leo.core import leoGlobals as g
+from leo.core.leoQt import Qt, QtCore, QtGui, QtWidgets
+from leo.core.leoQt import QAction, Weight
+#
+# Fail fast, right after all imports.
+g.assertUi('qt')  # May raise g.UiTypeException, caught by the plugins manager.
+#
+# Aliases...
+#
+QFont = QtGui.QFont
+QInputDialog = QtWidgets.QInputDialog
+QLineEdit = QtWidgets.QLineEdit
+QMainWindow = QtWidgets.QMainWindow
+QMdiArea = QtWidgets.QMdiArea
+QTextCharFormat = QtGui.QTextCharFormat
+QTextEdit = QtWidgets.QTextEdit
+QTimer = QtCore.QTimer
 #@-<< imports >>
+
+# Keys are commanders. Values are inner dicts: keys are gnx's; values are widgets.
+outer_dict = {}  # #2471
 #@+others
 #@+node:vivainio2.20091008140054.14555: ** decorate_window
 def decorate_window(c, w):
     w.setStyleSheet(c.styleSheetManager.get_master_widget().styleSheet())
     g.app.gui.attachLeoIcon(w)
-        ### w.setWindowIcon(QIcon(g.app.leoDir + "/Icons/leoapp32.png"))
+        # w.setWindowIcon(QIcon(g.app.leoDir + "/Icons/leoapp32.png"))
     w.resize(600, 300)
 #@+node:vivainio2.20091008133028.5824: ** init
-def init ():
-    '''Return True if the plugin has loaded successfully.'''
+def init():
+    """Return True if the plugin has loaded successfully."""
     ok = g.app.gui.guiName() == 'qt'
     if ok:
         g.plugin_signon(__name__)
-    g.app.stickynotes = {}
+        g.registerHandler('close-frame', onCloseFrame)
     return ok
+#@+node:ekr.20220310040820.1: ** onCloseFrame
+def onCloseFrame(tag, kwargs):
+    """Close all stickynotes in c's outline."""
+    global outer_dict
+    c = kwargs.get('c')
+    if not c:
+        return
+    d = outer_dict.get(c.hash(), {})
+    for gnx in d:
+        w = d.get(gnx)
+        w.close()
+    outer_dict[c.hash()] = {}
 #@+node:ekr.20160403065412.1: ** commands
 #@+node:vivainio2.20091008133028.5825: *3* g.command('stickynote')
 @g.command('stickynote')
@@ -129,7 +134,7 @@ def stickynote_new_f(event):
     p2.h = time.asctime()
     mknote(c, p2)
     # Fix #249: Leo and Stickynote plugin do not request to save.
-    c.setChanged(True)
+    c.setChanged()
     c.redraw(p2)
 #@+node:ville.20091023181249.5266: *3* g.command('stickynoter')
 @g.command('stickynoter')
@@ -138,29 +143,38 @@ def stickynoter_f(event):
     Launch editable 'sticky note' for the node.
     The result is saved as rich text, that is, html...
     """
+    global outer_dict
     c = event['c']
     p = c.p
     v = p.v
+    # #2471: Just show the node if it already exists.
+    d = outer_dict.get(c.hash(), {})
+    nf = d.get(p.gnx)
+    if nf:
+        nf.show()
+        nf.raise_()
+        return
 
-    def focusin():
+    def stickynoter_focusin():
         if v is c.p.v:
-            nf.setHtml(g.u(v.b))
+            nf.setHtml(v.b)
             nf.setWindowTitle(p.h)
             nf.dirty = False
 
-    def focusout():
+    def stickynoter_focusout():
         if nf.dirty:
-            v.b = g.u(nf.toHtml())
+            v.b = nf.toHtml()
             v.setDirty()
             nf.dirty = False
             p = c.p
             if p.v is v:
                 c.selectPosition(c.p)
             # Fix #249: Leo and Stickynote plugin do not request to save
-            c.setChanged(True)
+            c.setChanged()
             c.redraw()
 
-    nf = SimpleRichText(focusin, focusout)  # not LessSimpleRichText
+    # not LessSimpleRichText
+    nf = SimpleRichText(stickynoter_focusin, stickynoter_focusout)
     nf.dirty = False
     decorate_window(c, nf)
     nf.setWindowTitle(p.h)
@@ -168,7 +182,7 @@ def stickynoter_f(event):
     # Fix #249: Leo and Stickynote plugin do not request to save.
     # Do this only on focusout:
         # p.setDirty()
-        # c.setChanged(True)
+        # c.setChanged()
         # c.redraw()
 
     def textchanged_cb():
@@ -176,7 +190,9 @@ def stickynoter_f(event):
 
     nf.textChanged.connect(textchanged_cb)
     nf.show()
-    g.app.stickynotes[p.gnx] = nf
+    d = outer_dict.get(c.hash(), {})
+    d[p.gnx] = nf
+    outer_dict[c.hash()] = d
 #@+node:tbrown.20100120100336.7829: *3* g.command('stickynoteenc')
 if encOK:
     @g.command('stickynoterekey')
@@ -195,7 +211,7 @@ if encOK:
         p = c.p
         v = p.v
 
-        def focusin():
+        def stickynoteenc_focusin():
             if v is c.p.v:
                 decoded = sn_decode(v.b)
                 if decoded is None:
@@ -206,7 +222,7 @@ if encOK:
                 nf.setWindowTitle(p.h)
                 nf.dirty = False
 
-        def focusout():
+        def stickynoteenc_focusout():
             if not nf.dirty:
                 return
             enc = sn_encode(str(nf.toPlainText()))
@@ -214,7 +230,7 @@ if encOK:
                 v.b = enc
                 v.setDirty()
                 # Fix #249: Leo and Stickynote plugin do not request to save
-                c.setChanged(True)
+                c.setChanged()
             nf.dirty = False
             p = c.p
             if p.v is v:
@@ -229,47 +245,59 @@ if encOK:
             secret = sn_encode(unsecret)
             v.b = secret
 
-        ### c = event['c']
-        ### p = c.p
-        decoded = sn_decode(v.b)
-        if decoded is None:
-            return
-        nf = mknote(c,p, focusin=focusin, focusout=focusout)
+        if v.b:
+            decoded = sn_decode(v.b)
+            if decoded is None:
+                return
+        else:
+            decoded = v.b
+        nf = mknote(c, p,
+            focusin=stickynoteenc_focusin,
+            focusout=stickynoteenc_focusout)
         nf.setPlainText(decoded)
         if rekey:
             g.es("Key updated, data decoded with new key shown in window")
 #@+node:tbrown.20100120100336.7830: *3* g.command('stickynoteenckey')
 if encOK:
+
+    def get_AES():
+        if hasattr(AES, 'MODE_EAX'):
+            # pylint: disable=no-member
+            return AES.new(__ENCKEY[0], AES.MODE_EAX)
+                # #1265: When in doubt, use MODE_EAX.
+                # https://pycryptodome.readthedocs.io/en/latest/src/cipher/aes.html
+        # pylint: disable=no-value-for-parameter
+        return AES.new(__ENCKEY[0])
+
     def sn_decode(s):
         try:
-            return AES.new(__ENCKEY[0]).decrypt(base64.b64decode(s)).decode('utf-8').strip()
-        except UnicodeDecodeError:
-            g.es("Decode failed")
+            s1 = base64.b64decode(s)
+            return get_AES().decrypt(s1).decode('utf8').strip()
+        except Exception:
+            g.es("encryption failed")
             __ENCKEY[0] = None
             return None
 
     def sn_encode(s):
-        pad = ' '*(16-len(s)%16)
-        txt = base64.b64encode(AES.new(__ENCKEY[0]).encrypt((s+pad).encode('utf-8')))
-        if g.isPython3:
-            txt = str(txt, 'utf-8')
+        s1 = s.encode('utf8')
+        pad = b' ' * (16 - len(s1) % 16)
+        txta = get_AES().encrypt(s1 + pad)
+        txt = base64.b64encode(txta)
+        txt = str(txt, 'utf-8')
         wrapped = textwrap.wrap(txt, break_long_words=True)
         return '\n'.join(wrapped)
 
     @g.command('stickynoteenckey')
     def sn_getenckey(dummy=None):
-        txt,ok = QInputDialog.getText(None,
+        txt, ok = QInputDialog.getText(None,
             'Enter key',
             'Enter key.\nData lost if key is lost.\nSee docs. for key upgrade notes.',
         )
         if not ok:
             return
-
-        if str(txt).startswith('v0:'):
-            txt = QString(txt[3:])
-        else:
-            txt = g.toUnicode(txt)
-
+        txt = g.toUnicode(txt)
+        if txt.startswith('v0:'):
+            txt = txt[3:]
         # arbitrary kludge to convert string to 256 bits - don't change
         sha = SHA.new()
         md5 = MD5.new()
@@ -296,7 +324,7 @@ class TextEditSearch(QtWidgets.QWidget):
         return f
 
     def __init__(self, *args, **kwargs):
-        QtWidgets.QWidget.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.textedit = QtWidgets.QTextEdit(*args, **kwargs)
         # need to call focusin/out set on parent by FocusingPlaintextEdit / mknote
         self.textedit.focusInEvent = self._call_old_first(
@@ -315,10 +343,10 @@ class TextEditSearch(QtWidgets.QWidget):
         layout = QtWidgets.QVBoxLayout()
         self.setLayout(layout)
         layout.setSpacing(0)
-        layout.setContentsMargins(0,0,0,0)
+        layout.setContentsMargins(0, 0, 0, 0)
         layout.addWidget(self.textedit)
         hlayout = QtWidgets.QHBoxLayout()
-        hlayout.setContentsMargins(0,0,0,0)
+        hlayout.setContentsMargins(0, 0, 0, 0)
         hlayout.addWidget(QtWidgets.QLabel("Find:"))
         hlayout.addWidget(self.searchbox)
         layout.addLayout(hlayout)
@@ -338,16 +366,16 @@ class TextEditSearch(QtWidgets.QWidget):
 #@+node:ville.20091008210853.7616: ** class FocusingPlainTextEdit
 class FocusingPlaintextEdit(TextEditSearch):
 
-    def __init__(self, focusin, focusout, closed = None, parent = None):
+    def __init__(self, focusin, focusout, closed=None, parent=None):
         self.focusin = focusin
         self.focusout = focusout
         self.closed = closed
-        TextEditSearch.__init__(self, parent)
+        super().__init__(parent)
 
-    def focusOutEvent (self, event):
+    def focusOutEvent(self, event):
         self.focusout()
 
-    def focusInEvent (self, event):
+    def focusInEvent(self, event):
         self.focusin()
 
     def closeEvent(self, event):
@@ -361,17 +389,15 @@ class SimpleRichText(QTextEdit):
     # pylint: disable=method-hidden
 
     def __init__(self, focusin, focusout):
-        QTextEdit.__init__(self)
+        super().__init__()
         self.focusin = focusin
         self.focusout = focusout
         self.createActions()
 
-        #self.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
-
-    def focusOutEvent ( self, event ):
+    def focusOutEvent(self, event):
         self.focusout()
 
-    def focusInEvent ( self, event ):
+    def focusInEvent(self, event):
         self.focusin()
 
 
@@ -384,7 +410,6 @@ class SimpleRichText(QTextEdit):
         self.boldAct.setShortcut(self.tr("Ctrl+B"))
         self.boldAct.setStatusTip(self.tr("Make the text bold"))
         self.boldAct.triggered.connect(self.setBold)
-        ### self.connect(self.boldAct, SIGNAL("triggered()"), self.setBold)
         self.addAction(self.boldAct)
 
         boldFont = self.boldAct.font()
@@ -396,21 +421,19 @@ class SimpleRichText(QTextEdit):
         self.italicAct.setShortcut(self.tr("Ctrl+I"))
         self.italicAct.setStatusTip(self.tr("Make the text italic"))
         self.italicAct.triggered.connect(self.setItalic)
-        ### self.connect(self.italicAct, SIGNAL("triggered()"), self.setItalic)
         self.addAction(self.italicAct)
 
     def setBold(self):
         format = QTextCharFormat()
         if self.boldAct.isChecked():
-            weight = QFont.Bold
+            weight = Weight.Bold
         else:
-            weight = QFont.Normal
+            weight = Weight.Normal
         format.setFontWeight(weight)
         self.setFormat(format)
 
     def setItalic(self):
         format = QTextCharFormat()
-        #format.setFontItalic(self.__italic.isChecked())
         format.setFontItalic(self.italicAct.isChecked())
         self.setFormat(format)
 
@@ -428,9 +451,6 @@ class SimpleRichText(QTextEdit):
 
     def italic(self):
         print("italic")
-
-
-
 #@+node:ekr.20160403065519.1: ** Utils
 #@+node:ville.20100707205336.5610: *3* create_subnode
 def create_subnode(c, heading):
@@ -470,40 +490,54 @@ def get_workbook():
     for co in g.app.commanders():
         if co.mFileName.endswith('workbook.leo'):
             return co
+    return None
 #@+node:ville.20100703194946.5587: *3* mknote
-def mknote(c,p, parent=None, focusin=None, focusout=None):
+def mknote(c, p, parent=None, focusin=None, focusout=None):
     """ Launch editable 'sticky note' for the node """
-    # pylint: disable=function-redefined
-    # focusin and focusout are redefined elsewhere.
+    global outer_dict
     v = p.v
-
     if focusin is None:
-        def focusin():
+        def mknote_focusin():
             if v is c.p.v:
                 if v.b.encode('utf-8') != nf.toPlainText():
                     # only when needed to avoid scroll jumping
-                    nf.setPlainText(g.u(v.b))
+                    nf.setPlainText(v.b)
                 nf.setWindowTitle(v.h)
                 nf.dirty = False
+    else:
+        mknote_focusin = focusin
 
     if focusout is None:
-        def focusout():
+        def mknote_focusout():
             if nf.dirty:
                 if v.b.encode('utf-8') != nf.toPlainText():
-                    v.b = g.u(nf.toPlainText())
+                    v.b = nf.toPlainText()
                     v.setDirty()
                     # Fix #249: Leo and Stickynote plugin do not request to save
-                    c.setChanged(True)
+                    c.setChanged()
                 nf.dirty = False
                 p = c.p
                 if p.v is v:
                     c.selectPosition(c.p)
                 c.redraw()
+    else:
+        mknote_focusout = focusout
 
     def closeevent():
         pass
 
-    nf = FocusingPlaintextEdit(focusin, focusout, closeevent, parent = parent)
+    # #2471: Create a new editor only if it doesn't already exist.
+    d = outer_dict.get(c.hash(), {})
+    nf = d.get(p.gnx)
+    if nf:
+        nf.show()
+        nf.raise_()
+        return nf
+    nf = FocusingPlaintextEdit(
+        focusin=mknote_focusin,
+        focusout=mknote_focusout,
+        closed=closeevent,
+        parent=parent)
     decorate_window(c, nf)
     nf.dirty = False
     nf.resize(600, 300)
@@ -518,7 +552,9 @@ def mknote(c,p, parent=None, focusin=None, focusout=None):
 
     nf.textChanged.connect(textchanged_cb)
     nf.show()
-    g.app.stickynotes[p.gnx] = nf
+    d = outer_dict.get(c.hash(), {})
+    d[p.gnx] = nf
+    outer_dict[c.hash()] = d
     return nf
 #@+node:ville.20100703234124.9976: ** Tabula
 #@+node:ville.20100704010850.5589: *3* def tabula_show
@@ -534,7 +570,7 @@ def tabula_show(c):
 @g.command('tabula')
 def tabula_f(event):
     """ Show "tabula" - a MDI window with stickynotes that remember their status """
-    c= event['c']
+    c = event['c']
     t = tabula_show(c)
     p = c.p
     t.add_note(p)
@@ -543,17 +579,17 @@ def tabula_f(event):
 #@+node:ville.20100704010850.5588: *3* @g.command('tabula-show')
 @g.command('tabula-show')
 def tabula_show_f(event):
-    '''Show the`Tabula` sticky note dock window, without adding the current node.'''
-    c= event['c']
+    """Show the`Tabula` sticky note dock window, without adding the current node."""
+    c = event['c']
 
     tabula_show(c)
 #@+node:ville.20100704125228.5592: *3* @g.command('tabula-marked')
 @g.command('tabula-marked')
 def tabula_marked_f(event):
     """ Create tabula from all marked nodes """
-    c= event['c']
+    c = event['c']
 
-    t=tabula_show(c)
+    t = tabula_show(c)
 
     for p in c.all_unique_positions():
         if p.isMarked():
@@ -563,9 +599,9 @@ def tabula_marked_f(event):
 @g.command('tabula-subtree')
 def tabula_subtree_f(event):
     """ Create tabula from all nodes in subtree """
-    c= event['c']
+    c = event['c']
 
-    t=tabula_show(c)
+    t = tabula_show(c)
 
     for p in c.p.self_and_subtree():
         t.add_note(p)
@@ -577,7 +613,7 @@ class Tabula(QMainWindow):
     #@+node:ekr.20101114061906.5445: *4* __init__
     def __init__(self, c):
 
-        QMainWindow.__init__(self)
+        super().__init__()
         mdi = self.mdi = QMdiArea(self)
         self.setCentralWidget(mdi)
         self.create_actions()
@@ -590,7 +626,7 @@ class Tabula(QMainWindow):
 
         QTimer.singleShot(0, delayed_load)
         self.setWindowTitle("Tabula " + os.path.basename(self.c.mFileName))
-        g.registerHandler("end1",self.on_quit)
+        g.registerHandler("end1", self.on_quit)
     #@+node:ekr.20101114061906.5443: *4* add_note
     def add_note(self, p):
 
@@ -599,7 +635,7 @@ class Tabula(QMainWindow):
             n = self.notes[gnx]
             n.show()
             return n
-        n = mknote(self.c, p, parent = self.mdi)
+        n = mknote(self.c, p, parent=self.mdi)
         sw = self.mdi.addSubWindow(n)
         # pylint: disable=maybe-no-member
         # Qt.WA_DeleteOnClose does exist.
@@ -610,11 +646,11 @@ class Tabula(QMainWindow):
         self.notes[gnx] = n
         n.show()
         return n
-    #@+node:ekr.20101114061906.5442: *4* closeEvent
-    def closeEvent(self,event):
+    #@+node:ekr.20101114061906.5442: *4* closeEvent (Tabula)
+    def closeEvent(self, event):
 
         self.save_states()
-        event.accept() # EKR: doesn't help: we don't get the event.
+        event.accept()  # EKR: doesn't help: we don't get the event.
     #@+node:ekr.20101114061906.5444: *4* create_actions (has all toolbar commands!)
     def create_actions(self):
 
@@ -625,11 +661,11 @@ class Tabula(QMainWindow):
         def do_tile():
             self.mdi.setViewMode(QMdiArea.SubWindowView)
             self.mdi.tileSubWindows()
-        
+
         def do_cascade():
             self.mdi.setViewMode(QMdiArea.SubWindowView)
             self.mdi.cascadeSubWindows()
-            
+
         def do_un_tab():
             if self.mdi.viewMode() == QMdiArea.SubWindowView:
                 self.mdi.setViewMode(QMdiArea.TabbedView)
@@ -654,11 +690,10 @@ class Tabula(QMainWindow):
         def do_edit_h():
             p, w = self.get_current_pos()
             new, r = QInputDialog.getText(None,
-                "Edit headline", "", 
+                "Edit headline", "",
                 QLineEdit.Normal, p.h)
             if not r:
                 return
-            new = g.u(new)
             p.h = new
             w.setWindowTitle(new)
 
@@ -701,7 +736,7 @@ class Tabula(QMainWindow):
             n = self.add_note(ncache[gnx])
             n.parent().restoreGeometry(geom)
     #@+node:ekr.20101114061906.5446: *4* on_quit
-    def on_quit(self,tag, kw):
+    def on_quit(self, tag, kw):
 
         # saving when hidden nukes all
 
@@ -720,7 +755,7 @@ class Tabula(QMainWindow):
         active = [gnx for (gnx, n) in self.notes.items() if n.parent() == cur]
         if not active:
             g.trace("no node")
-            return None,None
+            return None, None
         tgt = active[0]
 
         p = next(p for p in self.c.all_unique_positions() if p.gnx == tgt)
@@ -728,11 +763,11 @@ class Tabula(QMainWindow):
 
     #@+node:ekr.20101114061906.5441: *4* save_states
     def save_states(self):
-        
+
         self.update_notes()
 
         # n.parent() because the wrapper QMdiSubWindow holds the geom relative to parent
-        geoms = dict (
+        geoms = dict(
             (gnx, n.parent().saveGeometry())
                 for (gnx, n) in self.notes.items() if n.isVisible())
 
@@ -742,7 +777,7 @@ class Tabula(QMainWindow):
             self.c.db['tabulanotes'] = geoms
     #@+node:ekr.20180822134952.1: *4* update_nodes (new)
     def update_notes(self):
-        
+
         # #940: update self.notes. Ensure note n still exists.
         visible = []
         for (gnx, n) in self.notes.items():
@@ -751,7 +786,7 @@ class Tabula(QMainWindow):
                     visible.append(gnx)
             except RuntimeError:
                 pass
-        self.notes = dict (
+        self.notes = dict(
             (gnx, n) for (gnx, n) in self.notes.items()
                 if gnx in visible
         )
