@@ -1,5 +1,5 @@
 #@+leo-ver=5-thin
-#@+node:mork.20041010095009: * @file xsltWithNodes.py
+#@+node:mork.20041010095009: * @file ../plugins/xsltWithNodes.py
 #@+<< docstring >>
 #@+node:ekr.20050226120104: ** << docstring >>
 """ Adds the Outline:XSLT menu containing XSLT-related commands.
@@ -18,32 +18,23 @@ Requires 4Suite 1.0a3 or better, downloadable from http://4Suite.org.
 
 """
 #@-<< docstring >>
-
-#@@language python
-#@@tabwidth -4
-
 #@+<< imports >>
 #@+node:mork.20041025113509: ** << imports >>
-import leo.core.leoGlobals as g
-
+import io
+import weakref
 from xml.dom import minidom
-
-if g.isPython3:
-    import io
-    StringIO = io.StringIO
-else:
-    import cStringIO
-    StringIO = cStringIO.StringIO
-
+from leo.core import leoGlobals as g
+# Third-part imports
 try:
+    # pylint: disable=import-error
     import Ft
     from Ft.Xml import InputSource
     from Ft.Xml.Xslt.Processor import Processor
 except ImportError:
-    g.cantImport("Ft",__name__)
+    g.cantImport("Ft", __name__)
     Ft = None
-
-import weakref
+# Abbreviation.
+StringIO = io.StringIO
 #@-<< imports >>
 #@+<<parser problems>>
 #@+node:mork.20041024091024: ** <<parser problems>>
@@ -53,50 +44,22 @@ import weakref
 # 1. Having space before the start of the document caused it not to work. I fixed
 #    this by striping the whitespace from the start and end of the data at xslt
 #    time.
-# 
+#
 # 2. having a @ right before a tag causes it to not process.
 #     It appears to be safe to follow this pattern:
 #     @ </end>
 #     but not:
 #     @</end>
-# 
+#
 #     I dont know at this point if its just illegal xml, or its a problem in the parser. ??
 #@-<<parser problems>>
-#@+<<future directions>>
-#@+node:mork.20041025101943: ** <<future directions>>
-#@+at
-# 1. Add more XSLT boilerplate insertions.( done in .3 )
-# 2. Maybe add a well-formedness check. (done in .3, test with minidom )
-#@-<<future directions>>
-__version__ = '0.6'
-#@+<< version history >>
-#@+node:mork.20041025113211: ** << version history >>
-#@@killcolor
-
-#@+at
-# 
-# 0.1: Original code.
-# 
-# 0.2 EKR: Converted to outline.
-# 
-# 0.3: Added more XSLT boilerplate. Added Test with Minidom Discovered parser problem(?).
-# 
-# 0.4 EKR:
-#     - Added init function.
-# 0.5 EKR:
-#     - Remove 'start2' hook & haveseen dict.
-#     - Use keywords.get('c') instead of g.top().
-# 0.6 EKR:
-#     - Removed g.top from example code.
-#@-<< version history >>
-
 #@+others
 #@+node:ekr.20050226120104.1: ** init
 def init():
-    '''Return True if the plugin has loaded successfully.'''
+    """Return True if the plugin has loaded successfully."""
     ok = Ft
     if ok:
-        g.registerHandler(('menu2',"new"),addMenu)
+        g.registerHandler(('menu2', "new"), addMenu)
         g.plugin_signon(__name__)
     return ok
 #@+node:mork.20041025115037: ** xslt elements
@@ -112,7 +75,7 @@ xslt = {
 'comment': "<xsl:comment> </xsl:comment>",
 'copy': "<xsl:copy> </xsl:copy>",
 'copy-of': "<xsl:copy-of select='' />",
-'decimal-format' : "<xsl:decimal-format   />",
+'decimal-format': "<xsl:decimal-format   />",
 'element': "<xsl:element name='' > </xsl:element>",
 'fallback': "<xsl:fallback> </xsl:fallback>",
 'for-each': "<xsl:for-each select='' >  </xsl:for-each>",
@@ -143,199 +106,190 @@ xslt = {
 #@+node:mork.20041010095202: ** setStyleNode
 stylenodes = weakref.WeakKeyDictionary()
 
-def setStyleNode( c ):
-    '''this command sets what the current style node is'''
+def setStyleNode(c):
+    """this command sets what the current style node is"""
     position = c.p
-    stylenodes[ c ] = position
+    stylenodes[c] = position
 
 
 #@+node:mork.20041010095202.1: ** processDocumentNode
-def processDocumentNode( c ):
-    '''this executes the stylesheet node against the current node'''
+def processDocumentNode(c):
+    """this executes the stylesheet node against the current node"""
     try:
-        if not styleNodeSelected( c ): return
+        if not styleNodeSelected(c):
+            return
         proc = Processor()
-        stylenode = stylenodes[ c ]
+        stylenode = stylenodes[c]
         pos = c.p
-        c.selectPosition( stylenode )
-        sIO = getString( c )
-        mdom1 = minidom.parseString( sIO )
-        sIO = str( mdom1.toxml() )
-        hstring = str( stylenode.h )
-        if hstring == "": hstring = "no headline"
-        stylesource = InputSource.DefaultFactory.fromString( sIO, uri = hstring)
-        proc.appendStylesheet( stylesource )
-        c.selectPosition( pos )
+        c.selectPosition(stylenode)
+        sIO = getString(c)
+        mdom1 = minidom.parseString(sIO)
+        sIO = str(mdom1.toxml())
+        hstring = str(stylenode.h)
+        if hstring == "":
+            hstring = "no headline"
+        stylesource = InputSource.DefaultFactory.fromString(sIO, uri=hstring)
+        proc.appendStylesheet(stylesource)
+        c.selectPosition(pos)
         xmlnode = pos.v
-        xIO = getString( c )
-        mdom2 = minidom.parseString( xIO )
-        xIO = str( mdom2.toxml())
-        xhead = str( xmlnode.headString )
-        if xhead == "": xhead = "no headline"
-        xmlsource = InputSource.DefaultFactory.fromString( xIO, uri = xhead )
-        result = proc.run( xmlsource )
-        nhline = "xsl:transform of " + str( xmlnode.headString )
-        p2 = pos.insertAfter() # tnode )
+        xIO = getString(c)
+        mdom2 = minidom.parseString(xIO)
+        xIO = str(mdom2.toxml())
+        xhead = str(xmlnode.headString)
+        if xhead == "":
+            xhead = "no headline"
+        xmlsource = InputSource.DefaultFactory.fromString(xIO, uri=xhead)
+        result = proc.run(xmlsource)
+        nhline = "xsl:transform of " + str(xmlnode.headString)
+        p2 = pos.insertAfter()
         p2.setBodyString(result)
         p2.setHeadString(nhline)
         c.redraw()
 
     except Exception as x:
-        g.es( 'exception ' + str( x ))
+        g.es('exception ' + str(x))
     c.redraw()
 #@+node:mork.20041025121608: ** addXSLTNode
-def addXSLTNode (c):
-    '''creates a node and inserts some xslt boilerplate'''
+def addXSLTNode(c):
+    """creates a node and inserts some xslt boilerplate"""
     pos = c.p
 
-    #body = '''<?xml version="1.0"?>'''
-    # body = '''<?xml version="1.0"?>
-    #<xsl:transform xmlns:xsl="http:///www.w3.org/1999/XSL/Transform" version="1.0">'''
-
     body = '''<?xml version="1.0"?>
 <xsl:transform xmlns:xsl="http:///www.w3.org/1999/XSL/Transform" version="1.0">
 </xsl:transform>'''
 
-    p2 = pos.insertAfter() # tnode)
+    p2 = pos.insertAfter()
     p2.setBodyString(body)
     p2.setHeadString("xslt stylesheet")
     c.redraw()
 #@+node:mork.20041010110121: ** addXSLTElement
-def addXSLTElement( c , element):
-    '''adds some xslt to the text node'''
+def addXSLTElement(c, element):
+    """adds some xslt to the text node"""
     w = c.frame.body.wrapper
-    w.insert( 'insert', element )
-    ### w.event_generate( '<Key>' )
-    ### w.update_idletasks()
-
+    w.insert('insert', element)
 #@+node:mork.20041025113021: ** getString (xsltWithNodes.py)
-def getString (c):
-    '''
+def getString(c):
+    """
     This def turns a node into a string using Leo's file-nosent write logic.
-    '''
+    """
     at = c.atFileCommands
     # EKR: 2017/04/10: needs testing.
     at.toString = True
     at.writeOpenFile(c.p, sentinels=False)
     return cleanString(at.stringOutput)
 #@+node:mork.20041025120706: ** doMinidomTest
-def doMinidomTest( c ):
-    '''
+def doMinidomTest(c):
+    """
     This def performs a simple test on a node.
     Can the data be successfully parsed by minidom or not?
     Results are output to the log.
-    '''
-    s = getString( c )
+    """
+    s = getString(c)
     try:
-        minidom.parseString( s )
+        minidom.parseString(s)
     except Exception as x:
         g.error("Minidom could not parse node because of:\n %s" % x)
         return
     g.blue("Minidom could parse the node")
 #@+node:mork.20041025090303: ** cleanString
-def cleanString( data ):
-    '''This method cleans a string up for the processor.  It currently just removes
-       leading and trailing whitespace'''
+def cleanString(data):
+    """This method cleans a string up for the processor.  It currently just removes
+       leading and trailing whitespace"""
 
     val = data.strip()
     return val
 
 #@+node:mork.20041010125444: ** jumpToStyleNode
-def jumpToStyleNode( c ):
-    '''Simple method that jumps us to the current XSLT node'''
-    if not styleNodeSelected( c ): return
-    pos = stylenodes[ c ]
-    c.selectPosition( pos )
+def jumpToStyleNode(c):
+    """Simple method that jumps us to the current XSLT node"""
+    if not styleNodeSelected(c):
+        return
+    pos = stylenodes[c]
+    c.selectPosition(pos)
     c.redraw()
 
 
 #@+node:mork.20041010125444.1: ** styleNodeSelected
-def styleNodeSelected( c ):
-    '''Determines if a XSLT Style node has not been selected'''
+def styleNodeSelected(c):
+    """Determines if a XSLT Style node has not been selected"""
     if c not in stylenodes:
-        g.es( "No Style Node selected" )
+        g.es("No Style Node selected")
         return False
     return True
 
 
 #@+node:mork.20041010100633: ** addMenu
-def addMenu( tag, keywords ):
+def addMenu(tag, keywords):
 
     # pylint: disable=undefined-variable
     # c *is* defined.
     c = keywords.get('c')
-    if not c: return
+    if not c:
+        return
 
     mc = c.frame.menu
 
     # men = men.getMenu( 'Outline' )
     # xmen = Tk.Menu(men,tearoff = False)
-    xmen = mc.createNewMenu ('XSLT',"Outline")
+    xmen = mc.createNewMenu('XSLT', "Outline")
 
     c.add_command(xmen,
-        label = "Set Stylesheet Node",
-        command = lambda c = c : setStyleNode(c))
+        label="Set Stylesheet Node",
+        command=lambda c=c: setStyleNode(c))
     c.add_command(xmen,
-        label = "Jump To Style Node",
-        command = lambda c = c: jumpToStyleNode(c))
+        label="Jump To Style Node",
+        command=lambda c=c: jumpToStyleNode(c))
     c.add_command(xmen,
-        label = "Process Node with Stylesheet Node",
-        command = lambda c=c : processDocumentNode(c))
+        label="Process Node with Stylesheet Node",
+        command=lambda c=c: processDocumentNode(c))
     xmen.add_separator(xmen)
     c.add_command(xmen,
-        label = "Create Stylesheet Node",
-        command = lambda c = c : addXSLTNode(c))
+        label="Create Stylesheet Node",
+        command=lambda c=c: addXSLTNode(c))
 
     # elmen= Tk.Menu( xmen, tearoff = False )
     # xmen.add_cascade( label = "Insert XSL Element", menu = elmen )
-    m2 = mc.createNewMenu ('Insert XSL Element','XSLT')
+    m2 = mc.createNewMenu('Insert XSL Element', 'XSLT')
 
     xsltkeys = list(xslt.keys())
     xsltkeys.sort()
     for z in xsltkeys:
         # pylint: disable=cell-var-from-loop
         c.add_command(m2,
-            label = z,
-            command = lambda c=c,element=xslt[ z ]: addXSLTElement(c,element))
+            label=z,
+            command=lambda c=c, element=xslt[z]: addXSLTElement(c, element))
 
     # men.add_cascade(menu = xmen, label = "XSLT-Node Commands")
-    m3 = mc.createNewMenu('XSLT-Node Commands','XSLT')
+    m3 = mc.createNewMenu('XSLT-Node Commands', 'XSLT')
     c.add_command(m3,
-        label = 'Test Node with Minidom',
-        command = lambda c=c: doMinidomTest(c))
+        label='Test Node with Minidom',
+        command=lambda c=c: doMinidomTest(c))
 #@+node:mork.20041025100716: ** examples/tests
 #@+at
 # table.leo contains the xml.  xslt is in the other node.
-# 
+#
 # To test this plugin, set the xslt node to be the xslt node.
-# 
+#
 # Process it against the table.leo node.
 #@@c
 
 # pylint: disable=pointless-string-statement
 
-r'''
+r"""
 #@+others
 #@+node:ekr.20140906065955.18786: *3* table.leo
 #@@path /boboo/leo-4.2-final/plugins
 #@+node:ekr.20140906065955.18787: *4* @@nosent table.py
-
-if g.isPython3:
-    import io
-    StringIO = io.StringIO
-else:
-    import cStringIO
-    StringIO = cStringIO.StringIO
-
+import csv
+import io
+StringIO = io.StringIO
+import Pmw
 import Tkinter as Tk
 import tktable as tktab
-import leo.core.leoGlobals as g
-
-import csv
 import weakref
-import Pmw
+from leo.core import leoGlobals as g
 
-class CSVVisualizer(object):
+class CSVVisualizer:
     arrays = []
     #@+others
     #@+node:ekr.20140906065955.18788: *5* init
@@ -553,7 +507,6 @@ class CSVVisualizer(object):
     if 1:
 
         registerHandler( ('start2' , 'open2', "new") , addMenu )
-        __version__ = ".125"
         g.plugin_signon( __name__ )
 
     #@-others
@@ -604,6 +557,9 @@ class CSVVisualizer(object):
 </xsl:template>
 </xsl:transform>
 #@-others
-'''
+"""
 #@-others
+#@@language python
+#@@tabwidth -4
+
 #@-leo
